/* Generated by Cython 0.25.2 */

/* BEGIN: Cython Metadata
{
    "distutils": {}, 
    "module_name": "gauge.deterministic"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_25_2"
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x03030000 || (PY_MAJOR_VERSION == 2 && PY_VERSION_HEX >= 0x02070000)
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_FASTCALL
  #define METH_FASTCALL 0x80
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args,
                                              Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__gauge__deterministic
#define __PYX_HAVE_API__gauge__deterministic
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "gauge/deterministic.pyx",
  "gauge/deterministic.pxd",
  "gauge/core.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_5gauge_4core_Gauge;
struct __pyx_obj_5gauge_4core_Momentum;
struct __pyx_obj_5gauge_13deterministic_Determination;
struct __pyx_obj_5gauge_13deterministic_Line;
struct __pyx_obj_5gauge_13deterministic_Boundary;
struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__;
struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr;
struct __pyx_opt_args_5gauge_13deterministic_13Determination__determine;
struct __pyx_ctuple_double__and_double;
typedef struct __pyx_ctuple_double__and_double __pyx_ctuple_double__and_double;
struct __pyx_ctuple_int__and___dunderpyx_ctuple_double__dunderand_double;
typedef struct __pyx_ctuple_int__and___dunderpyx_ctuple_double__dunderand_double __pyx_ctuple_int__and___dunderpyx_ctuple_double__dunderand_double;
struct __pyx_ctuple_int__and_double;
typedef struct __pyx_ctuple_int__and_double __pyx_ctuple_int__and_double;

/* "gauge/deterministic.pxd":11
 *         bint _in_range
 * 
 *     cdef void _determine(self, double time, double value, bint in_range=?)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_5gauge_13deterministic_13Determination__determine {
  int __pyx_n;
  int in_range;
};

/* "gauge/deterministic.pyx":261
 *         self.extra = extra
 * 
 *     cdef (bint, (double, double)) _intersect(self, Line line):             # <<<<<<<<<<<<<<
 *         """Gets the intersection with the given line.
 * 
 */
struct __pyx_ctuple_double__and_double {
  double f0;
  double f1;
};
struct __pyx_ctuple_int__and___dunderpyx_ctuple_double__dunderand_double {
  int f0;
  __pyx_ctuple_double__and_double f1;
};

/* "gauge/deterministic.pyx":313
 *         return self.value - self.velocity() * self.since
 * 
 *     cdef (bint, double) _get(self, double at):             # <<<<<<<<<<<<<<
 *         if not self.since <= at <= self.until:
 *             return False, 0
 */
struct __pyx_ctuple_int__and_double {
  int f0;
  double f1;
};

/* "gauge/core.pxd":7
 * 
 * 
 * cdef class Gauge:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_obj_5gauge_4core_Gauge {
  PyObject_HEAD
  struct __pyx_vtabstruct_5gauge_4core_Gauge *__pyx_vtab;
  double _base_time;
  double _base_value;
  double _max_value;
  struct __pyx_obj_5gauge_4core_Gauge *_max_gauge;
  double _min_value;
  struct __pyx_obj_5gauge_4core_Gauge *_min_gauge;
  PyObject *momenta;
  struct __pyx_obj_5gauge_13deterministic_Determination *_determination;
  PyObject *_events;
  PyObject *_limited_gauges;
  PyObject *__weakref__;
};


/* "gauge/core.pxd":39
 * 
 * 
 * cdef class Momentum:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_obj_5gauge_4core_Momentum {
  PyObject_HEAD
  double velocity;
  double since;
  double until;
};


/* "gauge/deterministic.pxd":4
 * 
 * 
 * cdef class Determination(list):             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_obj_5gauge_13deterministic_Determination {
  PyListObject __pyx_base;
  struct __pyx_vtabstruct_5gauge_13deterministic_Determination *__pyx_vtab;
  double _in_range_since;
  int _in_range;
};


/* "gauge/deterministic.pyx":233
 * 
 * 
 * cdef class Line:             # <<<<<<<<<<<<<<
 *     """An abstract class to represent lines between 2 times which start from
 *     `value`.  Subclasses should describe where lines end.
 */
struct __pyx_obj_5gauge_13deterministic_Line {
  PyObject_HEAD
  struct __pyx_vtabstruct_5gauge_13deterministic_Line *__pyx_vtab;
  int type;
  double since;
  double until;
  double value;
  double extra;
};


/* "gauge/deterministic.pyx":450
 * 
 * 
 * cdef class Boundary:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_obj_5gauge_13deterministic_Boundary {
  PyObject_HEAD
  struct __pyx_vtabstruct_5gauge_13deterministic_Boundary *__pyx_vtab;
  struct __pyx_obj_5gauge_13deterministic_Line *line;
  PyObject *lines_iter;
  PyObject *cmp;
  PyObject *best;
};


/* "gauge/deterministic.pyx":81
 *         self.append((time, value))
 * 
 *     def __init__(self, Gauge gauge):             # <<<<<<<<<<<<<<
 *         """Determines the transformations from the time when the value set to
 *         the farthest future.
 */
struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ {
  PyObject_HEAD
  struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_bound;
  PyObject *__pyx_v_velocities;
};


/* "gauge/deterministic.pyx":168
 *                                           bound.line.velocity())
 *                 else:
 *                     velocity = sum(v for v in velocities if bound.cmp(v, 0))             # <<<<<<<<<<<<<<
 *                 # is still bound?
 *                 if overlapped and bound.cmp(velocity, bound.line.velocity()):
 */
struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr {
  PyObject_HEAD
  struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ *__pyx_outer_scope;
  PyObject *__pyx_v_v;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
};



/* "gauge/core.pxd":7
 * 
 * 
 * cdef class Gauge:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */

struct __pyx_vtabstruct_5gauge_4core_Gauge {
  __pyx_ctuple_double__and_double (*_predict)(struct __pyx_obj_5gauge_4core_Gauge *, double);
  double (*_clamp)(struct __pyx_obj_5gauge_4core_Gauge *, double, double);
  PyObject *(*momentum_events)(struct __pyx_obj_5gauge_4core_Gauge *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5gauge_4core_Gauge *__pyx_vtabptr_5gauge_4core_Gauge;


/* "gauge/deterministic.pyx":61
 * 
 * 
 * cdef class Determination(list):             # <<<<<<<<<<<<<<
 *     """Determination of a gauge is a list of `(time, value)` pairs.
 * 
 */

struct __pyx_vtabstruct_5gauge_13deterministic_Determination {
  void (*_determine)(struct __pyx_obj_5gauge_13deterministic_Determination *, double, double, struct __pyx_opt_args_5gauge_13deterministic_13Determination__determine *__pyx_optional_args);
};
static struct __pyx_vtabstruct_5gauge_13deterministic_Determination *__pyx_vtabptr_5gauge_13deterministic_Determination;


/* "gauge/deterministic.pyx":233
 * 
 * 
 * cdef class Line:             # <<<<<<<<<<<<<<
 *     """An abstract class to represent lines between 2 times which start from
 *     `value`.  Subclasses should describe where lines end.
 */

struct __pyx_vtabstruct_5gauge_13deterministic_Line {
  __pyx_ctuple_int__and___dunderpyx_ctuple_double__dunderand_double (*_intersect)(struct __pyx_obj_5gauge_13deterministic_Line *, struct __pyx_obj_5gauge_13deterministic_Line *);
  double (*intercept)(struct __pyx_obj_5gauge_13deterministic_Line *, int __pyx_skip_dispatch);
  __pyx_ctuple_int__and_double (*_get)(struct __pyx_obj_5gauge_13deterministic_Line *, double);
  __pyx_ctuple_int__and_double (*_guess)(struct __pyx_obj_5gauge_13deterministic_Line *, double);
  double (*velocity)(struct __pyx_obj_5gauge_13deterministic_Line *, int __pyx_skip_dispatch);
  __pyx_ctuple_int__and_double (*_get_horizon)(struct __pyx_obj_5gauge_13deterministic_Line *, double);
  __pyx_ctuple_int__and_double (*_earlier_horizon)(struct __pyx_obj_5gauge_13deterministic_Line *, double);
  __pyx_ctuple_int__and_double (*_later_horizon)(struct __pyx_obj_5gauge_13deterministic_Line *, double);
  double (*_velocity_horizon)(struct __pyx_obj_5gauge_13deterministic_Line *);
  __pyx_ctuple_int__and_double (*_get_ray)(struct __pyx_obj_5gauge_13deterministic_Line *, double);
  __pyx_ctuple_int__and_double (*_earlier_ray)(struct __pyx_obj_5gauge_13deterministic_Line *, double);
  __pyx_ctuple_int__and_double (*_later_ray)(struct __pyx_obj_5gauge_13deterministic_Line *, double);
  double (*_velocity_ray)(struct __pyx_obj_5gauge_13deterministic_Line *);
  __pyx_ctuple_int__and_double (*_get_segment)(struct __pyx_obj_5gauge_13deterministic_Line *, double);
  __pyx_ctuple_int__and_double (*_earlier_segment)(struct __pyx_obj_5gauge_13deterministic_Line *, double);
  __pyx_ctuple_int__and_double (*_later_segment)(struct __pyx_obj_5gauge_13deterministic_Line *, double);
  double (*_velocity_segment)(struct __pyx_obj_5gauge_13deterministic_Line *);
};
static struct __pyx_vtabstruct_5gauge_13deterministic_Line *__pyx_vtabptr_5gauge_13deterministic_Line;
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__get_horizon(struct __pyx_obj_5gauge_13deterministic_Line *, double);
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__earlier_horizon(struct __pyx_obj_5gauge_13deterministic_Line *, double);
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__later_horizon(struct __pyx_obj_5gauge_13deterministic_Line *, double);
static CYTHON_INLINE double __pyx_f_5gauge_13deterministic_4Line__velocity_horizon(struct __pyx_obj_5gauge_13deterministic_Line *);
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__get_ray(struct __pyx_obj_5gauge_13deterministic_Line *, double);
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__earlier_ray(struct __pyx_obj_5gauge_13deterministic_Line *, double);
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__later_ray(struct __pyx_obj_5gauge_13deterministic_Line *, double);
static CYTHON_INLINE double __pyx_f_5gauge_13deterministic_4Line__velocity_ray(struct __pyx_obj_5gauge_13deterministic_Line *);
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__get_segment(struct __pyx_obj_5gauge_13deterministic_Line *, double);
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__earlier_segment(struct __pyx_obj_5gauge_13deterministic_Line *, double);
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__later_segment(struct __pyx_obj_5gauge_13deterministic_Line *, double);
static CYTHON_INLINE double __pyx_f_5gauge_13deterministic_4Line__velocity_segment(struct __pyx_obj_5gauge_13deterministic_Line *);


/* "gauge/deterministic.pyx":450
 * 
 * 
 * cdef class Boundary:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */

struct __pyx_vtabstruct_5gauge_13deterministic_Boundary {
  PyObject *(*walk)(struct __pyx_obj_5gauge_13deterministic_Boundary *, int __pyx_skip_dispatch);
  int (*cmp_eq)(struct __pyx_obj_5gauge_13deterministic_Boundary *, double, double, int __pyx_skip_dispatch);
  int (*cmp_inv)(struct __pyx_obj_5gauge_13deterministic_Boundary *, double, double, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5gauge_13deterministic_Boundary *__pyx_vtabptr_5gauge_13deterministic_Boundary;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* SliceObject.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(
        PyObject* obj, Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** py_start, PyObject** py_stop, PyObject** py_slice,
        int has_cstart, int has_cstop, int wraparound);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = PyThreadState_GET();
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTest.proto */
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact);

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname);

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#else
    #define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#endif

/* IterNext.proto */
#define __Pyx_PyIter_Next(obj) __Pyx_PyIter_Next2(obj, NULL)
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next2(PyObject *, PyObject *);

/* IncludeStringH.proto */
#include <string.h>

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* ToPyCTupleUtility.proto */
static PyObject* __pyx_convert__to_py___pyx_ctuple_double__and_double(__pyx_ctuple_double__and_double);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* CoroutineBase.proto */
typedef PyObject *(*__pyx_coroutine_body_t)(PyObject *, PyObject *);
typedef struct {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static int __Pyx_Coroutine_clear(PyObject *self);
#if 1 || PY_VERSION_HEX < 0x030300B0
static int __Pyx_PyGen_FetchStopIterationValue(PyObject **pvalue);
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue) PyGen_FetchStopIterationValue(pvalue)
#endif

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
static PyTypeObject *__pyx_GeneratorType = 0;
#define __Pyx_Generator_CheckExact(obj) (Py_TYPE(obj) == __pyx_GeneratorType)
#define __Pyx_Generator_New(body, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static void __pyx_f_5gauge_13deterministic_13Determination__determine(struct __pyx_obj_5gauge_13deterministic_Determination *__pyx_v_self, double __pyx_v_time, double __pyx_v_value, struct __pyx_opt_args_5gauge_13deterministic_13Determination__determine *__pyx_optional_args); /* proto*/
static __pyx_ctuple_int__and___dunderpyx_ctuple_double__dunderand_double __pyx_f_5gauge_13deterministic_4Line__intersect(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_line); /* proto*/
static double __pyx_f_5gauge_13deterministic_4Line_intercept(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__get(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, double __pyx_v_at); /* proto*/
static __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__guess(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, double __pyx_v_at); /* proto*/
static double __pyx_f_5gauge_13deterministic_4Line_velocity(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__get_horizon(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, CYTHON_UNUSED double __pyx_v_at); /* proto*/
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__earlier_horizon(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, CYTHON_UNUSED double __pyx_v_at); /* proto*/
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__later_horizon(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, CYTHON_UNUSED double __pyx_v_at); /* proto*/
static CYTHON_INLINE double __pyx_f_5gauge_13deterministic_4Line__velocity_horizon(CYTHON_UNUSED struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self); /* proto*/
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__get_ray(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, double __pyx_v_at); /* proto*/
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__earlier_ray(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, CYTHON_UNUSED double __pyx_v_at); /* proto*/
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__later_ray(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, CYTHON_UNUSED double __pyx_v_at); /* proto*/
static CYTHON_INLINE double __pyx_f_5gauge_13deterministic_4Line__velocity_ray(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self); /* proto*/
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__get_segment(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, double __pyx_v_at); /* proto*/
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__earlier_segment(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, CYTHON_UNUSED double __pyx_v_at); /* proto*/
static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__later_segment(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, CYTHON_UNUSED double __pyx_v_at); /* proto*/
static CYTHON_INLINE double __pyx_f_5gauge_13deterministic_4Line__velocity_segment(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_5gauge_13deterministic_8Boundary_walk(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_5gauge_13deterministic_8Boundary_cmp_eq(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_5gauge_13deterministic_8Boundary_cmp_inv(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'gauge.constants' */
static int *__pyx_vp_5gauge_9constants_EV_NONE = 0;
#define __pyx_v_5gauge_9constants_EV_NONE (*__pyx_vp_5gauge_9constants_EV_NONE)
static int *__pyx_vp_5gauge_9constants_EV_ADD = 0;
#define __pyx_v_5gauge_9constants_EV_ADD (*__pyx_vp_5gauge_9constants_EV_ADD)
static int *__pyx_vp_5gauge_9constants_EV_REMOVE = 0;
#define __pyx_v_5gauge_9constants_EV_REMOVE (*__pyx_vp_5gauge_9constants_EV_REMOVE)
static int *__pyx_vp_5gauge_9constants_LI_ERROR = 0;
#define __pyx_v_5gauge_9constants_LI_ERROR (*__pyx_vp_5gauge_9constants_LI_ERROR)
static int *__pyx_vp_5gauge_9constants_LI_OK = 0;
#define __pyx_v_5gauge_9constants_LI_OK (*__pyx_vp_5gauge_9constants_LI_OK)
static int *__pyx_vp_5gauge_9constants_LI_ONCE = 0;
#define __pyx_v_5gauge_9constants_LI_ONCE (*__pyx_vp_5gauge_9constants_LI_ONCE)
static int *__pyx_vp_5gauge_9constants_LI_CLAMP = 0;
#define __pyx_v_5gauge_9constants_LI_CLAMP (*__pyx_vp_5gauge_9constants_LI_CLAMP)
static double *__pyx_vp_5gauge_9constants_INF = 0;
#define __pyx_v_5gauge_9constants_INF (*__pyx_vp_5gauge_9constants_INF)
static CYTHON_INLINE PyObject *(*__pyx_f_5gauge_9constants_CLASS_NAME)(PyObject *); /*proto*/

/* Module declarations from 'gauge.core' */
static PyTypeObject *__pyx_ptype_5gauge_4core_Gauge = 0;
static PyTypeObject *__pyx_ptype_5gauge_4core_Momentum = 0;

/* Module declarations from 'gauge.deterministic' */
static PyTypeObject *__pyx_ptype_5gauge_13deterministic_Determination = 0;
static PyTypeObject *__pyx_ptype_5gauge_13deterministic_Line = 0;
static PyTypeObject *__pyx_ptype_5gauge_13deterministic_Boundary = 0;
static PyTypeObject *__pyx_ptype_5gauge_13deterministic___pyx_scope_struct____init__ = 0;
static PyTypeObject *__pyx_ptype_5gauge_13deterministic___pyx_scope_struct_1_genexpr = 0;
static CYTHON_INLINE double __pyx_f_5gauge_13deterministic_SEGMENT_VALUE(double, double, double, double, double); /*proto*/
static CYTHON_INLINE double __pyx_f_5gauge_13deterministic_SEGMENT_VELOCITY(double, double, double, double); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_5gauge_13deterministic_VALUE_LINES(struct __pyx_obj_5gauge_4core_Gauge *, double); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_5gauge_13deterministic_GAUGE_LINES(struct __pyx_obj_5gauge_4core_Gauge *, struct __pyx_obj_5gauge_4core_Gauge *); /*proto*/
static struct __pyx_obj_5gauge_13deterministic_Line *__pyx_f_5gauge_13deterministic_Horizon(double, double, double, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_5gauge_13deterministic_Line *__pyx_f_5gauge_13deterministic_Ray(double, double, double, double, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_5gauge_13deterministic_Line *__pyx_f_5gauge_13deterministic_Segment(double, double, double, double, int __pyx_skip_dispatch); /*proto*/
#define __Pyx_MODULE_NAME "gauge.deterministic"
int __pyx_module_is_main_gauge__deterministic = 0;

/* Implementation of 'gauge.deterministic' */
static PyObject *__pyx_builtin_zip;
static PyObject *__pyx_builtin_sum;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_AssertionError;
static PyObject *__pyx_builtin_min;
static PyObject *__pyx_builtin_max;
static const char __pyx_k_x[] = "x";
static const char __pyx_k_y[] = "y";
static const char __pyx_k_gt[] = "gt";
static const char __pyx_k_lt[] = "lt";
static const char __pyx_k_RAY[] = "RAY";
static const char __pyx_k_Ray[] = "Ray";
static const char __pyx_k_all[] = "__all__";
static const char __pyx_k_cmp[] = "cmp";
static const char __pyx_k_max[] = "max";
static const char __pyx_k_min[] = "min";
static const char __pyx_k_sum[] = "sum";
static const char __pyx_k_zip[] = "zip";
static const char __pyx_k_Line[] = "Line";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_math[] = "math";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_type[] = "type";
static const char __pyx_k_walk[] = "walk";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_extra[] = "extra";
static const char __pyx_k_final[] = "final";
static const char __pyx_k_gauge[] = "gauge";
static const char __pyx_k_isinf[] = "isinf";
static const char __pyx_k_lines[] = "lines";
static const char __pyx_k_since[] = "since";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_until[] = "until";
static const char __pyx_k_value[] = "value";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_cmp_eq[] = "cmp_eq";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_remove[] = "remove";
static const char __pyx_k_HORIZON[] = "HORIZON";
static const char __pyx_k_Horizon[] = "Horizon";
static const char __pyx_k_SEGMENT[] = "SEGMENT";
static const char __pyx_k_Segment[] = "Segment";
static const char __pyx_k_cmp_inv[] = "cmp_inv";
static const char __pyx_k_genexpr[] = "genexpr";
static const char __pyx_k_Boundary[] = "Boundary";
static const char __pyx_k_operator[] = "operator";
static const char __pyx_k_velocity[] = "velocity";
static const char __pyx_k_intercept[] = "intercept";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_HORIZON_0_2f[] = "[HORIZON] {0:.2f}";
static const char __pyx_k_Determination[] = "Determination";
static const char __pyx_k_determination[] = "determination";
static const char __pyx_k_0_line_1_cmp_2[] = "<{0} line={1}, cmp={2}>";
static const char __pyx_k_AssertionError[] = "AssertionError";
static const char __pyx_k_RAY_0_2f_1_2f_s[] = "[RAY] {0:.2f}{1:+.2f}/s";
static const char __pyx_k_0_1_for_2_2f_3_2f[] = "<{0}{1} for {2:.2f}~{3:.2f}>";
static const char __pyx_k_SEGMENT_0_2f_1_2f[] = "[SEGMENT] {0:.2f}~{1:.2f}";
static const char __pyx_k_unexpected_failure[] = "unexpected failure";
static const char __pyx_k_gauge_deterministic[] = "gauge.deterministic";
static const char __pyx_k_init___locals_genexpr[] = "__init__.<locals>.genexpr";
static const char __pyx_k_intersection_not_available[] = "intersection not available";
static const char __pyx_k_gauge_deterministic_Determining[] = "\n   gauge.deterministic\n   ~~~~~~~~~~~~~~~~~~~\n\n   Determining logics for gauge.\n\n   :copyright: (c) 2013-2017 by What! Studio\n   :license: BSD, see LICENSE for more details.\n\n";
static const char __pyx_k_out_of_the_time_range_0_2f_1_2f[] = "out of the time range: {0:.2f}~{1:.2f}";
static PyObject *__pyx_kp_s_0_1_for_2_2f_3_2f;
static PyObject *__pyx_kp_s_0_line_1_cmp_2;
static PyObject *__pyx_n_s_AssertionError;
static PyObject *__pyx_n_s_Boundary;
static PyObject *__pyx_n_s_Determination;
static PyObject *__pyx_n_s_HORIZON;
static PyObject *__pyx_kp_s_HORIZON_0_2f;
static PyObject *__pyx_n_s_Horizon;
static PyObject *__pyx_n_s_Line;
static PyObject *__pyx_n_s_RAY;
static PyObject *__pyx_kp_s_RAY_0_2f_1_2f_s;
static PyObject *__pyx_n_s_Ray;
static PyObject *__pyx_n_s_SEGMENT;
static PyObject *__pyx_kp_s_SEGMENT_0_2f_1_2f;
static PyObject *__pyx_n_s_Segment;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_all;
static PyObject *__pyx_n_s_append;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_s_cmp;
static PyObject *__pyx_n_s_cmp_eq;
static PyObject *__pyx_n_s_cmp_inv;
static PyObject *__pyx_n_s_determination;
static PyObject *__pyx_n_s_extra;
static PyObject *__pyx_n_s_final;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_gauge;
static PyObject *__pyx_n_s_gauge_deterministic;
static PyObject *__pyx_n_s_genexpr;
static PyObject *__pyx_n_s_gt;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_init___locals_genexpr;
static PyObject *__pyx_n_s_intercept;
static PyObject *__pyx_kp_s_intersection_not_available;
static PyObject *__pyx_n_s_isinf;
static PyObject *__pyx_n_s_lines;
static PyObject *__pyx_n_s_lt;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_math;
static PyObject *__pyx_n_s_max;
static PyObject *__pyx_n_s_min;
static PyObject *__pyx_n_s_operator;
static PyObject *__pyx_kp_s_out_of_the_time_range_0_2f_1_2f;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_remove;
static PyObject *__pyx_n_s_send;
static PyObject *__pyx_n_s_since;
static PyObject *__pyx_n_s_sum;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_throw;
static PyObject *__pyx_n_s_type;
static PyObject *__pyx_kp_s_unexpected_failure;
static PyObject *__pyx_n_s_until;
static PyObject *__pyx_n_s_value;
static PyObject *__pyx_n_s_velocity;
static PyObject *__pyx_n_s_walk;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_n_s_y;
static PyObject *__pyx_n_s_zip;
static PyObject *__pyx_pf_5gauge_13deterministic_13Determination_14in_range_since___get__(struct __pyx_obj_5gauge_13deterministic_Determination *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_13Determination_8__init___genexpr(PyObject *__pyx_self); /* proto */
static int __pyx_pf_5gauge_13deterministic_13Determination___init__(struct __pyx_obj_5gauge_13deterministic_Determination *__pyx_v_self, struct __pyx_obj_5gauge_4core_Gauge *__pyx_v_gauge); /* proto */
static int __pyx_pf_5gauge_13deterministic_4Line___cinit__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, int __pyx_v_type, double __pyx_v_since, double __pyx_v_until, double __pyx_v_value, double __pyx_v_extra); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_4Line_2intersect(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_line); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_4Line_4intercept(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_4Line_6get(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, double __pyx_v_at); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_4Line_8guess(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, double __pyx_v_at); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_4Line_10velocity(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_4Line_12__repr__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_4Line_4type___get__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self); /* proto */
static int __pyx_pf_5gauge_13deterministic_4Line_4type_2__set__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_4Line_5since___get__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self); /* proto */
static int __pyx_pf_5gauge_13deterministic_4Line_5since_2__set__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_4Line_5until___get__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self); /* proto */
static int __pyx_pf_5gauge_13deterministic_4Line_5until_2__set__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_4Line_5value___get__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self); /* proto */
static int __pyx_pf_5gauge_13deterministic_4Line_5value_2__set__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_4Line_5extra___get__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self); /* proto */
static int __pyx_pf_5gauge_13deterministic_4Line_5extra_2__set__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_Horizon(CYTHON_UNUSED PyObject *__pyx_self, double __pyx_v_since, double __pyx_v_until, double __pyx_v_value); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_2Ray(CYTHON_UNUSED PyObject *__pyx_self, double __pyx_v_since, double __pyx_v_until, double __pyx_v_value, double __pyx_v_velocity); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_4Segment(CYTHON_UNUSED PyObject *__pyx_self, double __pyx_v_since, double __pyx_v_until, double __pyx_v_value, double __pyx_v_final); /* proto */
static int __pyx_pf_5gauge_13deterministic_8Boundary___init__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, PyObject *__pyx_v_lines, PyObject *__pyx_v_cmp); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_2walk(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_4cmp_eq(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, double __pyx_v_x, double __pyx_v_y); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_6cmp_inv(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, double __pyx_v_x, double __pyx_v_y); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_8__repr__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_4line___get__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self); /* proto */
static int __pyx_pf_5gauge_13deterministic_8Boundary_4line_2__set__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5gauge_13deterministic_8Boundary_4line_4__del__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_10lines_iter___get__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self); /* proto */
static int __pyx_pf_5gauge_13deterministic_8Boundary_10lines_iter_2__set__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5gauge_13deterministic_8Boundary_10lines_iter_4__del__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_3cmp___get__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self); /* proto */
static int __pyx_pf_5gauge_13deterministic_8Boundary_3cmp_2__set__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5gauge_13deterministic_8Boundary_3cmp_4__del__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_4best___get__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self); /* proto */
static int __pyx_pf_5gauge_13deterministic_8Boundary_4best_2__set__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_5gauge_13deterministic_8Boundary_4best_4__del__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self); /* proto */
static PyObject *__pyx_tp_new_5gauge_13deterministic_Determination(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5gauge_13deterministic_Line(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5gauge_13deterministic_Boundary(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5gauge_13deterministic___pyx_scope_struct____init__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_5gauge_13deterministic___pyx_scope_struct_1_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_3;
static PyObject *__pyx_int_neg_1;
static PyObject *__pyx_k__5;
static PyObject *__pyx_slice_;
static PyObject *__pyx_slice__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;

/* "gauge/deterministic.pyx":39
 * 
 * 
 * cdef inline list VALUE_LINES(Gauge gauge, double value):             # <<<<<<<<<<<<<<
 *     return [Line(LN_HORIZON, gauge._base_time, +INF, value)]
 * 
 */

static CYTHON_INLINE PyObject *__pyx_f_5gauge_13deterministic_VALUE_LINES(struct __pyx_obj_5gauge_4core_Gauge *__pyx_v_gauge, double __pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("VALUE_LINES", 0);

  /* "gauge/deterministic.pyx":40
 * 
 * cdef inline list VALUE_LINES(Gauge gauge, double value):
 *     return [Line(LN_HORIZON, gauge._base_time, +INF, value)]             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_gauge->_base_time); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_5gauge_9constants_INF); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_int_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5gauge_13deterministic_Line), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyList_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_3);
  PyList_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
  __pyx_t_3 = 0;
  __pyx_r = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":39
 * 
 * 
 * cdef inline list VALUE_LINES(Gauge gauge, double value):             # <<<<<<<<<<<<<<
 *     return [Line(LN_HORIZON, gauge._base_time, +INF, value)]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("gauge.deterministic.VALUE_LINES", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":43
 * 
 * 
 * cdef inline list GAUGE_LINES(Gauge gauge, Gauge other_gauge):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Line line
 */

static CYTHON_INLINE PyObject *__pyx_f_5gauge_13deterministic_GAUGE_LINES(struct __pyx_obj_5gauge_4core_Gauge *__pyx_v_gauge, struct __pyx_obj_5gauge_4core_Gauge *__pyx_v_other_gauge) {
  struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_line = 0;
  PyObject *__pyx_v_lines = 0;
  struct __pyx_obj_5gauge_13deterministic_Determination *__pyx_v_determination = 0;
  PyObject *__pyx_v_first = NULL;
  PyObject *__pyx_v_last = NULL;
  PyObject *__pyx_v_zipped_determination = NULL;
  PyObject *__pyx_v_time1 = NULL;
  PyObject *__pyx_v_value1 = NULL;
  PyObject *__pyx_v_time2 = NULL;
  PyObject *__pyx_v_value2 = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  PyObject *(*__pyx_t_10)(PyObject *);
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  __Pyx_RefNannySetupContext("GAUGE_LINES", 0);

  /* "gauge/deterministic.pyx":46
 *     cdef:
 *         Line line
 *         list lines = []             # <<<<<<<<<<<<<<
 *         Determination determination = other_gauge.determination
 *     first, last = determination[0], determination[-1]
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_lines = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "gauge/deterministic.pyx":47
 *         Line line
 *         list lines = []
 *         Determination determination = other_gauge.determination             # <<<<<<<<<<<<<<
 *     first, last = determination[0], determination[-1]
 *     if gauge._base_time < first[TIME]:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_other_gauge), __pyx_n_s_determination); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5gauge_13deterministic_Determination))))) __PYX_ERR(0, 47, __pyx_L1_error)
  __pyx_v_determination = ((struct __pyx_obj_5gauge_13deterministic_Determination *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "gauge/deterministic.pyx":48
 *         list lines = []
 *         Determination determination = other_gauge.determination
 *     first, last = determination[0], determination[-1]             # <<<<<<<<<<<<<<
 *     if gauge._base_time < first[TIME]:
 *         line = Line(LN_HORIZON, gauge._base_time, first[TIME], first[VALUE])
 */
  __pyx_t_1 = __Pyx_GetItemInt(((PyObject *)__pyx_v_determination), 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_GetItemInt(((PyObject *)__pyx_v_determination), -1L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_first = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_v_last = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "gauge/deterministic.pyx":49
 *         Determination determination = other_gauge.determination
 *     first, last = determination[0], determination[-1]
 *     if gauge._base_time < first[TIME]:             # <<<<<<<<<<<<<<
 *         line = Line(LN_HORIZON, gauge._base_time, first[TIME], first[VALUE])
 *         lines.append(line)
 */
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_gauge->_base_time); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_first, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_2, __pyx_t_1, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {

    /* "gauge/deterministic.pyx":50
 *     first, last = determination[0], determination[-1]
 *     if gauge._base_time < first[TIME]:
 *         line = Line(LN_HORIZON, gauge._base_time, first[TIME], first[VALUE])             # <<<<<<<<<<<<<<
 *         lines.append(line)
 *     zipped_determination = zip(determination[:-1], determination[1:])
 */
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_gauge->_base_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 50, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_first, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 50, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_GetItemInt(__pyx_v_first, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 50, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PyTuple_New(4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 50, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_int_1);
    __Pyx_GIVEREF(__pyx_int_1);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_int_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_2);
    __pyx_t_3 = 0;
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5gauge_13deterministic_Line), __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 50, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_line = ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "gauge/deterministic.pyx":51
 *     if gauge._base_time < first[TIME]:
 *         line = Line(LN_HORIZON, gauge._base_time, first[TIME], first[VALUE])
 *         lines.append(line)             # <<<<<<<<<<<<<<
 *     zipped_determination = zip(determination[:-1], determination[1:])
 *     for (time1, value1), (time2, value2) in zipped_determination:
 */
    __pyx_t_6 = __Pyx_PyList_Append(__pyx_v_lines, ((PyObject *)__pyx_v_line)); if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 51, __pyx_L1_error)

    /* "gauge/deterministic.pyx":49
 *         Determination determination = other_gauge.determination
 *     first, last = determination[0], determination[-1]
 *     if gauge._base_time < first[TIME]:             # <<<<<<<<<<<<<<
 *         line = Line(LN_HORIZON, gauge._base_time, first[TIME], first[VALUE])
 *         lines.append(line)
 */
  }

  /* "gauge/deterministic.pyx":52
 *         line = Line(LN_HORIZON, gauge._base_time, first[TIME], first[VALUE])
 *         lines.append(line)
 *     zipped_determination = zip(determination[:-1], determination[1:])             # <<<<<<<<<<<<<<
 *     for (time1, value1), (time2, value2) in zipped_determination:
 *         line = Line(LN_SEGMENT, time1, time2, value1, value2)
 */
  __pyx_t_2 = __Pyx_PyObject_GetSlice(((PyObject *)__pyx_v_determination), 0, -1L, NULL, NULL, &__pyx_slice_, 0, 1, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetSlice(((PyObject *)__pyx_v_determination), 1, 0, NULL, NULL, &__pyx_slice__2, 1, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_5);
  __pyx_t_2 = 0;
  __pyx_t_5 = 0;
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_zip, __pyx_t_1, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_zipped_determination = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "gauge/deterministic.pyx":53
 *         lines.append(line)
 *     zipped_determination = zip(determination[:-1], determination[1:])
 *     for (time1, value1), (time2, value2) in zipped_determination:             # <<<<<<<<<<<<<<
 *         line = Line(LN_SEGMENT, time1, time2, value1, value2)
 *         lines.append(line)
 */
  if (likely(PyList_CheckExact(__pyx_v_zipped_determination)) || PyTuple_CheckExact(__pyx_v_zipped_determination)) {
    __pyx_t_5 = __pyx_v_zipped_determination; __Pyx_INCREF(__pyx_t_5); __pyx_t_7 = 0;
    __pyx_t_8 = NULL;
  } else {
    __pyx_t_7 = -1; __pyx_t_5 = PyObject_GetIter(__pyx_v_zipped_determination); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 53, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = Py_TYPE(__pyx_t_5)->tp_iternext; if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 53, __pyx_L1_error)
  }
  for (;;) {
    if (likely(!__pyx_t_8)) {
      if (likely(PyList_CheckExact(__pyx_t_5))) {
        if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_5)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_5, __pyx_t_7); __Pyx_INCREF(__pyx_t_1); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 53, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_5, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 53, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_7 >= PyTuple_GET_SIZE(__pyx_t_5)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_5, __pyx_t_7); __Pyx_INCREF(__pyx_t_1); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 53, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_5, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 53, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_8(__pyx_t_5);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 53, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 53, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      #else
      __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 53, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 53, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 53, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_10 = Py_TYPE(__pyx_t_9)->tp_iternext;
      index = 0; __pyx_t_2 = __pyx_t_10(__pyx_t_9); if (unlikely(!__pyx_t_2)) goto __pyx_L6_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_2);
      index = 1; __pyx_t_3 = __pyx_t_10(__pyx_t_9); if (unlikely(!__pyx_t_3)) goto __pyx_L6_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_9), 2) < 0) __PYX_ERR(0, 53, __pyx_L1_error)
      __pyx_t_10 = NULL;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L7_unpacking_done;
      __pyx_L6_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_10 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 53, __pyx_L1_error)
      __pyx_L7_unpacking_done:;
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 53, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_11 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_9 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_11 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_9);
      __Pyx_INCREF(__pyx_t_11);
      #else
      __pyx_t_9 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 53, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_11 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 53, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_12 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 53, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = Py_TYPE(__pyx_t_12)->tp_iternext;
      index = 0; __pyx_t_9 = __pyx_t_10(__pyx_t_12); if (unlikely(!__pyx_t_9)) goto __pyx_L8_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_9);
      index = 1; __pyx_t_11 = __pyx_t_10(__pyx_t_12); if (unlikely(!__pyx_t_11)) goto __pyx_L8_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_11);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_12), 2) < 0) __PYX_ERR(0, 53, __pyx_L1_error)
      __pyx_t_10 = NULL;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      goto __pyx_L9_unpacking_done;
      __pyx_L8_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_10 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 53, __pyx_L1_error)
      __pyx_L9_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_time1, __pyx_t_9);
    __pyx_t_9 = 0;
    __Pyx_XDECREF_SET(__pyx_v_value1, __pyx_t_11);
    __pyx_t_11 = 0;
    if ((likely(PyTuple_CheckExact(__pyx_t_3))) || (PyList_CheckExact(__pyx_t_3))) {
      PyObject* sequence = __pyx_t_3;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 53, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_11 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1); 
      } else {
        __pyx_t_11 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_9 = PyList_GET_ITEM(sequence, 1); 
      }
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_t_9);
      #else
      __pyx_t_11 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 53, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_9 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 53, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      #endif
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_12 = PyObject_GetIter(__pyx_t_3); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 53, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_10 = Py_TYPE(__pyx_t_12)->tp_iternext;
      index = 0; __pyx_t_11 = __pyx_t_10(__pyx_t_12); if (unlikely(!__pyx_t_11)) goto __pyx_L10_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_11);
      index = 1; __pyx_t_9 = __pyx_t_10(__pyx_t_12); if (unlikely(!__pyx_t_9)) goto __pyx_L10_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_9);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_12), 2) < 0) __PYX_ERR(0, 53, __pyx_L1_error)
      __pyx_t_10 = NULL;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      goto __pyx_L11_unpacking_done;
      __pyx_L10_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_t_10 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 53, __pyx_L1_error)
      __pyx_L11_unpacking_done:;
    }
    __Pyx_XDECREF_SET(__pyx_v_time2, __pyx_t_11);
    __pyx_t_11 = 0;
    __Pyx_XDECREF_SET(__pyx_v_value2, __pyx_t_9);
    __pyx_t_9 = 0;

    /* "gauge/deterministic.pyx":54
 *     zipped_determination = zip(determination[:-1], determination[1:])
 *     for (time1, value1), (time2, value2) in zipped_determination:
 *         line = Line(LN_SEGMENT, time1, time2, value1, value2)             # <<<<<<<<<<<<<<
 *         lines.append(line)
 *     line = Line(LN_HORIZON, last[TIME], +INF, last[VALUE])
 */
    __pyx_t_1 = PyTuple_New(5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 54, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_int_3);
    __Pyx_GIVEREF(__pyx_int_3);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_int_3);
    __Pyx_INCREF(__pyx_v_time1);
    __Pyx_GIVEREF(__pyx_v_time1);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_time1);
    __Pyx_INCREF(__pyx_v_time2);
    __Pyx_GIVEREF(__pyx_v_time2);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_time2);
    __Pyx_INCREF(__pyx_v_value1);
    __Pyx_GIVEREF(__pyx_v_value1);
    PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_v_value1);
    __Pyx_INCREF(__pyx_v_value2);
    __Pyx_GIVEREF(__pyx_v_value2);
    PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_v_value2);
    __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5gauge_13deterministic_Line), __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 54, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF_SET(__pyx_v_line, ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "gauge/deterministic.pyx":55
 *     for (time1, value1), (time2, value2) in zipped_determination:
 *         line = Line(LN_SEGMENT, time1, time2, value1, value2)
 *         lines.append(line)             # <<<<<<<<<<<<<<
 *     line = Line(LN_HORIZON, last[TIME], +INF, last[VALUE])
 *     lines.append(line)
 */
    __pyx_t_6 = __Pyx_PyList_Append(__pyx_v_lines, ((PyObject *)__pyx_v_line)); if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 55, __pyx_L1_error)

    /* "gauge/deterministic.pyx":53
 *         lines.append(line)
 *     zipped_determination = zip(determination[:-1], determination[1:])
 *     for (time1, value1), (time2, value2) in zipped_determination:             # <<<<<<<<<<<<<<
 *         line = Line(LN_SEGMENT, time1, time2, value1, value2)
 *         lines.append(line)
 */
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "gauge/deterministic.pyx":56
 *         line = Line(LN_SEGMENT, time1, time2, value1, value2)
 *         lines.append(line)
 *     line = Line(LN_HORIZON, last[TIME], +INF, last[VALUE])             # <<<<<<<<<<<<<<
 *     lines.append(line)
 *     return lines
 */
  __pyx_t_5 = __Pyx_GetItemInt(__pyx_v_last, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_5gauge_9constants_INF); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_1 = __Pyx_GetItemInt(__pyx_v_last, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_int_1);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 3, __pyx_t_1);
  __pyx_t_5 = 0;
  __pyx_t_3 = 0;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5gauge_13deterministic_Line), __pyx_t_2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XDECREF_SET(__pyx_v_line, ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "gauge/deterministic.pyx":57
 *         lines.append(line)
 *     line = Line(LN_HORIZON, last[TIME], +INF, last[VALUE])
 *     lines.append(line)             # <<<<<<<<<<<<<<
 *     return lines
 * 
 */
  __pyx_t_6 = __Pyx_PyList_Append(__pyx_v_lines, ((PyObject *)__pyx_v_line)); if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 57, __pyx_L1_error)

  /* "gauge/deterministic.pyx":58
 *     line = Line(LN_HORIZON, last[TIME], +INF, last[VALUE])
 *     lines.append(line)
 *     return lines             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_lines);
  __pyx_r = __pyx_v_lines;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":43
 * 
 * 
 * cdef inline list GAUGE_LINES(Gauge gauge, Gauge other_gauge):             # <<<<<<<<<<<<<<
 *     cdef:
 *         Line line
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("gauge.deterministic.GAUGE_LINES", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_line);
  __Pyx_XDECREF(__pyx_v_lines);
  __Pyx_XDECREF((PyObject *)__pyx_v_determination);
  __Pyx_XDECREF(__pyx_v_first);
  __Pyx_XDECREF(__pyx_v_last);
  __Pyx_XDECREF(__pyx_v_zipped_determination);
  __Pyx_XDECREF(__pyx_v_time1);
  __Pyx_XDECREF(__pyx_v_value1);
  __Pyx_XDECREF(__pyx_v_time2);
  __Pyx_XDECREF(__pyx_v_value2);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":69
 * 
 *     @property
 *     def in_range_since(self):             # <<<<<<<<<<<<<<
 *         if self._in_range:
 *             return self._in_range_since
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_13Determination_14in_range_since_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_13Determination_14in_range_since_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_13Determination_14in_range_since___get__(((struct __pyx_obj_5gauge_13deterministic_Determination *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_13Determination_14in_range_since___get__(struct __pyx_obj_5gauge_13deterministic_Determination *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "gauge/deterministic.pyx":70
 *     @property
 *     def in_range_since(self):
 *         if self._in_range:             # <<<<<<<<<<<<<<
 *             return self._in_range_since
 * 
 */
  __pyx_t_1 = (__pyx_v_self->_in_range != 0);
  if (__pyx_t_1) {

    /* "gauge/deterministic.pyx":71
 *     def in_range_since(self):
 *         if self._in_range:
 *             return self._in_range_since             # <<<<<<<<<<<<<<
 * 
 *     cdef void _determine(self, double time, double value, bint in_range=True):
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->_in_range_since); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 71, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "gauge/deterministic.pyx":70
 *     @property
 *     def in_range_since(self):
 *         if self._in_range:             # <<<<<<<<<<<<<<
 *             return self._in_range_since
 * 
 */
  }

  /* "gauge/deterministic.pyx":69
 * 
 *     @property
 *     def in_range_since(self):             # <<<<<<<<<<<<<<
 *         if self._in_range:
 *             return self._in_range_since
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("gauge.deterministic.Determination.in_range_since.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":73
 *             return self._in_range_since
 * 
 *     cdef void _determine(self, double time, double value, bint in_range=True):             # <<<<<<<<<<<<<<
 *         if self and self[-1][TIME] == time:
 *             return
 */

static void __pyx_f_5gauge_13deterministic_13Determination__determine(struct __pyx_obj_5gauge_13deterministic_Determination *__pyx_v_self, double __pyx_v_time, double __pyx_v_value, struct __pyx_opt_args_5gauge_13deterministic_13Determination__determine *__pyx_optional_args) {
  int __pyx_v_in_range = ((int)1);
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("_determine", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_in_range = __pyx_optional_args->in_range;
    }
  }

  /* "gauge/deterministic.pyx":74
 * 
 *     cdef void _determine(self, double time, double value, bint in_range=True):
 *         if self and self[-1][TIME] == time:             # <<<<<<<<<<<<<<
 *             return
 *         if in_range and not self._in_range:
 */
  __pyx_t_2 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_self)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 74, __pyx_L1_error)
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_GetItemInt(((PyObject *)__pyx_v_self), -1L, long, 1, __Pyx_PyInt_From_long, 0, 1, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_GetItemInt(__pyx_t_3, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_4, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 74, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "gauge/deterministic.pyx":75
 *     cdef void _determine(self, double time, double value, bint in_range=True):
 *         if self and self[-1][TIME] == time:
 *             return             # <<<<<<<<<<<<<<
 *         if in_range and not self._in_range:
 *             self._in_range = True
 */
    goto __pyx_L0;

    /* "gauge/deterministic.pyx":74
 * 
 *     cdef void _determine(self, double time, double value, bint in_range=True):
 *         if self and self[-1][TIME] == time:             # <<<<<<<<<<<<<<
 *             return
 *         if in_range and not self._in_range:
 */
  }

  /* "gauge/deterministic.pyx":76
 *         if self and self[-1][TIME] == time:
 *             return
 *         if in_range and not self._in_range:             # <<<<<<<<<<<<<<
 *             self._in_range = True
 *             self._in_range_since = time
 */
  __pyx_t_2 = (__pyx_v_in_range != 0);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L7_bool_binop_done;
  }
  __pyx_t_2 = ((!(__pyx_v_self->_in_range != 0)) != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L7_bool_binop_done:;
  if (__pyx_t_1) {

    /* "gauge/deterministic.pyx":77
 *             return
 *         if in_range and not self._in_range:
 *             self._in_range = True             # <<<<<<<<<<<<<<
 *             self._in_range_since = time
 *         self.append((time, value))
 */
    __pyx_v_self->_in_range = 1;

    /* "gauge/deterministic.pyx":78
 *         if in_range and not self._in_range:
 *             self._in_range = True
 *             self._in_range_since = time             # <<<<<<<<<<<<<<
 *         self.append((time, value))
 * 
 */
    __pyx_v_self->_in_range_since = __pyx_v_time;

    /* "gauge/deterministic.pyx":76
 *         if self and self[-1][TIME] == time:
 *             return
 *         if in_range and not self._in_range:             # <<<<<<<<<<<<<<
 *             self._in_range = True
 *             self._in_range_since = time
 */
  }

  /* "gauge/deterministic.pyx":79
 *             self._in_range = True
 *             self._in_range_since = time
 *         self.append((time, value))             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, Gauge gauge):
 */
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_time); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_5);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_3);
  __pyx_t_5 = 0;
  __pyx_t_3 = 0;
  __pyx_t_6 = __Pyx_PyObject_Append(((PyObject *)__pyx_v_self), __pyx_t_4); if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "gauge/deterministic.pyx":73
 *             return self._in_range_since
 * 
 *     cdef void _determine(self, double time, double value, bint in_range=True):             # <<<<<<<<<<<<<<
 *         if self and self[-1][TIME] == time:
 *             return
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_WriteUnraisable("gauge.deterministic.Determination._determine", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "gauge/deterministic.pyx":81
 *         self.append((time, value))
 * 
 *     def __init__(self, Gauge gauge):             # <<<<<<<<<<<<<<
 *         """Determines the transformations from the time when the value set to
 *         the farthest future.
 */

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_13Determination_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_5gauge_13deterministic_13Determination___init__[] = "Determines the transformations from the time when the value set to\n        the farthest future.\n        ";
#if CYTHON_COMPILING_IN_CPYTHON
struct wrapperbase __pyx_wrapperbase_5gauge_13deterministic_13Determination___init__;
#endif
static int __pyx_pw_5gauge_13deterministic_13Determination_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_5gauge_4core_Gauge *__pyx_v_gauge = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_gauge,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_gauge)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 81, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_gauge = ((struct __pyx_obj_5gauge_4core_Gauge *)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 81, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("gauge.deterministic.Determination.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_gauge), __pyx_ptype_5gauge_4core_Gauge, 1, "gauge", 0))) __PYX_ERR(0, 81, __pyx_L1_error)
  __pyx_r = __pyx_pf_5gauge_13deterministic_13Determination___init__(((struct __pyx_obj_5gauge_13deterministic_Determination *)__pyx_v_self), __pyx_v_gauge);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_5gauge_13deterministic_13Determination_8__init___2generator(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value); /* proto */

/* "gauge/deterministic.pyx":168
 *                                           bound.line.velocity())
 *                 else:
 *                     velocity = sum(v for v in velocities if bound.cmp(v, 0))             # <<<<<<<<<<<<<<
 *                 # is still bound?
 *                 if overlapped and bound.cmp(velocity, bound.line.velocity()):
 */

static PyObject *__pyx_pf_5gauge_13deterministic_13Determination_8__init___genexpr(PyObject *__pyx_self) {
  struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr *)__pyx_tp_new_5gauge_13deterministic___pyx_scope_struct_1_genexpr(__pyx_ptype_5gauge_13deterministic___pyx_scope_struct_1_genexpr, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 168, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_outer_scope = (struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ *) __pyx_self;
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_outer_scope));
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_outer_scope);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_5gauge_13deterministic_13Determination_8__init___2generator, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_init___locals_genexpr, __pyx_n_s_gauge_deterministic); if (unlikely(!gen)) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("gauge.deterministic.Determination.__init__.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_5gauge_13deterministic_13Determination_8__init___2generator(__pyx_CoroutineObject *__pyx_generator, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr *__pyx_cur_scope = ((struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("None", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L7_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 168, __pyx_L1_error)
  if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_velocities)) { __Pyx_RaiseClosureNameError("velocities"); __PYX_ERR(0, 168, __pyx_L1_error) }
  if (unlikely(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_velocities == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 168, __pyx_L1_error)
  }
  __pyx_t_1 = __pyx_cur_scope->__pyx_outer_scope->__pyx_v_velocities; __Pyx_INCREF(__pyx_t_1); __pyx_t_2 = 0;
  for (;;) {
    if (__pyx_t_2 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_2); __Pyx_INCREF(__pyx_t_3); __pyx_t_2++; if (unlikely(0 < 0)) __PYX_ERR(0, 168, __pyx_L1_error)
    #else
    __pyx_t_3 = PySequence_ITEM(__pyx_t_1, __pyx_t_2); __pyx_t_2++; if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_v);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_v, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;
    if (unlikely(!__pyx_cur_scope->__pyx_outer_scope->__pyx_v_bound)) { __Pyx_RaiseClosureNameError("bound"); __PYX_ERR(0, 168, __pyx_L1_error) }
    __Pyx_INCREF(__pyx_cur_scope->__pyx_outer_scope->__pyx_v_bound->cmp);
    __pyx_t_4 = __pyx_cur_scope->__pyx_outer_scope->__pyx_v_bound->cmp; __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_cur_scope->__pyx_v_v, __pyx_int_0};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 168, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_cur_scope->__pyx_v_v, __pyx_int_0};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 168, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 168, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_v);
      __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_v);
      PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_cur_scope->__pyx_v_v);
      __Pyx_INCREF(__pyx_int_0);
      __Pyx_GIVEREF(__pyx_int_0);
      PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_int_0);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 168, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_8 < 0)) __PYX_ERR(0, 168, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_8) {
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_v);
      __pyx_r = __pyx_cur_scope->__pyx_v_v;
      __Pyx_XGIVEREF(__pyx_t_1);
      __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
      __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_RefNannyFinishContext();
      /* return from generator, yielding value */
      __pyx_generator->resume_label = 1;
      return __pyx_r;
      __pyx_L7_resume_from_yield:;
      __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
      __pyx_cur_scope->__pyx_t_0 = 0;
      __Pyx_XGOTREF(__pyx_t_1);
      __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 168, __pyx_L1_error)
    }
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":81
 *         self.append((time, value))
 * 
 *     def __init__(self, Gauge gauge):             # <<<<<<<<<<<<<<
 *         """Determines the transformations from the time when the value set to
 *         the farthest future.
 */

static int __pyx_pf_5gauge_13deterministic_13Determination___init__(struct __pyx_obj_5gauge_13deterministic_Determination *__pyx_v_self, struct __pyx_obj_5gauge_4core_Gauge *__pyx_v_gauge) {
  struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ *__pyx_cur_scope;
  double __pyx_v_since;
  double __pyx_v_until;
  double __pyx_v_time;
  double __pyx_v_value;
  double __pyx_v_velocity;
  struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_boundary = 0;
  struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_b = 0;
  double __pyx_v_boundary_value;
  double __pyx_v_value_at_bound;
  double __pyx_v_bound_until;
  int __pyx_v_again;
  int __pyx_v_ok;
  int __pyx_v_method;
  struct __pyx_obj_5gauge_4core_Momentum *__pyx_v_momentum = 0;
  PyObject *__pyx_v_walked_boundaries = 0;
  struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_line = 0;
  __pyx_ctuple_double__and_double __pyx_v_intersection;
  PyObject *__pyx_v_ceil_lines = 0;
  PyObject *__pyx_v_floor_lines = 0;
  PyObject *__pyx_v_ceil = 0;
  PyObject *__pyx_v_floor = 0;
  PyObject *__pyx_v_boundaries = 0;
  int __pyx_v_bounded;
  int __pyx_v_overlapped;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  double __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  __pyx_ctuple_int__and_double __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *(*__pyx_t_15)(PyObject *);
  double __pyx_t_16;
  Py_ssize_t __pyx_t_17;
  __pyx_ctuple_int__and___dunderpyx_ctuple_double__dunderand_double __pyx_t_18;
  __pyx_ctuple_double__and_double __pyx_t_19;
  PyObject *__pyx_t_20 = NULL;
  struct __pyx_opt_args_5gauge_13deterministic_13Determination__determine __pyx_t_21;
  int __pyx_t_22;
  __Pyx_RefNannySetupContext("__init__", 0);
  __pyx_cur_scope = (struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ *)__pyx_tp_new_5gauge_13deterministic___pyx_scope_struct____init__(__pyx_ptype_5gauge_13deterministic___pyx_scope_struct____init__, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 81, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }

  /* "gauge/deterministic.pyx":90
 *             double time
 *             double value
 *             double velocity = 0             # <<<<<<<<<<<<<<
 *             list velocities = []
 *             Boundary boundary
 */
  __pyx_v_velocity = 0.0;

  /* "gauge/deterministic.pyx":91
 *             double value
 *             double velocity = 0
 *             list velocities = []             # <<<<<<<<<<<<<<
 *             Boundary boundary
 *             Boundary bound
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_velocities = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "gauge/deterministic.pyx":105
 *             Line line
 *             (double, double) intersection
 *         since, value = gauge._base_time, gauge._base_value             # <<<<<<<<<<<<<<
 *         self._in_range = False
 *         # boundaries.
 */
  __pyx_t_2 = __pyx_v_gauge->_base_time;
  __pyx_t_3 = __pyx_v_gauge->_base_value;
  __pyx_v_since = __pyx_t_2;
  __pyx_v_value = __pyx_t_3;

  /* "gauge/deterministic.pyx":106
 *             (double, double) intersection
 *         since, value = gauge._base_time, gauge._base_value
 *         self._in_range = False             # <<<<<<<<<<<<<<
 *         # boundaries.
 *         cdef list ceil_lines, floor_lines
 */
  __pyx_v_self->_in_range = 0;

  /* "gauge/deterministic.pyx":109
 *         # boundaries.
 *         cdef list ceil_lines, floor_lines
 *         if gauge._max_gauge is None:             # <<<<<<<<<<<<<<
 *             ceil_lines = VALUE_LINES(gauge, gauge._max_value)
 *         else:
 */
  __pyx_t_4 = (((PyObject *)__pyx_v_gauge->_max_gauge) == Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "gauge/deterministic.pyx":110
 *         cdef list ceil_lines, floor_lines
 *         if gauge._max_gauge is None:
 *             ceil_lines = VALUE_LINES(gauge, gauge._max_value)             # <<<<<<<<<<<<<<
 *         else:
 *             ceil_lines = GAUGE_LINES(gauge, gauge._max_gauge)
 */
    __pyx_t_1 = __pyx_f_5gauge_13deterministic_VALUE_LINES(__pyx_v_gauge, __pyx_v_gauge->_max_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 110, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_ceil_lines = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "gauge/deterministic.pyx":109
 *         # boundaries.
 *         cdef list ceil_lines, floor_lines
 *         if gauge._max_gauge is None:             # <<<<<<<<<<<<<<
 *             ceil_lines = VALUE_LINES(gauge, gauge._max_value)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "gauge/deterministic.pyx":112
 *             ceil_lines = VALUE_LINES(gauge, gauge._max_value)
 *         else:
 *             ceil_lines = GAUGE_LINES(gauge, gauge._max_gauge)             # <<<<<<<<<<<<<<
 *         if gauge._min_gauge is None:
 *             floor_lines = VALUE_LINES(gauge, gauge._min_value)
 */
  /*else*/ {
    __pyx_t_1 = ((PyObject *)__pyx_v_gauge->_max_gauge);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_6 = __pyx_f_5gauge_13deterministic_GAUGE_LINES(__pyx_v_gauge, ((struct __pyx_obj_5gauge_4core_Gauge *)__pyx_t_1)); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 112, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_ceil_lines = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;
  }
  __pyx_L3:;

  /* "gauge/deterministic.pyx":113
 *         else:
 *             ceil_lines = GAUGE_LINES(gauge, gauge._max_gauge)
 *         if gauge._min_gauge is None:             # <<<<<<<<<<<<<<
 *             floor_lines = VALUE_LINES(gauge, gauge._min_value)
 *         else:
 */
  __pyx_t_5 = (((PyObject *)__pyx_v_gauge->_min_gauge) == Py_None);
  __pyx_t_4 = (__pyx_t_5 != 0);
  if (__pyx_t_4) {

    /* "gauge/deterministic.pyx":114
 *             ceil_lines = GAUGE_LINES(gauge, gauge._max_gauge)
 *         if gauge._min_gauge is None:
 *             floor_lines = VALUE_LINES(gauge, gauge._min_value)             # <<<<<<<<<<<<<<
 *         else:
 *             floor_lines = GAUGE_LINES(gauge, gauge._min_gauge)
 */
    __pyx_t_6 = __pyx_f_5gauge_13deterministic_VALUE_LINES(__pyx_v_gauge, __pyx_v_gauge->_min_value); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 114, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_v_floor_lines = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "gauge/deterministic.pyx":113
 *         else:
 *             ceil_lines = GAUGE_LINES(gauge, gauge._max_gauge)
 *         if gauge._min_gauge is None:             # <<<<<<<<<<<<<<
 *             floor_lines = VALUE_LINES(gauge, gauge._min_value)
 *         else:
 */
    goto __pyx_L4;
  }

  /* "gauge/deterministic.pyx":116
 *             floor_lines = VALUE_LINES(gauge, gauge._min_value)
 *         else:
 *             floor_lines = GAUGE_LINES(gauge, gauge._min_gauge)             # <<<<<<<<<<<<<<
 *         cdef:
 *             ceil = Boundary(ceil_lines, operator.lt)
 */
  /*else*/ {
    __pyx_t_6 = ((PyObject *)__pyx_v_gauge->_min_gauge);
    __Pyx_INCREF(__pyx_t_6);
    __pyx_t_1 = __pyx_f_5gauge_13deterministic_GAUGE_LINES(__pyx_v_gauge, ((struct __pyx_obj_5gauge_4core_Gauge *)__pyx_t_6)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_v_floor_lines = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;
  }
  __pyx_L4:;

  /* "gauge/deterministic.pyx":118
 *             floor_lines = GAUGE_LINES(gauge, gauge._min_gauge)
 *         cdef:
 *             ceil = Boundary(ceil_lines, operator.lt)             # <<<<<<<<<<<<<<
 *             floor = Boundary(floor_lines, operator.gt)
 *             list boundaries = [ceil, floor]
 */
  __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_operator); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_lt); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_ceil_lines);
  __Pyx_GIVEREF(__pyx_v_ceil_lines);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_ceil_lines);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_6);
  __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5gauge_13deterministic_Boundary), __pyx_t_1, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 118, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_ceil = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "gauge/deterministic.pyx":119
 *         cdef:
 *             ceil = Boundary(ceil_lines, operator.lt)
 *             floor = Boundary(floor_lines, operator.gt)             # <<<<<<<<<<<<<<
 *             list boundaries = [ceil, floor]
 *             bint bounded = False
 */
  __pyx_t_6 = __Pyx_GetModuleGlobalName(__pyx_n_s_operator); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_gt); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_v_floor_lines);
  __Pyx_GIVEREF(__pyx_v_floor_lines);
  PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_floor_lines);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5gauge_13deterministic_Boundary), __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_v_floor = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "gauge/deterministic.pyx":120
 *             ceil = Boundary(ceil_lines, operator.lt)
 *             floor = Boundary(floor_lines, operator.gt)
 *             list boundaries = [ceil, floor]             # <<<<<<<<<<<<<<
 *             bint bounded = False
 *             bint overlapped = False
 */
  __pyx_t_1 = PyList_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_ceil);
  __Pyx_GIVEREF(__pyx_v_ceil);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_v_ceil);
  __Pyx_INCREF(__pyx_v_floor);
  __Pyx_GIVEREF(__pyx_v_floor);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_v_floor);
  __pyx_v_boundaries = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "gauge/deterministic.pyx":121
 *             floor = Boundary(floor_lines, operator.gt)
 *             list boundaries = [ceil, floor]
 *             bint bounded = False             # <<<<<<<<<<<<<<
 *             bint overlapped = False
 *         for boundary in boundaries:
 */
  __pyx_v_bounded = 0;

  /* "gauge/deterministic.pyx":122
 *             list boundaries = [ceil, floor]
 *             bint bounded = False
 *             bint overlapped = False             # <<<<<<<<<<<<<<
 *         for boundary in boundaries:
 *             # skip past boundaries.
 */
  __pyx_v_overlapped = 0;

  /* "gauge/deterministic.pyx":123
 *             bint bounded = False
 *             bint overlapped = False
 *         for boundary in boundaries:             # <<<<<<<<<<<<<<
 *             # skip past boundaries.
 *             while boundary.line.until <= since:
 */
  __pyx_t_1 = __pyx_v_boundaries; __Pyx_INCREF(__pyx_t_1); __pyx_t_7 = 0;
  for (;;) {
    if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_1)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_6 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_7); __Pyx_INCREF(__pyx_t_6); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 123, __pyx_L1_error)
    #else
    __pyx_t_6 = PySequence_ITEM(__pyx_t_1, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 123, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    #endif
    if (!(likely(((__pyx_t_6) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_6, __pyx_ptype_5gauge_13deterministic_Boundary))))) __PYX_ERR(0, 123, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_boundary, ((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_t_6));
    __pyx_t_6 = 0;

    /* "gauge/deterministic.pyx":125
 *         for boundary in boundaries:
 *             # skip past boundaries.
 *             while boundary.line.until <= since:             # <<<<<<<<<<<<<<
 *                 boundary.walk()
 *             # check overflowing.
 */
    while (1) {
      __pyx_t_4 = ((__pyx_v_boundary->line->until <= __pyx_v_since) != 0);
      if (!__pyx_t_4) break;

      /* "gauge/deterministic.pyx":126
 *             # skip past boundaries.
 *             while boundary.line.until <= since:
 *                 boundary.walk()             # <<<<<<<<<<<<<<
 *             # check overflowing.
 *             if bounded:
 */
      __pyx_t_6 = ((struct __pyx_vtabstruct_5gauge_13deterministic_Boundary *)__pyx_v_boundary->__pyx_vtab)->walk(__pyx_v_boundary, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 126, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }

    /* "gauge/deterministic.pyx":128
 *                 boundary.walk()
 *             # check overflowing.
 *             if bounded:             # <<<<<<<<<<<<<<
 *                 continue
 *             ok, boundary_value = boundary.line._guess(since)
 */
    __pyx_t_4 = (__pyx_v_bounded != 0);
    if (__pyx_t_4) {

      /* "gauge/deterministic.pyx":129
 *             # check overflowing.
 *             if bounded:
 *                 continue             # <<<<<<<<<<<<<<
 *             ok, boundary_value = boundary.line._guess(since)
 *             assert ok
 */
      goto __pyx_L5_continue;

      /* "gauge/deterministic.pyx":128
 *                 boundary.walk()
 *             # check overflowing.
 *             if bounded:             # <<<<<<<<<<<<<<
 *                 continue
 *             ok, boundary_value = boundary.line._guess(since)
 */
    }

    /* "gauge/deterministic.pyx":130
 *             if bounded:
 *                 continue
 *             ok, boundary_value = boundary.line._guess(since)             # <<<<<<<<<<<<<<
 *             assert ok
 *             if boundary.cmp(boundary_value, value):
 */
    __pyx_t_8 = ((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_boundary->line->__pyx_vtab)->_guess(__pyx_v_boundary->line, __pyx_v_since);
    __pyx_t_4 = __pyx_t_8.f0;
    __pyx_t_3 = __pyx_t_8.f1;
    __pyx_v_ok = __pyx_t_4;
    __pyx_v_boundary_value = __pyx_t_3;

    /* "gauge/deterministic.pyx":131
 *                 continue
 *             ok, boundary_value = boundary.line._guess(since)
 *             assert ok             # <<<<<<<<<<<<<<
 *             if boundary.cmp(boundary_value, value):
 *                 bound, bounded, overlapped = boundary, True, False
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!(__pyx_v_ok != 0))) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 131, __pyx_L1_error)
      }
    }
    #endif

    /* "gauge/deterministic.pyx":132
 *             ok, boundary_value = boundary.line._guess(since)
 *             assert ok
 *             if boundary.cmp(boundary_value, value):             # <<<<<<<<<<<<<<
 *                 bound, bounded, overlapped = boundary, True, False
 *         for time, method, momentum in gauge.momentum_events():
 */
    __pyx_t_9 = PyFloat_FromDouble(__pyx_v_boundary_value); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 132, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 132, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_INCREF(__pyx_v_boundary->cmp);
    __pyx_t_11 = __pyx_v_boundary->cmp; __pyx_t_12 = NULL;
    __pyx_t_13 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
      __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_11);
      if (likely(__pyx_t_12)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_12);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_11, function);
        __pyx_t_13 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_11)) {
      PyObject *__pyx_temp[3] = {__pyx_t_12, __pyx_t_9, __pyx_t_10};
      __pyx_t_6 = __Pyx_PyFunction_FastCall(__pyx_t_11, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 132, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_11)) {
      PyObject *__pyx_temp[3] = {__pyx_t_12, __pyx_t_9, __pyx_t_10};
      __pyx_t_6 = __Pyx_PyCFunction_FastCall(__pyx_t_11, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 132, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    } else
    #endif
    {
      __pyx_t_14 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 132, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      if (__pyx_t_12) {
        __Pyx_GIVEREF(__pyx_t_12); PyTuple_SET_ITEM(__pyx_t_14, 0, __pyx_t_12); __pyx_t_12 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_14, 0+__pyx_t_13, __pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_10);
      PyTuple_SET_ITEM(__pyx_t_14, 1+__pyx_t_13, __pyx_t_10);
      __pyx_t_9 = 0;
      __pyx_t_10 = 0;
      __pyx_t_6 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_t_14, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 132, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    }
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 132, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (__pyx_t_4) {

      /* "gauge/deterministic.pyx":133
 *             assert ok
 *             if boundary.cmp(boundary_value, value):
 *                 bound, bounded, overlapped = boundary, True, False             # <<<<<<<<<<<<<<
 *         for time, method, momentum in gauge.momentum_events():
 *             # normalize time.
 */
      __pyx_t_6 = ((PyObject *)__pyx_v_boundary);
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_4 = 1;
      __pyx_t_5 = 0;
      __Pyx_XGOTREF(((PyObject *)__pyx_cur_scope->__pyx_v_bound));
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_bound, ((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_t_6));
      __Pyx_GIVEREF(__pyx_t_6);
      __pyx_t_6 = 0;
      __pyx_v_bounded = __pyx_t_4;
      __pyx_v_overlapped = __pyx_t_5;

      /* "gauge/deterministic.pyx":132
 *             ok, boundary_value = boundary.line._guess(since)
 *             assert ok
 *             if boundary.cmp(boundary_value, value):             # <<<<<<<<<<<<<<
 *                 bound, bounded, overlapped = boundary, True, False
 *         for time, method, momentum in gauge.momentum_events():
 */
    }

    /* "gauge/deterministic.pyx":123
 *             bint bounded = False
 *             bint overlapped = False
 *         for boundary in boundaries:             # <<<<<<<<<<<<<<
 *             # skip past boundaries.
 *             while boundary.line.until <= since:
 */
    __pyx_L5_continue:;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "gauge/deterministic.pyx":134
 *             if boundary.cmp(boundary_value, value):
 *                 bound, bounded, overlapped = boundary, True, False
 *         for time, method, momentum in gauge.momentum_events():             # <<<<<<<<<<<<<<
 *             # normalize time.
 *             until = max(time, gauge._base_time)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5gauge_4core_Gauge *)__pyx_v_gauge->__pyx_vtab)->momentum_events(__pyx_v_gauge, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 134, __pyx_L1_error)
  }
  __pyx_t_6 = __pyx_t_1; __Pyx_INCREF(__pyx_t_6); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (__pyx_t_7 >= PyList_GET_SIZE(__pyx_t_6)) break;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_1 = PyList_GET_ITEM(__pyx_t_6, __pyx_t_7); __Pyx_INCREF(__pyx_t_1); __pyx_t_7++; if (unlikely(0 < 0)) __PYX_ERR(0, 134, __pyx_L1_error)
    #else
    __pyx_t_1 = PySequence_ITEM(__pyx_t_6, __pyx_t_7); __pyx_t_7++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    #endif
    if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
      PyObject* sequence = __pyx_t_1;
      #if !CYTHON_COMPILING_IN_PYPY
      Py_ssize_t size = Py_SIZE(sequence);
      #else
      Py_ssize_t size = PySequence_Size(sequence);
      #endif
      if (unlikely(size != 3)) {
        if (size > 3) __Pyx_RaiseTooManyValuesError(3);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 134, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_11 = PyTuple_GET_ITEM(sequence, 0); 
        __pyx_t_14 = PyTuple_GET_ITEM(sequence, 1); 
        __pyx_t_10 = PyTuple_GET_ITEM(sequence, 2); 
      } else {
        __pyx_t_11 = PyList_GET_ITEM(sequence, 0); 
        __pyx_t_14 = PyList_GET_ITEM(sequence, 1); 
        __pyx_t_10 = PyList_GET_ITEM(sequence, 2); 
      }
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_t_14);
      __Pyx_INCREF(__pyx_t_10);
      #else
      __pyx_t_11 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 134, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_14 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 134, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_10 = PySequence_ITEM(sequence, 2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 134, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_9 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 134, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_15 = Py_TYPE(__pyx_t_9)->tp_iternext;
      index = 0; __pyx_t_11 = __pyx_t_15(__pyx_t_9); if (unlikely(!__pyx_t_11)) goto __pyx_L13_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_11);
      index = 1; __pyx_t_14 = __pyx_t_15(__pyx_t_9); if (unlikely(!__pyx_t_14)) goto __pyx_L13_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_14);
      index = 2; __pyx_t_10 = __pyx_t_15(__pyx_t_9); if (unlikely(!__pyx_t_10)) goto __pyx_L13_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_10);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_15(__pyx_t_9), 3) < 0) __PYX_ERR(0, 134, __pyx_L1_error)
      __pyx_t_15 = NULL;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L14_unpacking_done;
      __pyx_L13_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_15 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 134, __pyx_L1_error)
      __pyx_L14_unpacking_done:;
    }
    __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_11); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 134, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_13 = __Pyx_PyInt_As_int(__pyx_t_14); if (unlikely((__pyx_t_13 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 134, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    if (!(likely(((__pyx_t_10) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_10, __pyx_ptype_5gauge_4core_Momentum))))) __PYX_ERR(0, 134, __pyx_L1_error)
    __pyx_v_time = __pyx_t_3;
    __pyx_v_method = __pyx_t_13;
    __Pyx_XDECREF_SET(__pyx_v_momentum, ((struct __pyx_obj_5gauge_4core_Momentum *)__pyx_t_10));
    __pyx_t_10 = 0;

    /* "gauge/deterministic.pyx":136
 *         for time, method, momentum in gauge.momentum_events():
 *             # normalize time.
 *             until = max(time, gauge._base_time)             # <<<<<<<<<<<<<<
 *             # if True, An iteration doesn't choose next boundaries.  The first
 *             # iteration doesn't require to choose next boundaries.
 */
    __pyx_t_3 = __pyx_v_gauge->_base_time;
    __pyx_t_2 = __pyx_v_time;
    if (((__pyx_t_3 > __pyx_t_2) != 0)) {
      __pyx_t_16 = __pyx_t_3;
    } else {
      __pyx_t_16 = __pyx_t_2;
    }
    __pyx_v_until = __pyx_t_16;

    /* "gauge/deterministic.pyx":139
 *             # if True, An iteration doesn't choose next boundaries.  The first
 *             # iteration doesn't require to choose next boundaries.
 *             again = True             # <<<<<<<<<<<<<<
 *             while since < until:
 *                 if again:
 */
    __pyx_v_again = 1;

    /* "gauge/deterministic.pyx":140
 *             # iteration doesn't require to choose next boundaries.
 *             again = True
 *             while since < until:             # <<<<<<<<<<<<<<
 *                 if again:
 *                     again = False
 */
    while (1) {
      __pyx_t_5 = ((__pyx_v_since < __pyx_v_until) != 0);
      if (!__pyx_t_5) break;

      /* "gauge/deterministic.pyx":141
 *             again = True
 *             while since < until:
 *                 if again:             # <<<<<<<<<<<<<<
 *                     again = False
 *                     if bounded:
 */
      __pyx_t_5 = (__pyx_v_again != 0);
      if (__pyx_t_5) {

        /* "gauge/deterministic.pyx":142
 *             while since < until:
 *                 if again:
 *                     again = False             # <<<<<<<<<<<<<<
 *                     if bounded:
 *                         walked_boundaries = [bound]
 */
        __pyx_v_again = 0;

        /* "gauge/deterministic.pyx":143
 *                 if again:
 *                     again = False
 *                     if bounded:             # <<<<<<<<<<<<<<
 *                         walked_boundaries = [bound]
 *                     else:
 */
        __pyx_t_5 = (__pyx_v_bounded != 0);
        if (__pyx_t_5) {

          /* "gauge/deterministic.pyx":144
 *                     again = False
 *                     if bounded:
 *                         walked_boundaries = [bound]             # <<<<<<<<<<<<<<
 *                     else:
 *                         walked_boundaries = boundaries
 */
          if (unlikely(!__pyx_cur_scope->__pyx_v_bound)) { __Pyx_RaiseUnboundLocalError("bound"); __PYX_ERR(0, 144, __pyx_L1_error) }
          __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_v_bound));
          __Pyx_GIVEREF(((PyObject *)__pyx_cur_scope->__pyx_v_bound));
          PyList_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_cur_scope->__pyx_v_bound));
          __Pyx_XDECREF_SET(__pyx_v_walked_boundaries, ((PyObject*)__pyx_t_1));
          __pyx_t_1 = 0;

          /* "gauge/deterministic.pyx":143
 *                 if again:
 *                     again = False
 *                     if bounded:             # <<<<<<<<<<<<<<
 *                         walked_boundaries = [bound]
 *                     else:
 */
          goto __pyx_L18;
        }

        /* "gauge/deterministic.pyx":146
 *                         walked_boundaries = [bound]
 *                     else:
 *                         walked_boundaries = boundaries             # <<<<<<<<<<<<<<
 *                 else:
 *                     # stop the loop if all boundaries have been proceeded.
 */
        /*else*/ {
          __Pyx_INCREF(__pyx_v_boundaries);
          __Pyx_XDECREF_SET(__pyx_v_walked_boundaries, __pyx_v_boundaries);
        }
        __pyx_L18:;

        /* "gauge/deterministic.pyx":141
 *             again = True
 *             while since < until:
 *                 if again:             # <<<<<<<<<<<<<<
 *                     again = False
 *                     if bounded:
 */
        goto __pyx_L17;
      }

      /* "gauge/deterministic.pyx":149
 *                 else:
 *                     # stop the loop if all boundaries have been proceeded.
 *                     for b in boundaries:             # <<<<<<<<<<<<<<
 *                         if b.line.until < until:
 *                             break
 */
      /*else*/ {
        __pyx_t_1 = __pyx_v_boundaries; __Pyx_INCREF(__pyx_t_1); __pyx_t_17 = 0;
        for (;;) {
          if (__pyx_t_17 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_10 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_17); __Pyx_INCREF(__pyx_t_10); __pyx_t_17++; if (unlikely(0 < 0)) __PYX_ERR(0, 149, __pyx_L1_error)
          #else
          __pyx_t_10 = PySequence_ITEM(__pyx_t_1, __pyx_t_17); __pyx_t_17++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 149, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          #endif
          if (!(likely(((__pyx_t_10) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_10, __pyx_ptype_5gauge_13deterministic_Boundary))))) __PYX_ERR(0, 149, __pyx_L1_error)
          __Pyx_XDECREF_SET(__pyx_v_b, ((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_t_10));
          __pyx_t_10 = 0;

          /* "gauge/deterministic.pyx":150
 *                     # stop the loop if all boundaries have been proceeded.
 *                     for b in boundaries:
 *                         if b.line.until < until:             # <<<<<<<<<<<<<<
 *                             break
 *                     else:
 */
          __pyx_t_5 = ((__pyx_v_b->line->until < __pyx_v_until) != 0);
          if (__pyx_t_5) {

            /* "gauge/deterministic.pyx":151
 *                     for b in boundaries:
 *                         if b.line.until < until:
 *                             break             # <<<<<<<<<<<<<<
 *                     else:
 *                         break
 */
            goto __pyx_L20_break;

            /* "gauge/deterministic.pyx":150
 *                     # stop the loop if all boundaries have been proceeded.
 *                     for b in boundaries:
 *                         if b.line.until < until:             # <<<<<<<<<<<<<<
 *                             break
 *                     else:
 */
          }

          /* "gauge/deterministic.pyx":149
 *                 else:
 *                     # stop the loop if all boundaries have been proceeded.
 *                     for b in boundaries:             # <<<<<<<<<<<<<<
 *                         if b.line.until < until:
 *                             break
 */
        }
        /*else*/ {

          /* "gauge/deterministic.pyx":153
 *                             break
 *                     else:
 *                         break             # <<<<<<<<<<<<<<
 *                     # choose the next boundary.
 *                     boundary = boundaries[0]
 */
          goto __pyx_L16_break;
        }

        /* "gauge/deterministic.pyx":149
 *                 else:
 *                     # stop the loop if all boundaries have been proceeded.
 *                     for b in boundaries:             # <<<<<<<<<<<<<<
 *                         if b.line.until < until:
 *                             break
 */
        __pyx_L20_break:;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "gauge/deterministic.pyx":155
 *                         break
 *                     # choose the next boundary.
 *                     boundary = boundaries[0]             # <<<<<<<<<<<<<<
 *                     for b in boundaries:
 *                         if b.line.until < boundary.line.until:
 */
        __pyx_t_1 = __Pyx_GetItemInt_List(__pyx_v_boundaries, 0, long, 1, __Pyx_PyInt_From_long, 1, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 155, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5gauge_13deterministic_Boundary))))) __PYX_ERR(0, 155, __pyx_L1_error)
        __Pyx_XDECREF_SET(__pyx_v_boundary, ((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "gauge/deterministic.pyx":156
 *                     # choose the next boundary.
 *                     boundary = boundaries[0]
 *                     for b in boundaries:             # <<<<<<<<<<<<<<
 *                         if b.line.until < boundary.line.until:
 *                             boundary = b
 */
        __pyx_t_1 = __pyx_v_boundaries; __Pyx_INCREF(__pyx_t_1); __pyx_t_17 = 0;
        for (;;) {
          if (__pyx_t_17 >= PyList_GET_SIZE(__pyx_t_1)) break;
          #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
          __pyx_t_10 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_17); __Pyx_INCREF(__pyx_t_10); __pyx_t_17++; if (unlikely(0 < 0)) __PYX_ERR(0, 156, __pyx_L1_error)
          #else
          __pyx_t_10 = PySequence_ITEM(__pyx_t_1, __pyx_t_17); __pyx_t_17++; if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 156, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          #endif
          if (!(likely(((__pyx_t_10) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_10, __pyx_ptype_5gauge_13deterministic_Boundary))))) __PYX_ERR(0, 156, __pyx_L1_error)
          __Pyx_DECREF_SET(__pyx_v_b, ((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_t_10));
          __pyx_t_10 = 0;

          /* "gauge/deterministic.pyx":157
 *                     boundary = boundaries[0]
 *                     for b in boundaries:
 *                         if b.line.until < boundary.line.until:             # <<<<<<<<<<<<<<
 *                             boundary = b
 *                     boundary.walk()
 */
          __pyx_t_5 = ((__pyx_v_b->line->until < __pyx_v_boundary->line->until) != 0);
          if (__pyx_t_5) {

            /* "gauge/deterministic.pyx":158
 *                     for b in boundaries:
 *                         if b.line.until < boundary.line.until:
 *                             boundary = b             # <<<<<<<<<<<<<<
 *                     boundary.walk()
 *                     walked_boundaries = [boundary]
 */
            __Pyx_INCREF(((PyObject *)__pyx_v_b));
            __Pyx_DECREF_SET(__pyx_v_boundary, __pyx_v_b);

            /* "gauge/deterministic.pyx":157
 *                     boundary = boundaries[0]
 *                     for b in boundaries:
 *                         if b.line.until < boundary.line.until:             # <<<<<<<<<<<<<<
 *                             boundary = b
 *                     boundary.walk()
 */
          }

          /* "gauge/deterministic.pyx":156
 *                     # choose the next boundary.
 *                     boundary = boundaries[0]
 *                     for b in boundaries:             # <<<<<<<<<<<<<<
 *                         if b.line.until < boundary.line.until:
 *                             boundary = b
 */
        }
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "gauge/deterministic.pyx":159
 *                         if b.line.until < boundary.line.until:
 *                             boundary = b
 *                     boundary.walk()             # <<<<<<<<<<<<<<
 *                     walked_boundaries = [boundary]
 *                 # calculate velocity.
 */
        __pyx_t_1 = ((struct __pyx_vtabstruct_5gauge_13deterministic_Boundary *)__pyx_v_boundary->__pyx_vtab)->walk(__pyx_v_boundary, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "gauge/deterministic.pyx":160
 *                             boundary = b
 *                     boundary.walk()
 *                     walked_boundaries = [boundary]             # <<<<<<<<<<<<<<
 *                 # calculate velocity.
 *                 if not bounded:
 */
        __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 160, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_INCREF(((PyObject *)__pyx_v_boundary));
        __Pyx_GIVEREF(((PyObject *)__pyx_v_boundary));
        PyList_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_boundary));
        __Pyx_XDECREF_SET(__pyx_v_walked_boundaries, ((PyObject*)__pyx_t_1));
        __pyx_t_1 = 0;
      }
      __pyx_L17:;

      /* "gauge/deterministic.pyx":162
 *                     walked_boundaries = [boundary]
 *                 # calculate velocity.
 *                 if not bounded:             # <<<<<<<<<<<<<<
 *                     velocity = sum(velocities)
 *                 elif overlapped:
 */
      __pyx_t_5 = ((!(__pyx_v_bounded != 0)) != 0);
      if (__pyx_t_5) {

        /* "gauge/deterministic.pyx":163
 *                 # calculate velocity.
 *                 if not bounded:
 *                     velocity = sum(velocities)             # <<<<<<<<<<<<<<
 *                 elif overlapped:
 *                     velocity = bound.best(sum(velocities),
 */
        __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_cur_scope->__pyx_v_velocities);
        __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_velocities);
        PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_cur_scope->__pyx_v_velocities);
        __pyx_t_10 = __Pyx_PyObject_Call(__pyx_builtin_sum, __pyx_t_1, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_16 = __pyx_PyFloat_AsDouble(__pyx_t_10); if (unlikely((__pyx_t_16 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_v_velocity = __pyx_t_16;

        /* "gauge/deterministic.pyx":162
 *                     walked_boundaries = [boundary]
 *                 # calculate velocity.
 *                 if not bounded:             # <<<<<<<<<<<<<<
 *                     velocity = sum(velocities)
 *                 elif overlapped:
 */
        goto __pyx_L26;
      }

      /* "gauge/deterministic.pyx":164
 *                 if not bounded:
 *                     velocity = sum(velocities)
 *                 elif overlapped:             # <<<<<<<<<<<<<<
 *                     velocity = bound.best(sum(velocities),
 *                                           bound.line.velocity())
 */
      __pyx_t_5 = (__pyx_v_overlapped != 0);
      if (__pyx_t_5) {

        /* "gauge/deterministic.pyx":165
 *                     velocity = sum(velocities)
 *                 elif overlapped:
 *                     velocity = bound.best(sum(velocities),             # <<<<<<<<<<<<<<
 *                                           bound.line.velocity())
 *                 else:
 */
        if (unlikely(!__pyx_cur_scope->__pyx_v_bound)) { __Pyx_RaiseUnboundLocalError("bound"); __PYX_ERR(0, 165, __pyx_L1_error) }
        __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 165, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_cur_scope->__pyx_v_velocities);
        __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_velocities);
        PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_cur_scope->__pyx_v_velocities);
        __pyx_t_14 = __Pyx_PyObject_Call(__pyx_builtin_sum, __pyx_t_1, NULL); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 165, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_14);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "gauge/deterministic.pyx":166
 *                 elif overlapped:
 *                     velocity = bound.best(sum(velocities),
 *                                           bound.line.velocity())             # <<<<<<<<<<<<<<
 *                 else:
 *                     velocity = sum(v for v in velocities if bound.cmp(v, 0))
 */
        if (unlikely(!__pyx_cur_scope->__pyx_v_bound)) { __Pyx_RaiseUnboundLocalError("bound"); __PYX_ERR(0, 166, __pyx_L1_error) }
        __pyx_t_1 = PyFloat_FromDouble(((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_cur_scope->__pyx_v_bound->line->__pyx_vtab)->velocity(__pyx_cur_scope->__pyx_v_bound->line, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_cur_scope->__pyx_v_bound->best);
        __pyx_t_11 = __pyx_cur_scope->__pyx_v_bound->best; __pyx_t_9 = NULL;
        __pyx_t_13 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_11);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_11, function);
            __pyx_t_13 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_11)) {
          PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_t_14, __pyx_t_1};
          __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_11, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 165, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_11)) {
          PyObject *__pyx_temp[3] = {__pyx_t_9, __pyx_t_14, __pyx_t_1};
          __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_11, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 165, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        } else
        #endif
        {
          __pyx_t_12 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 165, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          if (__pyx_t_9) {
            __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_12, 0, __pyx_t_9); __pyx_t_9 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_14);
          PyTuple_SET_ITEM(__pyx_t_12, 0+__pyx_t_13, __pyx_t_14);
          __Pyx_GIVEREF(__pyx_t_1);
          PyTuple_SET_ITEM(__pyx_t_12, 1+__pyx_t_13, __pyx_t_1);
          __pyx_t_14 = 0;
          __pyx_t_1 = 0;
          __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_t_12, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 165, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_10);
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        }
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

        /* "gauge/deterministic.pyx":165
 *                     velocity = sum(velocities)
 *                 elif overlapped:
 *                     velocity = bound.best(sum(velocities),             # <<<<<<<<<<<<<<
 *                                           bound.line.velocity())
 *                 else:
 */
        __pyx_t_16 = __pyx_PyFloat_AsDouble(__pyx_t_10); if (unlikely((__pyx_t_16 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 165, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_v_velocity = __pyx_t_16;

        /* "gauge/deterministic.pyx":164
 *                 if not bounded:
 *                     velocity = sum(velocities)
 *                 elif overlapped:             # <<<<<<<<<<<<<<
 *                     velocity = bound.best(sum(velocities),
 *                                           bound.line.velocity())
 */
        goto __pyx_L26;
      }

      /* "gauge/deterministic.pyx":168
 *                                           bound.line.velocity())
 *                 else:
 *                     velocity = sum(v for v in velocities if bound.cmp(v, 0))             # <<<<<<<<<<<<<<
 *                 # is still bound?
 *                 if overlapped and bound.cmp(velocity, bound.line.velocity()):
 */
      /*else*/ {
        __pyx_t_10 = __pyx_pf_5gauge_13deterministic_13Determination_8__init___genexpr(((PyObject*)__pyx_cur_scope)); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_11 = PyTuple_New(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_10);
        PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10);
        __pyx_t_10 = 0;
        __pyx_t_10 = __Pyx_PyObject_Call(__pyx_builtin_sum, __pyx_t_11, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_16 = __pyx_PyFloat_AsDouble(__pyx_t_10); if (unlikely((__pyx_t_16 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 168, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_v_velocity = __pyx_t_16;
      }
      __pyx_L26:;

      /* "gauge/deterministic.pyx":170
 *                     velocity = sum(v for v in velocities if bound.cmp(v, 0))
 *                 # is still bound?
 *                 if overlapped and bound.cmp(velocity, bound.line.velocity()):             # <<<<<<<<<<<<<<
 *                     bounded, overlapped = False, False
 *                     again = True
 */
      __pyx_t_4 = (__pyx_v_overlapped != 0);
      if (__pyx_t_4) {
      } else {
        __pyx_t_5 = __pyx_t_4;
        goto __pyx_L28_bool_binop_done;
      }
      if (unlikely(!__pyx_cur_scope->__pyx_v_bound)) { __Pyx_RaiseUnboundLocalError("bound"); __PYX_ERR(0, 170, __pyx_L1_error) }
      __pyx_t_11 = PyFloat_FromDouble(__pyx_v_velocity); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 170, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      if (unlikely(!__pyx_cur_scope->__pyx_v_bound)) { __Pyx_RaiseUnboundLocalError("bound"); __PYX_ERR(0, 170, __pyx_L1_error) }
      __pyx_t_12 = PyFloat_FromDouble(((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_cur_scope->__pyx_v_bound->line->__pyx_vtab)->velocity(__pyx_cur_scope->__pyx_v_bound->line, 0)); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 170, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_INCREF(__pyx_cur_scope->__pyx_v_bound->cmp);
      __pyx_t_1 = __pyx_cur_scope->__pyx_v_bound->cmp; __pyx_t_14 = NULL;
      __pyx_t_13 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_1);
        if (likely(__pyx_t_14)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
          __Pyx_INCREF(__pyx_t_14);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_1, function);
          __pyx_t_13 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[3] = {__pyx_t_14, __pyx_t_11, __pyx_t_12};
        __pyx_t_10 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 170, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
        PyObject *__pyx_temp[3] = {__pyx_t_14, __pyx_t_11, __pyx_t_12};
        __pyx_t_10 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 170, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      } else
      #endif
      {
        __pyx_t_9 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 170, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (__pyx_t_14) {
          __Pyx_GIVEREF(__pyx_t_14); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_14); __pyx_t_14 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_11);
        PyTuple_SET_ITEM(__pyx_t_9, 0+__pyx_t_13, __pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_12);
        PyTuple_SET_ITEM(__pyx_t_9, 1+__pyx_t_13, __pyx_t_12);
        __pyx_t_11 = 0;
        __pyx_t_12 = 0;
        __pyx_t_10 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_9, NULL); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 170, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_10); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 170, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __pyx_t_5 = __pyx_t_4;
      __pyx_L28_bool_binop_done:;
      if (__pyx_t_5) {

        /* "gauge/deterministic.pyx":171
 *                 # is still bound?
 *                 if overlapped and bound.cmp(velocity, bound.line.velocity()):
 *                     bounded, overlapped = False, False             # <<<<<<<<<<<<<<
 *                     again = True
 *                     continue
 */
        __pyx_t_5 = 0;
        __pyx_t_4 = 0;
        __pyx_v_bounded = __pyx_t_5;
        __pyx_v_overlapped = __pyx_t_4;

        /* "gauge/deterministic.pyx":172
 *                 if overlapped and bound.cmp(velocity, bound.line.velocity()):
 *                     bounded, overlapped = False, False
 *                     again = True             # <<<<<<<<<<<<<<
 *                     continue
 *                 # current value line.
 */
        __pyx_v_again = 1;

        /* "gauge/deterministic.pyx":173
 *                     bounded, overlapped = False, False
 *                     again = True
 *                     continue             # <<<<<<<<<<<<<<
 *                 # current value line.
 *                 line = Line(LN_RAY, since, until, value, velocity)
 */
        goto __pyx_L15_continue;

        /* "gauge/deterministic.pyx":170
 *                     velocity = sum(v for v in velocities if bound.cmp(v, 0))
 *                 # is still bound?
 *                 if overlapped and bound.cmp(velocity, bound.line.velocity()):             # <<<<<<<<<<<<<<
 *                     bounded, overlapped = False, False
 *                     again = True
 */
      }

      /* "gauge/deterministic.pyx":175
 *                     continue
 *                 # current value line.
 *                 line = Line(LN_RAY, since, until, value, velocity)             # <<<<<<<<<<<<<<
 *                 if overlapped:
 *                     bound_until = min(bound.line.until, until)
 */
      __pyx_t_10 = PyFloat_FromDouble(__pyx_v_since); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 175, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_1 = PyFloat_FromDouble(__pyx_v_until); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 175, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_9 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 175, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_12 = PyFloat_FromDouble(__pyx_v_velocity); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 175, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_11 = PyTuple_New(5); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 175, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_INCREF(__pyx_int_2);
      __Pyx_GIVEREF(__pyx_int_2);
      PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_int_2);
      __Pyx_GIVEREF(__pyx_t_10);
      PyTuple_SET_ITEM(__pyx_t_11, 1, __pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_1);
      PyTuple_SET_ITEM(__pyx_t_11, 2, __pyx_t_1);
      __Pyx_GIVEREF(__pyx_t_9);
      PyTuple_SET_ITEM(__pyx_t_11, 3, __pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_12);
      PyTuple_SET_ITEM(__pyx_t_11, 4, __pyx_t_12);
      __pyx_t_10 = 0;
      __pyx_t_1 = 0;
      __pyx_t_9 = 0;
      __pyx_t_12 = 0;
      __pyx_t_12 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5gauge_13deterministic_Line), __pyx_t_11, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 175, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF_SET(__pyx_v_line, ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_t_12));
      __pyx_t_12 = 0;

      /* "gauge/deterministic.pyx":176
 *                 # current value line.
 *                 line = Line(LN_RAY, since, until, value, velocity)
 *                 if overlapped:             # <<<<<<<<<<<<<<
 *                     bound_until = min(bound.line.until, until)
 *                     if bound_until == +INF:
 */
      __pyx_t_4 = (__pyx_v_overlapped != 0);
      if (__pyx_t_4) {

        /* "gauge/deterministic.pyx":177
 *                 line = Line(LN_RAY, since, until, value, velocity)
 *                 if overlapped:
 *                     bound_until = min(bound.line.until, until)             # <<<<<<<<<<<<<<
 *                     if bound_until == +INF:
 *                         break
 */
        __pyx_t_16 = __pyx_v_until;
        if (unlikely(!__pyx_cur_scope->__pyx_v_bound)) { __Pyx_RaiseUnboundLocalError("bound"); __PYX_ERR(0, 177, __pyx_L1_error) }
        __pyx_t_3 = __pyx_cur_scope->__pyx_v_bound->line->until;
        if (((__pyx_t_16 < __pyx_t_3) != 0)) {
          __pyx_t_2 = __pyx_t_16;
        } else {
          __pyx_t_2 = __pyx_t_3;
        }
        __pyx_v_bound_until = __pyx_t_2;

        /* "gauge/deterministic.pyx":178
 *                 if overlapped:
 *                     bound_until = min(bound.line.until, until)
 *                     if bound_until == +INF:             # <<<<<<<<<<<<<<
 *                         break
 *                     # released from the boundary.
 */
        __pyx_t_4 = ((__pyx_v_bound_until == __pyx_v_5gauge_9constants_INF) != 0);
        if (__pyx_t_4) {

          /* "gauge/deterministic.pyx":179
 *                     bound_until = min(bound.line.until, until)
 *                     if bound_until == +INF:
 *                         break             # <<<<<<<<<<<<<<
 *                     # released from the boundary.
 *                     since = bound_until
 */
          goto __pyx_L16_break;

          /* "gauge/deterministic.pyx":178
 *                 if overlapped:
 *                     bound_until = min(bound.line.until, until)
 *                     if bound_until == +INF:             # <<<<<<<<<<<<<<
 *                         break
 *                     # released from the boundary.
 */
        }

        /* "gauge/deterministic.pyx":181
 *                         break
 *                     # released from the boundary.
 *                     since = bound_until             # <<<<<<<<<<<<<<
 *                     ok, value = bound.line._get(bound_until)
 *                     assert ok
 */
        __pyx_v_since = __pyx_v_bound_until;

        /* "gauge/deterministic.pyx":182
 *                     # released from the boundary.
 *                     since = bound_until
 *                     ok, value = bound.line._get(bound_until)             # <<<<<<<<<<<<<<
 *                     assert ok
 *                     self._determine(since, value)
 */
        if (unlikely(!__pyx_cur_scope->__pyx_v_bound)) { __Pyx_RaiseUnboundLocalError("bound"); __PYX_ERR(0, 182, __pyx_L1_error) }
        __pyx_t_8 = ((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_cur_scope->__pyx_v_bound->line->__pyx_vtab)->_get(__pyx_cur_scope->__pyx_v_bound->line, __pyx_v_bound_until);
        __pyx_t_4 = __pyx_t_8.f0;
        __pyx_t_2 = __pyx_t_8.f1;
        __pyx_v_ok = __pyx_t_4;
        __pyx_v_value = __pyx_t_2;

        /* "gauge/deterministic.pyx":183
 *                     since = bound_until
 *                     ok, value = bound.line._get(bound_until)
 *                     assert ok             # <<<<<<<<<<<<<<
 *                     self._determine(since, value)
 *                     continue
 */
        #ifndef CYTHON_WITHOUT_ASSERTIONS
        if (unlikely(!Py_OptimizeFlag)) {
          if (unlikely(!(__pyx_v_ok != 0))) {
            PyErr_SetNone(PyExc_AssertionError);
            __PYX_ERR(0, 183, __pyx_L1_error)
          }
        }
        #endif

        /* "gauge/deterministic.pyx":184
 *                     ok, value = bound.line._get(bound_until)
 *                     assert ok
 *                     self._determine(since, value)             # <<<<<<<<<<<<<<
 *                     continue
 *                 for boundary in walked_boundaries:
 */
        ((struct __pyx_vtabstruct_5gauge_13deterministic_Determination *)__pyx_v_self->__pyx_vtab)->_determine(__pyx_v_self, __pyx_v_since, __pyx_v_value, NULL);

        /* "gauge/deterministic.pyx":185
 *                     assert ok
 *                     self._determine(since, value)
 *                     continue             # <<<<<<<<<<<<<<
 *                 for boundary in walked_boundaries:
 *                     # find the intersection with a boundary.
 */
        goto __pyx_L15_continue;

        /* "gauge/deterministic.pyx":176
 *                 # current value line.
 *                 line = Line(LN_RAY, since, until, value, velocity)
 *                 if overlapped:             # <<<<<<<<<<<<<<
 *                     bound_until = min(bound.line.until, until)
 *                     if bound_until == +INF:
 */
      }

      /* "gauge/deterministic.pyx":186
 *                     self._determine(since, value)
 *                     continue
 *                 for boundary in walked_boundaries:             # <<<<<<<<<<<<<<
 *                     # find the intersection with a boundary.
 *                     ok, intersection = line._intersect(boundary.line)
 */
      __pyx_t_12 = __pyx_v_walked_boundaries; __Pyx_INCREF(__pyx_t_12); __pyx_t_17 = 0;
      for (;;) {
        if (__pyx_t_17 >= PyList_GET_SIZE(__pyx_t_12)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_11 = PyList_GET_ITEM(__pyx_t_12, __pyx_t_17); __Pyx_INCREF(__pyx_t_11); __pyx_t_17++; if (unlikely(0 < 0)) __PYX_ERR(0, 186, __pyx_L1_error)
        #else
        __pyx_t_11 = PySequence_ITEM(__pyx_t_12, __pyx_t_17); __pyx_t_17++; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 186, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        #endif
        if (!(likely(((__pyx_t_11) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_11, __pyx_ptype_5gauge_13deterministic_Boundary))))) __PYX_ERR(0, 186, __pyx_L1_error)
        __Pyx_XDECREF_SET(__pyx_v_boundary, ((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_t_11));
        __pyx_t_11 = 0;

        /* "gauge/deterministic.pyx":188
 *                 for boundary in walked_boundaries:
 *                     # find the intersection with a boundary.
 *                     ok, intersection = line._intersect(boundary.line)             # <<<<<<<<<<<<<<
 *                     if not ok:
 *                         continue
 */
        __pyx_t_11 = ((PyObject *)__pyx_v_boundary->line);
        __Pyx_INCREF(__pyx_t_11);
        __pyx_t_18 = ((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_line->__pyx_vtab)->_intersect(__pyx_v_line, ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_t_11));
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_t_4 = __pyx_t_18.f0;
        __pyx_t_19 = __pyx_t_18.f1;
        __pyx_v_ok = __pyx_t_4;
        __pyx_v_intersection = __pyx_t_19;

        /* "gauge/deterministic.pyx":189
 *                     # find the intersection with a boundary.
 *                     ok, intersection = line._intersect(boundary.line)
 *                     if not ok:             # <<<<<<<<<<<<<<
 *                         continue
 *                     if intersection[TIME] == since:
 */
        __pyx_t_4 = ((!(__pyx_v_ok != 0)) != 0);
        if (__pyx_t_4) {

          /* "gauge/deterministic.pyx":190
 *                     ok, intersection = line._intersect(boundary.line)
 *                     if not ok:
 *                         continue             # <<<<<<<<<<<<<<
 *                     if intersection[TIME] == since:
 *                         continue
 */
          goto __pyx_L32_continue;

          /* "gauge/deterministic.pyx":189
 *                     # find the intersection with a boundary.
 *                     ok, intersection = line._intersect(boundary.line)
 *                     if not ok:             # <<<<<<<<<<<<<<
 *                         continue
 *                     if intersection[TIME] == since:
 */
        }

        /* "gauge/deterministic.pyx":191
 *                     if not ok:
 *                         continue
 *                     if intersection[TIME] == since:             # <<<<<<<<<<<<<<
 *                         continue
 *                     again = True  # iterate with same boundaries again.
 */
        __pyx_t_4 = ((__pyx_v_intersection.f0 == __pyx_v_since) != 0);
        if (__pyx_t_4) {

          /* "gauge/deterministic.pyx":192
 *                         continue
 *                     if intersection[TIME] == since:
 *                         continue             # <<<<<<<<<<<<<<
 *                     again = True  # iterate with same boundaries again.
 *                     bound, bounded, overlapped = boundary, True, True
 */
          goto __pyx_L32_continue;

          /* "gauge/deterministic.pyx":191
 *                     if not ok:
 *                         continue
 *                     if intersection[TIME] == since:             # <<<<<<<<<<<<<<
 *                         continue
 *                     again = True  # iterate with same boundaries again.
 */
        }

        /* "gauge/deterministic.pyx":193
 *                     if intersection[TIME] == since:
 *                         continue
 *                     again = True  # iterate with same boundaries again.             # <<<<<<<<<<<<<<
 *                     bound, bounded, overlapped = boundary, True, True
 *                     since, value = intersection
 */
        __pyx_v_again = 1;

        /* "gauge/deterministic.pyx":194
 *                         continue
 *                     again = True  # iterate with same boundaries again.
 *                     bound, bounded, overlapped = boundary, True, True             # <<<<<<<<<<<<<<
 *                     since, value = intersection
 *                     # clamp by the boundary.
 */
        __pyx_t_11 = ((PyObject *)__pyx_v_boundary);
        __Pyx_INCREF(__pyx_t_11);
        __pyx_t_4 = 1;
        __pyx_t_5 = 1;
        __Pyx_XGOTREF(((PyObject *)__pyx_cur_scope->__pyx_v_bound));
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_bound, ((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_t_11));
        __Pyx_GIVEREF(__pyx_t_11);
        __pyx_t_11 = 0;
        __pyx_v_bounded = __pyx_t_4;
        __pyx_v_overlapped = __pyx_t_5;

        /* "gauge/deterministic.pyx":195
 *                     again = True  # iterate with same boundaries again.
 *                     bound, bounded, overlapped = boundary, True, True
 *                     since, value = intersection             # <<<<<<<<<<<<<<
 *                     # clamp by the boundary.
 *                     ok, boundary_value = boundary.line._guess(since)
 */
        __pyx_t_19 = __pyx_v_intersection;
        __pyx_t_2 = __pyx_t_19.f0;
        __pyx_t_16 = __pyx_t_19.f1;
        __pyx_v_since = __pyx_t_2;
        __pyx_v_value = __pyx_t_16;

        /* "gauge/deterministic.pyx":197
 *                     since, value = intersection
 *                     # clamp by the boundary.
 *                     ok, boundary_value = boundary.line._guess(since)             # <<<<<<<<<<<<<<
 *                     assert ok
 *                     value = boundary.best(value, boundary_value)
 */
        __pyx_t_8 = ((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_boundary->line->__pyx_vtab)->_guess(__pyx_v_boundary->line, __pyx_v_since);
        __pyx_t_5 = __pyx_t_8.f0;
        __pyx_t_16 = __pyx_t_8.f1;
        __pyx_v_ok = __pyx_t_5;
        __pyx_v_boundary_value = __pyx_t_16;

        /* "gauge/deterministic.pyx":198
 *                     # clamp by the boundary.
 *                     ok, boundary_value = boundary.line._guess(since)
 *                     assert ok             # <<<<<<<<<<<<<<
 *                     value = boundary.best(value, boundary_value)
 *                     self._determine(since, value)
 */
        #ifndef CYTHON_WITHOUT_ASSERTIONS
        if (unlikely(!Py_OptimizeFlag)) {
          if (unlikely(!(__pyx_v_ok != 0))) {
            PyErr_SetNone(PyExc_AssertionError);
            __PYX_ERR(0, 198, __pyx_L1_error)
          }
        }
        #endif

        /* "gauge/deterministic.pyx":199
 *                     ok, boundary_value = boundary.line._guess(since)
 *                     assert ok
 *                     value = boundary.best(value, boundary_value)             # <<<<<<<<<<<<<<
 *                     self._determine(since, value)
 *                     break
 */
        __pyx_t_9 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 199, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_1 = PyFloat_FromDouble(__pyx_v_boundary_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 199, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_v_boundary->best);
        __pyx_t_10 = __pyx_v_boundary->best; __pyx_t_14 = NULL;
        __pyx_t_13 = 0;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_10);
          if (likely(__pyx_t_14)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_14);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_10, function);
            __pyx_t_13 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[3] = {__pyx_t_14, __pyx_t_9, __pyx_t_1};
          __pyx_t_11 = __Pyx_PyFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 199, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_10)) {
          PyObject *__pyx_temp[3] = {__pyx_t_14, __pyx_t_9, __pyx_t_1};
          __pyx_t_11 = __Pyx_PyCFunction_FastCall(__pyx_t_10, __pyx_temp+1-__pyx_t_13, 2+__pyx_t_13); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 199, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        } else
        #endif
        {
          __pyx_t_20 = PyTuple_New(2+__pyx_t_13); if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 199, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_20);
          if (__pyx_t_14) {
            __Pyx_GIVEREF(__pyx_t_14); PyTuple_SET_ITEM(__pyx_t_20, 0, __pyx_t_14); __pyx_t_14 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_9);
          PyTuple_SET_ITEM(__pyx_t_20, 0+__pyx_t_13, __pyx_t_9);
          __Pyx_GIVEREF(__pyx_t_1);
          PyTuple_SET_ITEM(__pyx_t_20, 1+__pyx_t_13, __pyx_t_1);
          __pyx_t_9 = 0;
          __pyx_t_1 = 0;
          __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_20, NULL); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 199, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
          __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
        }
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_16 = __pyx_PyFloat_AsDouble(__pyx_t_11); if (unlikely((__pyx_t_16 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 199, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __pyx_v_value = __pyx_t_16;

        /* "gauge/deterministic.pyx":200
 *                     assert ok
 *                     value = boundary.best(value, boundary_value)
 *                     self._determine(since, value)             # <<<<<<<<<<<<<<
 *                     break
 *                 if bounded:
 */
        ((struct __pyx_vtabstruct_5gauge_13deterministic_Determination *)__pyx_v_self->__pyx_vtab)->_determine(__pyx_v_self, __pyx_v_since, __pyx_v_value, NULL);

        /* "gauge/deterministic.pyx":201
 *                     value = boundary.best(value, boundary_value)
 *                     self._determine(since, value)
 *                     break             # <<<<<<<<<<<<<<
 *                 if bounded:
 *                     continue  # the intersection was found.
 */
        goto __pyx_L33_break;

        /* "gauge/deterministic.pyx":186
 *                     self._determine(since, value)
 *                     continue
 *                 for boundary in walked_boundaries:             # <<<<<<<<<<<<<<
 *                     # find the intersection with a boundary.
 *                     ok, intersection = line._intersect(boundary.line)
 */
        __pyx_L32_continue:;
      }
      __pyx_L33_break:;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

      /* "gauge/deterministic.pyx":202
 *                     self._determine(since, value)
 *                     break
 *                 if bounded:             # <<<<<<<<<<<<<<
 *                     continue  # the intersection was found.
 *                 for boundary in walked_boundaries:
 */
      __pyx_t_5 = (__pyx_v_bounded != 0);
      if (__pyx_t_5) {

        /* "gauge/deterministic.pyx":203
 *                     break
 *                 if bounded:
 *                     continue  # the intersection was found.             # <<<<<<<<<<<<<<
 *                 for boundary in walked_boundaries:
 *                     # find missing intersection caused by floating-point
 */
        goto __pyx_L15_continue;

        /* "gauge/deterministic.pyx":202
 *                     self._determine(since, value)
 *                     break
 *                 if bounded:             # <<<<<<<<<<<<<<
 *                     continue  # the intersection was found.
 *                 for boundary in walked_boundaries:
 */
      }

      /* "gauge/deterministic.pyx":204
 *                 if bounded:
 *                     continue  # the intersection was found.
 *                 for boundary in walked_boundaries:             # <<<<<<<<<<<<<<
 *                     # find missing intersection caused by floating-point
 *                     # inaccuracy.
 */
      __pyx_t_12 = __pyx_v_walked_boundaries; __Pyx_INCREF(__pyx_t_12); __pyx_t_17 = 0;
      for (;;) {
        if (__pyx_t_17 >= PyList_GET_SIZE(__pyx_t_12)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_11 = PyList_GET_ITEM(__pyx_t_12, __pyx_t_17); __Pyx_INCREF(__pyx_t_11); __pyx_t_17++; if (unlikely(0 < 0)) __PYX_ERR(0, 204, __pyx_L1_error)
        #else
        __pyx_t_11 = PySequence_ITEM(__pyx_t_12, __pyx_t_17); __pyx_t_17++; if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 204, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        #endif
        if (!(likely(((__pyx_t_11) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_11, __pyx_ptype_5gauge_13deterministic_Boundary))))) __PYX_ERR(0, 204, __pyx_L1_error)
        __Pyx_XDECREF_SET(__pyx_v_boundary, ((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_t_11));
        __pyx_t_11 = 0;

        /* "gauge/deterministic.pyx":207
 *                     # find missing intersection caused by floating-point
 *                     # inaccuracy.
 *                     bound_until = min(boundary.line.until, until)             # <<<<<<<<<<<<<<
 *                     if bound_until == +INF or bound_until < since:
 *                         continue
 */
        __pyx_t_16 = __pyx_v_until;
        __pyx_t_2 = __pyx_v_boundary->line->until;
        if (((__pyx_t_16 < __pyx_t_2) != 0)) {
          __pyx_t_3 = __pyx_t_16;
        } else {
          __pyx_t_3 = __pyx_t_2;
        }
        __pyx_v_bound_until = __pyx_t_3;

        /* "gauge/deterministic.pyx":208
 *                     # inaccuracy.
 *                     bound_until = min(boundary.line.until, until)
 *                     if bound_until == +INF or bound_until < since:             # <<<<<<<<<<<<<<
 *                         continue
 *                     ok, boundary_value = boundary.line._get(bound_until)
 */
        __pyx_t_4 = ((__pyx_v_bound_until == __pyx_v_5gauge_9constants_INF) != 0);
        if (!__pyx_t_4) {
        } else {
          __pyx_t_5 = __pyx_t_4;
          goto __pyx_L40_bool_binop_done;
        }
        __pyx_t_4 = ((__pyx_v_bound_until < __pyx_v_since) != 0);
        __pyx_t_5 = __pyx_t_4;
        __pyx_L40_bool_binop_done:;
        if (__pyx_t_5) {

          /* "gauge/deterministic.pyx":209
 *                     bound_until = min(boundary.line.until, until)
 *                     if bound_until == +INF or bound_until < since:
 *                         continue             # <<<<<<<<<<<<<<
 *                     ok, boundary_value = boundary.line._get(bound_until)
 *                     assert ok
 */
          goto __pyx_L37_continue;

          /* "gauge/deterministic.pyx":208
 *                     # inaccuracy.
 *                     bound_until = min(boundary.line.until, until)
 *                     if bound_until == +INF or bound_until < since:             # <<<<<<<<<<<<<<
 *                         continue
 *                     ok, boundary_value = boundary.line._get(bound_until)
 */
        }

        /* "gauge/deterministic.pyx":210
 *                     if bound_until == +INF or bound_until < since:
 *                         continue
 *                     ok, boundary_value = boundary.line._get(bound_until)             # <<<<<<<<<<<<<<
 *                     assert ok
 *                     ok, value_at_bound = line._get(bound_until)
 */
        __pyx_t_8 = ((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_boundary->line->__pyx_vtab)->_get(__pyx_v_boundary->line, __pyx_v_bound_until);
        __pyx_t_5 = __pyx_t_8.f0;
        __pyx_t_3 = __pyx_t_8.f1;
        __pyx_v_ok = __pyx_t_5;
        __pyx_v_boundary_value = __pyx_t_3;

        /* "gauge/deterministic.pyx":211
 *                         continue
 *                     ok, boundary_value = boundary.line._get(bound_until)
 *                     assert ok             # <<<<<<<<<<<<<<
 *                     ok, value_at_bound = line._get(bound_until)
 *                     assert ok
 */
        #ifndef CYTHON_WITHOUT_ASSERTIONS
        if (unlikely(!Py_OptimizeFlag)) {
          if (unlikely(!(__pyx_v_ok != 0))) {
            PyErr_SetNone(PyExc_AssertionError);
            __PYX_ERR(0, 211, __pyx_L1_error)
          }
        }
        #endif

        /* "gauge/deterministic.pyx":212
 *                     ok, boundary_value = boundary.line._get(bound_until)
 *                     assert ok
 *                     ok, value_at_bound = line._get(bound_until)             # <<<<<<<<<<<<<<
 *                     assert ok
 *                     if boundary.cmp_eq(value_at_bound, boundary_value):
 */
        __pyx_t_8 = ((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_line->__pyx_vtab)->_get(__pyx_v_line, __pyx_v_bound_until);
        __pyx_t_5 = __pyx_t_8.f0;
        __pyx_t_3 = __pyx_t_8.f1;
        __pyx_v_ok = __pyx_t_5;
        __pyx_v_value_at_bound = __pyx_t_3;

        /* "gauge/deterministic.pyx":213
 *                     assert ok
 *                     ok, value_at_bound = line._get(bound_until)
 *                     assert ok             # <<<<<<<<<<<<<<
 *                     if boundary.cmp_eq(value_at_bound, boundary_value):
 *                         continue
 */
        #ifndef CYTHON_WITHOUT_ASSERTIONS
        if (unlikely(!Py_OptimizeFlag)) {
          if (unlikely(!(__pyx_v_ok != 0))) {
            PyErr_SetNone(PyExc_AssertionError);
            __PYX_ERR(0, 213, __pyx_L1_error)
          }
        }
        #endif

        /* "gauge/deterministic.pyx":214
 *                     ok, value_at_bound = line._get(bound_until)
 *                     assert ok
 *                     if boundary.cmp_eq(value_at_bound, boundary_value):             # <<<<<<<<<<<<<<
 *                         continue
 *                     bound, bounded, overlapped = boundary, True, True
 */
        __pyx_t_5 = (((struct __pyx_vtabstruct_5gauge_13deterministic_Boundary *)__pyx_v_boundary->__pyx_vtab)->cmp_eq(__pyx_v_boundary, __pyx_v_value_at_bound, __pyx_v_boundary_value, 0) != 0);
        if (__pyx_t_5) {

          /* "gauge/deterministic.pyx":215
 *                     assert ok
 *                     if boundary.cmp_eq(value_at_bound, boundary_value):
 *                         continue             # <<<<<<<<<<<<<<
 *                     bound, bounded, overlapped = boundary, True, True
 *                     since, value = bound_until, boundary_value
 */
          goto __pyx_L37_continue;

          /* "gauge/deterministic.pyx":214
 *                     ok, value_at_bound = line._get(bound_until)
 *                     assert ok
 *                     if boundary.cmp_eq(value_at_bound, boundary_value):             # <<<<<<<<<<<<<<
 *                         continue
 *                     bound, bounded, overlapped = boundary, True, True
 */
        }

        /* "gauge/deterministic.pyx":216
 *                     if boundary.cmp_eq(value_at_bound, boundary_value):
 *                         continue
 *                     bound, bounded, overlapped = boundary, True, True             # <<<<<<<<<<<<<<
 *                     since, value = bound_until, boundary_value
 *                     self._determine(since, value)
 */
        __pyx_t_11 = ((PyObject *)__pyx_v_boundary);
        __Pyx_INCREF(__pyx_t_11);
        __pyx_t_5 = 1;
        __pyx_t_4 = 1;
        __Pyx_XGOTREF(((PyObject *)__pyx_cur_scope->__pyx_v_bound));
        __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_bound, ((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_t_11));
        __Pyx_GIVEREF(__pyx_t_11);
        __pyx_t_11 = 0;
        __pyx_v_bounded = __pyx_t_5;
        __pyx_v_overlapped = __pyx_t_4;

        /* "gauge/deterministic.pyx":217
 *                         continue
 *                     bound, bounded, overlapped = boundary, True, True
 *                     since, value = bound_until, boundary_value             # <<<<<<<<<<<<<<
 *                     self._determine(since, value)
 *                     break
 */
        __pyx_t_3 = __pyx_v_bound_until;
        __pyx_t_16 = __pyx_v_boundary_value;
        __pyx_v_since = __pyx_t_3;
        __pyx_v_value = __pyx_t_16;

        /* "gauge/deterministic.pyx":218
 *                     bound, bounded, overlapped = boundary, True, True
 *                     since, value = bound_until, boundary_value
 *                     self._determine(since, value)             # <<<<<<<<<<<<<<
 *                     break
 *             if until == +INF:
 */
        ((struct __pyx_vtabstruct_5gauge_13deterministic_Determination *)__pyx_v_self->__pyx_vtab)->_determine(__pyx_v_self, __pyx_v_since, __pyx_v_value, NULL);

        /* "gauge/deterministic.pyx":219
 *                     since, value = bound_until, boundary_value
 *                     self._determine(since, value)
 *                     break             # <<<<<<<<<<<<<<
 *             if until == +INF:
 *                 break
 */
        goto __pyx_L38_break;

        /* "gauge/deterministic.pyx":204
 *                 if bounded:
 *                     continue  # the intersection was found.
 *                 for boundary in walked_boundaries:             # <<<<<<<<<<<<<<
 *                     # find missing intersection caused by floating-point
 *                     # inaccuracy.
 */
        __pyx_L37_continue:;
      }
      __pyx_L38_break:;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      __pyx_L15_continue:;
    }
    __pyx_L16_break:;

    /* "gauge/deterministic.pyx":220
 *                     self._determine(since, value)
 *                     break
 *             if until == +INF:             # <<<<<<<<<<<<<<
 *                 break
 *             # determine the final node in the current itreration.
 */
    __pyx_t_4 = ((__pyx_v_until == __pyx_v_5gauge_9constants_INF) != 0);
    if (__pyx_t_4) {

      /* "gauge/deterministic.pyx":221
 *                     break
 *             if until == +INF:
 *                 break             # <<<<<<<<<<<<<<
 *             # determine the final node in the current itreration.
 *             value += velocity * (until - since)
 */
      goto __pyx_L12_break;

      /* "gauge/deterministic.pyx":220
 *                     self._determine(since, value)
 *                     break
 *             if until == +INF:             # <<<<<<<<<<<<<<
 *                 break
 *             # determine the final node in the current itreration.
 */
    }

    /* "gauge/deterministic.pyx":223
 *                 break
 *             # determine the final node in the current itreration.
 *             value += velocity * (until - since)             # <<<<<<<<<<<<<<
 *             self._determine(until, value, in_range=not bounded or overlapped)
 *             # prepare the next iteration.
 */
    __pyx_v_value = (__pyx_v_value + (__pyx_v_velocity * (__pyx_v_until - __pyx_v_since)));

    /* "gauge/deterministic.pyx":224
 *             # determine the final node in the current itreration.
 *             value += velocity * (until - since)
 *             self._determine(until, value, in_range=not bounded or overlapped)             # <<<<<<<<<<<<<<
 *             # prepare the next iteration.
 *             if method == EV_ADD:
 */
    __pyx_t_5 = ((!(__pyx_v_bounded != 0)) != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L44_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_overlapped != 0);
    __pyx_t_4 = __pyx_t_5;
    __pyx_L44_bool_binop_done:;
    __pyx_t_21.__pyx_n = 1;
    __pyx_t_21.in_range = __pyx_t_4;
    ((struct __pyx_vtabstruct_5gauge_13deterministic_Determination *)__pyx_v_self->__pyx_vtab)->_determine(__pyx_v_self, __pyx_v_until, __pyx_v_value, &__pyx_t_21); 

    /* "gauge/deterministic.pyx":226
 *             self._determine(until, value, in_range=not bounded or overlapped)
 *             # prepare the next iteration.
 *             if method == EV_ADD:             # <<<<<<<<<<<<<<
 *                 velocities.append(momentum.velocity)
 *             elif method == EV_REMOVE:
 */
    __pyx_t_4 = ((__pyx_v_method == __pyx_v_5gauge_9constants_EV_ADD) != 0);
    if (__pyx_t_4) {

      /* "gauge/deterministic.pyx":227
 *             # prepare the next iteration.
 *             if method == EV_ADD:
 *                 velocities.append(momentum.velocity)             # <<<<<<<<<<<<<<
 *             elif method == EV_REMOVE:
 *                 velocities.remove(momentum.velocity)
 */
      __pyx_t_12 = PyFloat_FromDouble(__pyx_v_momentum->velocity); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 227, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_22 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_velocities, __pyx_t_12); if (unlikely(__pyx_t_22 == -1)) __PYX_ERR(0, 227, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

      /* "gauge/deterministic.pyx":226
 *             self._determine(until, value, in_range=not bounded or overlapped)
 *             # prepare the next iteration.
 *             if method == EV_ADD:             # <<<<<<<<<<<<<<
 *                 velocities.append(momentum.velocity)
 *             elif method == EV_REMOVE:
 */
      goto __pyx_L46;
    }

    /* "gauge/deterministic.pyx":228
 *             if method == EV_ADD:
 *                 velocities.append(momentum.velocity)
 *             elif method == EV_REMOVE:             # <<<<<<<<<<<<<<
 *                 velocities.remove(momentum.velocity)
 *             since = until
 */
    __pyx_t_4 = ((__pyx_v_method == __pyx_v_5gauge_9constants_EV_REMOVE) != 0);
    if (__pyx_t_4) {

      /* "gauge/deterministic.pyx":229
 *                 velocities.append(momentum.velocity)
 *             elif method == EV_REMOVE:
 *                 velocities.remove(momentum.velocity)             # <<<<<<<<<<<<<<
 *             since = until
 * 
 */
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_velocities, __pyx_n_s_remove); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_10 = PyFloat_FromDouble(__pyx_v_momentum->velocity); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 229, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __pyx_t_20 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_20 = PyMethod_GET_SELF(__pyx_t_11);
        if (likely(__pyx_t_20)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_20);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_11, function);
        }
      }
      if (!__pyx_t_20) {
        __pyx_t_12 = __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_t_10); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 229, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_12);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_11)) {
          PyObject *__pyx_temp[2] = {__pyx_t_20, __pyx_t_10};
          __pyx_t_12 = __Pyx_PyFunction_FastCall(__pyx_t_11, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 229, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_11)) {
          PyObject *__pyx_temp[2] = {__pyx_t_20, __pyx_t_10};
          __pyx_t_12 = __Pyx_PyCFunction_FastCall(__pyx_t_11, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 229, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        } else
        #endif
        {
          __pyx_t_1 = PyTuple_New(1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 229, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GIVEREF(__pyx_t_20); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_20); __pyx_t_20 = NULL;
          __Pyx_GIVEREF(__pyx_t_10);
          PyTuple_SET_ITEM(__pyx_t_1, 0+1, __pyx_t_10);
          __pyx_t_10 = 0;
          __pyx_t_12 = __Pyx_PyObject_Call(__pyx_t_11, __pyx_t_1, NULL); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 229, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_12);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

      /* "gauge/deterministic.pyx":228
 *             if method == EV_ADD:
 *                 velocities.append(momentum.velocity)
 *             elif method == EV_REMOVE:             # <<<<<<<<<<<<<<
 *                 velocities.remove(momentum.velocity)
 *             since = until
 */
    }
    __pyx_L46:;

    /* "gauge/deterministic.pyx":230
 *             elif method == EV_REMOVE:
 *                 velocities.remove(momentum.velocity)
 *             since = until             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_v_since = __pyx_v_until;

    /* "gauge/deterministic.pyx":134
 *             if boundary.cmp(boundary_value, value):
 *                 bound, bounded, overlapped = boundary, True, False
 *         for time, method, momentum in gauge.momentum_events():             # <<<<<<<<<<<<<<
 *             # normalize time.
 *             until = max(time, gauge._base_time)
 */
  }
  __pyx_L12_break:;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "gauge/deterministic.pyx":81
 *         self.append((time, value))
 * 
 *     def __init__(self, Gauge gauge):             # <<<<<<<<<<<<<<
 *         """Determines the transformations from the time when the value set to
 *         the farthest future.
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_20);
  __Pyx_AddTraceback("gauge.deterministic.Determination.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_boundary);
  __Pyx_XDECREF((PyObject *)__pyx_v_b);
  __Pyx_XDECREF((PyObject *)__pyx_v_momentum);
  __Pyx_XDECREF(__pyx_v_walked_boundaries);
  __Pyx_XDECREF((PyObject *)__pyx_v_line);
  __Pyx_XDECREF(__pyx_v_ceil_lines);
  __Pyx_XDECREF(__pyx_v_floor_lines);
  __Pyx_XDECREF(__pyx_v_ceil);
  __Pyx_XDECREF(__pyx_v_floor);
  __Pyx_XDECREF(__pyx_v_boundaries);
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":251
 *         public double extra
 * 
 *     def __cinit__(self, int type,             # <<<<<<<<<<<<<<
 *                   double since, double until, double value,
 *                   double extra=0):
 */

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_4Line_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5gauge_13deterministic_4Line_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_type;
  double __pyx_v_since;
  double __pyx_v_until;
  double __pyx_v_value;
  double __pyx_v_extra;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_type,&__pyx_n_s_since,&__pyx_n_s_until,&__pyx_n_s_value,&__pyx_n_s_extra,0};
    PyObject* values[5] = {0,0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_since)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 4, 5, 1); __PYX_ERR(0, 251, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_until)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 4, 5, 2); __PYX_ERR(0, 251, __pyx_L3_error)
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 4, 5, 3); __PYX_ERR(0, 251, __pyx_L3_error)
        }
        case  4:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_extra);
          if (value) { values[4] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__cinit__") < 0)) __PYX_ERR(0, 251, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  5: values[4] = PyTuple_GET_ITEM(__pyx_args, 4);
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_type = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_type == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 251, __pyx_L3_error)
    __pyx_v_since = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_since == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 252, __pyx_L3_error)
    __pyx_v_until = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_until == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 252, __pyx_L3_error)
    __pyx_v_value = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_value == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 252, __pyx_L3_error)
    if (values[4]) {
      __pyx_v_extra = __pyx_PyFloat_AsDouble(values[4]); if (unlikely((__pyx_v_extra == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 253, __pyx_L3_error)
    } else {
      __pyx_v_extra = ((double)0.0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__cinit__", 0, 4, 5, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 251, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("gauge.deterministic.Line.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line___cinit__(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self), __pyx_v_type, __pyx_v_since, __pyx_v_until, __pyx_v_value, __pyx_v_extra);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_4Line___cinit__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, int __pyx_v_type, double __pyx_v_since, double __pyx_v_until, double __pyx_v_value, double __pyx_v_extra) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "gauge/deterministic.pyx":254
 *                   double since, double until, double value,
 *                   double extra=0):
 *         assert type in (LN_HORIZON, LN_RAY, LN_SEGMENT)             # <<<<<<<<<<<<<<
 *         self.type = type
 *         self.since = since
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    switch (__pyx_v_type) {
      case 1:
      case 2:
      case 3:
      __pyx_t_1 = 1;
      break;
      default:
      __pyx_t_1 = 0;
      break;
    }
    if (unlikely(!(__pyx_t_1 != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 254, __pyx_L1_error)
    }
  }
  #endif

  /* "gauge/deterministic.pyx":255
 *                   double extra=0):
 *         assert type in (LN_HORIZON, LN_RAY, LN_SEGMENT)
 *         self.type = type             # <<<<<<<<<<<<<<
 *         self.since = since
 *         self.until = until
 */
  __pyx_v_self->type = __pyx_v_type;

  /* "gauge/deterministic.pyx":256
 *         assert type in (LN_HORIZON, LN_RAY, LN_SEGMENT)
 *         self.type = type
 *         self.since = since             # <<<<<<<<<<<<<<
 *         self.until = until
 *         self.value = value
 */
  __pyx_v_self->since = __pyx_v_since;

  /* "gauge/deterministic.pyx":257
 *         self.type = type
 *         self.since = since
 *         self.until = until             # <<<<<<<<<<<<<<
 *         self.value = value
 *         self.extra = extra
 */
  __pyx_v_self->until = __pyx_v_until;

  /* "gauge/deterministic.pyx":258
 *         self.since = since
 *         self.until = until
 *         self.value = value             # <<<<<<<<<<<<<<
 *         self.extra = extra
 * 
 */
  __pyx_v_self->value = __pyx_v_value;

  /* "gauge/deterministic.pyx":259
 *         self.until = until
 *         self.value = value
 *         self.extra = extra             # <<<<<<<<<<<<<<
 * 
 *     cdef (bint, (double, double)) _intersect(self, Line line):
 */
  __pyx_v_self->extra = __pyx_v_extra;

  /* "gauge/deterministic.pyx":251
 *         public double extra
 * 
 *     def __cinit__(self, int type,             # <<<<<<<<<<<<<<
 *                   double since, double until, double value,
 *                   double extra=0):
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("gauge.deterministic.Line.__cinit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":261
 *         self.extra = extra
 * 
 *     cdef (bint, (double, double)) _intersect(self, Line line):             # <<<<<<<<<<<<<<
 *         """Gets the intersection with the given line.
 * 
 */

static __pyx_ctuple_int__and___dunderpyx_ctuple_double__dunderand_double __pyx_f_5gauge_13deterministic_4Line__intersect(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_line) {
  double __pyx_v_time;
  double __pyx_v_value;
  double __pyx_v_since;
  double __pyx_v_until;
  double __pyx_v_velocity_delta;
  double __pyx_v_intercept_delta;
  struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_left = 0;
  struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_right = 0;
  int __pyx_v_ok;
  __pyx_ctuple_int__and___dunderpyx_ctuple_double__dunderand_double __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __pyx_ctuple_double__and_double __pyx_t_7;
  __pyx_ctuple_int__and___dunderpyx_ctuple_double__dunderand_double __pyx_t_8;
  double __pyx_t_9;
  double __pyx_t_10;
  double __pyx_t_11;
  int __pyx_t_12;
  __pyx_ctuple_int__and_double __pyx_t_13;
  __Pyx_RefNannySetupContext("_intersect", 0);

  /* "gauge/deterministic.pyx":277
 *             Line right
 *         # right is more reliable.
 *         if self.type < line.type:             # <<<<<<<<<<<<<<
 *             left, right = self, line
 *         else:
 */
  __pyx_t_1 = ((__pyx_v_self->type < __pyx_v_line->type) != 0);
  if (__pyx_t_1) {

    /* "gauge/deterministic.pyx":278
 *         # right is more reliable.
 *         if self.type < line.type:
 *             left, right = self, line             # <<<<<<<<<<<<<<
 *         else:
 *             left, right = line, self
 */
    __pyx_t_2 = ((PyObject *)__pyx_v_self);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = ((PyObject *)__pyx_v_line);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_left = ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_v_right = ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "gauge/deterministic.pyx":277
 *             Line right
 *         # right is more reliable.
 *         if self.type < line.type:             # <<<<<<<<<<<<<<
 *             left, right = self, line
 *         else:
 */
    goto __pyx_L3;
  }

  /* "gauge/deterministic.pyx":280
 *             left, right = self, line
 *         else:
 *             left, right = line, self             # <<<<<<<<<<<<<<
 *         if math.isinf(right.velocity()):
 *             # right is almost vertical.
 */
  /*else*/ {
    __pyx_t_3 = ((PyObject *)__pyx_v_line);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_2 = ((PyObject *)__pyx_v_self);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_left = ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_v_right = ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_t_2);
    __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "gauge/deterministic.pyx":281
 *         else:
 *             left, right = line, self
 *         if math.isinf(right.velocity()):             # <<<<<<<<<<<<<<
 *             # right is almost vertical.
 *             time = (right.since + right.until) / 2
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_math); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_isinf); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyFloat_FromDouble(((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_right->__pyx_vtab)->velocity(__pyx_v_right, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (!__pyx_t_5) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 281, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 281, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
      PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_3};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 281, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 281, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_3);
      __pyx_t_3 = 0;
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 281, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_1) {

    /* "gauge/deterministic.pyx":283
 *         if math.isinf(right.velocity()):
 *             # right is almost vertical.
 *             time = (right.since + right.until) / 2             # <<<<<<<<<<<<<<
 *         else:
 *             velocity_delta = left.velocity() - right.velocity()
 */
    __pyx_v_time = ((__pyx_v_right->since + __pyx_v_right->until) / 2.0);

    /* "gauge/deterministic.pyx":281
 *         else:
 *             left, right = line, self
 *         if math.isinf(right.velocity()):             # <<<<<<<<<<<<<<
 *             # right is almost vertical.
 *             time = (right.since + right.until) / 2
 */
    goto __pyx_L4;
  }

  /* "gauge/deterministic.pyx":285
 *             time = (right.since + right.until) / 2
 *         else:
 *             velocity_delta = left.velocity() - right.velocity()             # <<<<<<<<<<<<<<
 *             if velocity_delta == 0:
 *                 # parallel line given.
 */
  /*else*/ {
    __pyx_v_velocity_delta = (((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_left->__pyx_vtab)->velocity(__pyx_v_left, 0) - ((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_right->__pyx_vtab)->velocity(__pyx_v_right, 0));

    /* "gauge/deterministic.pyx":286
 *         else:
 *             velocity_delta = left.velocity() - right.velocity()
 *             if velocity_delta == 0:             # <<<<<<<<<<<<<<
 *                 # parallel line given.
 *                 return (False, (0, 0))
 */
    __pyx_t_1 = ((__pyx_v_velocity_delta == 0.0) != 0);
    if (__pyx_t_1) {

      /* "gauge/deterministic.pyx":288
 *             if velocity_delta == 0:
 *                 # parallel line given.
 *                 return (False, (0, 0))             # <<<<<<<<<<<<<<
 *             intercept_delta = right.intercept() - left.intercept()
 *             time = intercept_delta / velocity_delta
 */
      __pyx_t_7.f0 = 0.0;
      __pyx_t_7.f1 = 0.0;
      __pyx_t_8.f0 = 0;
      __pyx_t_8.f1 = __pyx_t_7;
      __pyx_r = __pyx_t_8;
      goto __pyx_L0;

      /* "gauge/deterministic.pyx":286
 *         else:
 *             velocity_delta = left.velocity() - right.velocity()
 *             if velocity_delta == 0:             # <<<<<<<<<<<<<<
 *                 # parallel line given.
 *                 return (False, (0, 0))
 */
    }

    /* "gauge/deterministic.pyx":289
 *                 # parallel line given.
 *                 return (False, (0, 0))
 *             intercept_delta = right.intercept() - left.intercept()             # <<<<<<<<<<<<<<
 *             time = intercept_delta / velocity_delta
 *         since = max(left.since, right.since)
 */
    __pyx_v_intercept_delta = (((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_right->__pyx_vtab)->intercept(__pyx_v_right, 0) - ((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_left->__pyx_vtab)->intercept(__pyx_v_left, 0));

    /* "gauge/deterministic.pyx":290
 *                 return (False, (0, 0))
 *             intercept_delta = right.intercept() - left.intercept()
 *             time = intercept_delta / velocity_delta             # <<<<<<<<<<<<<<
 *         since = max(left.since, right.since)
 *         until = min(left.until, right.until)
 */
    if (unlikely(__pyx_v_velocity_delta == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      __PYX_ERR(0, 290, __pyx_L1_error)
    }
    __pyx_v_time = (__pyx_v_intercept_delta / __pyx_v_velocity_delta);
  }
  __pyx_L4:;

  /* "gauge/deterministic.pyx":291
 *             intercept_delta = right.intercept() - left.intercept()
 *             time = intercept_delta / velocity_delta
 *         since = max(left.since, right.since)             # <<<<<<<<<<<<<<
 *         until = min(left.until, right.until)
 *         if not since <= time <= until:
 */
  __pyx_t_9 = __pyx_v_right->since;
  __pyx_t_10 = __pyx_v_left->since;
  if (((__pyx_t_9 > __pyx_t_10) != 0)) {
    __pyx_t_11 = __pyx_t_9;
  } else {
    __pyx_t_11 = __pyx_t_10;
  }
  __pyx_v_since = __pyx_t_11;

  /* "gauge/deterministic.pyx":292
 *             time = intercept_delta / velocity_delta
 *         since = max(left.since, right.since)
 *         until = min(left.until, right.until)             # <<<<<<<<<<<<<<
 *         if not since <= time <= until:
 *             # intersection not in the time range.
 */
  __pyx_t_11 = __pyx_v_right->until;
  __pyx_t_9 = __pyx_v_left->until;
  if (((__pyx_t_11 < __pyx_t_9) != 0)) {
    __pyx_t_10 = __pyx_t_11;
  } else {
    __pyx_t_10 = __pyx_t_9;
  }
  __pyx_v_until = __pyx_t_10;

  /* "gauge/deterministic.pyx":293
 *         since = max(left.since, right.since)
 *         until = min(left.until, right.until)
 *         if not since <= time <= until:             # <<<<<<<<<<<<<<
 *             # intersection not in the time range.
 *             return (False, (0, 0))
 */
  __pyx_t_1 = (__pyx_v_since <= __pyx_v_time);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_time <= __pyx_v_until);
  }
  __pyx_t_12 = ((!(__pyx_t_1 != 0)) != 0);
  if (__pyx_t_12) {

    /* "gauge/deterministic.pyx":295
 *         if not since <= time <= until:
 *             # intersection not in the time range.
 *             return (False, (0, 0))             # <<<<<<<<<<<<<<
 *         ok, value = left._get(time)
 *         if not ok:
 */
    __pyx_t_7.f0 = 0.0;
    __pyx_t_7.f1 = 0.0;
    __pyx_t_8.f0 = 0;
    __pyx_t_8.f1 = __pyx_t_7;
    __pyx_r = __pyx_t_8;
    goto __pyx_L0;

    /* "gauge/deterministic.pyx":293
 *         since = max(left.since, right.since)
 *         until = min(left.until, right.until)
 *         if not since <= time <= until:             # <<<<<<<<<<<<<<
 *             # intersection not in the time range.
 *             return (False, (0, 0))
 */
  }

  /* "gauge/deterministic.pyx":296
 *             # intersection not in the time range.
 *             return (False, (0, 0))
 *         ok, value = left._get(time)             # <<<<<<<<<<<<<<
 *         if not ok:
 *             return (False, (0, 0))
 */
  __pyx_t_13 = ((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_left->__pyx_vtab)->_get(__pyx_v_left, __pyx_v_time);
  __pyx_t_12 = __pyx_t_13.f0;
  __pyx_t_10 = __pyx_t_13.f1;
  __pyx_v_ok = __pyx_t_12;
  __pyx_v_value = __pyx_t_10;

  /* "gauge/deterministic.pyx":297
 *             return (False, (0, 0))
 *         ok, value = left._get(time)
 *         if not ok:             # <<<<<<<<<<<<<<
 *             return (False, (0, 0))
 *         return (True, (time, value))
 */
  __pyx_t_12 = ((!(__pyx_v_ok != 0)) != 0);
  if (__pyx_t_12) {

    /* "gauge/deterministic.pyx":298
 *         ok, value = left._get(time)
 *         if not ok:
 *             return (False, (0, 0))             # <<<<<<<<<<<<<<
 *         return (True, (time, value))
 * 
 */
    __pyx_t_7.f0 = 0.0;
    __pyx_t_7.f1 = 0.0;
    __pyx_t_8.f0 = 0;
    __pyx_t_8.f1 = __pyx_t_7;
    __pyx_r = __pyx_t_8;
    goto __pyx_L0;

    /* "gauge/deterministic.pyx":297
 *             return (False, (0, 0))
 *         ok, value = left._get(time)
 *         if not ok:             # <<<<<<<<<<<<<<
 *             return (False, (0, 0))
 *         return (True, (time, value))
 */
  }

  /* "gauge/deterministic.pyx":299
 *         if not ok:
 *             return (False, (0, 0))
 *         return (True, (time, value))             # <<<<<<<<<<<<<<
 * 
 *     def intersect(self, Line line):
 */
  __pyx_t_7.f0 = __pyx_v_time;
  __pyx_t_7.f1 = __pyx_v_value;
  __pyx_t_8.f0 = 1;
  __pyx_t_8.f1 = __pyx_t_7;
  __pyx_r = __pyx_t_8;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":261
 *         self.extra = extra
 * 
 *     cdef (bint, (double, double)) _intersect(self, Line line):             # <<<<<<<<<<<<<<
 *         """Gets the intersection with the given line.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("gauge.deterministic.Line._intersect", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_left);
  __Pyx_XDECREF((PyObject *)__pyx_v_right);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":301
 *         return (True, (time, value))
 * 
 *     def intersect(self, Line line):             # <<<<<<<<<<<<<<
 *         cdef bint ok
 *         cdef (double, double) intersection
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_3intersect(PyObject *__pyx_v_self, PyObject *__pyx_v_line); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_3intersect(PyObject *__pyx_v_self, PyObject *__pyx_v_line) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("intersect (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_line), __pyx_ptype_5gauge_13deterministic_Line, 1, "line", 0))) __PYX_ERR(0, 301, __pyx_L1_error)
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_2intersect(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self), ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_line));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_4Line_2intersect(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_line) {
  int __pyx_v_ok;
  __pyx_ctuple_double__and_double __pyx_v_intersection;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_ctuple_int__and___dunderpyx_ctuple_double__dunderand_double __pyx_t_1;
  int __pyx_t_2;
  __pyx_ctuple_double__and_double __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("intersect", 0);

  /* "gauge/deterministic.pyx":304
 *         cdef bint ok
 *         cdef (double, double) intersection
 *         ok, intersection = self._intersect(line)             # <<<<<<<<<<<<<<
 *         if not ok:
 *             raise ValueError('intersection not available')
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_self->__pyx_vtab)->_intersect(__pyx_v_self, __pyx_v_line);
  __pyx_t_2 = __pyx_t_1.f0;
  __pyx_t_3 = __pyx_t_1.f1;
  __pyx_v_ok = __pyx_t_2;
  __pyx_v_intersection = __pyx_t_3;

  /* "gauge/deterministic.pyx":305
 *         cdef (double, double) intersection
 *         ok, intersection = self._intersect(line)
 *         if not ok:             # <<<<<<<<<<<<<<
 *             raise ValueError('intersection not available')
 *         return intersection
 */
  __pyx_t_2 = ((!(__pyx_v_ok != 0)) != 0);
  if (__pyx_t_2) {

    /* "gauge/deterministic.pyx":306
 *         ok, intersection = self._intersect(line)
 *         if not ok:
 *             raise ValueError('intersection not available')             # <<<<<<<<<<<<<<
 *         return intersection
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 306, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 306, __pyx_L1_error)

    /* "gauge/deterministic.pyx":305
 *         cdef (double, double) intersection
 *         ok, intersection = self._intersect(line)
 *         if not ok:             # <<<<<<<<<<<<<<
 *             raise ValueError('intersection not available')
 *         return intersection
 */
  }

  /* "gauge/deterministic.pyx":307
 *         if not ok:
 *             raise ValueError('intersection not available')
 *         return intersection             # <<<<<<<<<<<<<<
 * 
 *     cpdef double intercept(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = __pyx_convert__to_py___pyx_ctuple_double__and_double(__pyx_v_intersection); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 307, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":301
 *         return (True, (time, value))
 * 
 *     def intersect(self, Line line):             # <<<<<<<<<<<<<<
 *         cdef bint ok
 *         cdef (double, double) intersection
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("gauge.deterministic.Line.intersect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":309
 *         return intersection
 * 
 *     cpdef double intercept(self):             # <<<<<<<<<<<<<<
 *         """Gets the value-intercept. (Y-intercept)"""
 *         return self.value - self.velocity() * self.since
 */

static PyObject *__pyx_pw_5gauge_13deterministic_4Line_5intercept(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_5gauge_13deterministic_4Line_intercept(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  __Pyx_RefNannySetupContext("intercept", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_intercept); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5gauge_13deterministic_4Line_5intercept)) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 309, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 309, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_r = __pyx_t_5;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "gauge/deterministic.pyx":311
 *     cpdef double intercept(self):
 *         """Gets the value-intercept. (Y-intercept)"""
 *         return self.value - self.velocity() * self.since             # <<<<<<<<<<<<<<
 * 
 *     cdef (bint, double) _get(self, double at):
 */
  __pyx_r = (__pyx_v_self->value - (((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_self->__pyx_vtab)->velocity(__pyx_v_self, 0) * __pyx_v_self->since));
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":309
 *         return intersection
 * 
 *     cpdef double intercept(self):             # <<<<<<<<<<<<<<
 *         """Gets the value-intercept. (Y-intercept)"""
 *         return self.value - self.velocity() * self.since
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("gauge.deterministic.Line.intercept", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_5intercept(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5gauge_13deterministic_4Line_4intercept[] = "Gets the value-intercept. (Y-intercept)";
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_5intercept(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("intercept (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_4intercept(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_4Line_4intercept(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("intercept", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_5gauge_13deterministic_4Line_intercept(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("gauge.deterministic.Line.intercept", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":313
 *         return self.value - self.velocity() * self.since
 * 
 *     cdef (bint, double) _get(self, double at):             # <<<<<<<<<<<<<<
 *         if not self.since <= at <= self.until:
 *             return False, 0
 */

static __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__get(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, double __pyx_v_at) {
  __pyx_ctuple_int__and_double __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __pyx_ctuple_int__and_double __pyx_t_3;
  __Pyx_RefNannySetupContext("_get", 0);

  /* "gauge/deterministic.pyx":314
 * 
 *     cdef (bint, double) _get(self, double at):
 *         if not self.since <= at <= self.until:             # <<<<<<<<<<<<<<
 *             return False, 0
 *         if self.type == LN_HORIZON:
 */
  __pyx_t_1 = (__pyx_v_self->since <= __pyx_v_at);
  if (__pyx_t_1) {
    __pyx_t_1 = (__pyx_v_at <= __pyx_v_self->until);
  }
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (__pyx_t_2) {

    /* "gauge/deterministic.pyx":315
 *     cdef (bint, double) _get(self, double at):
 *         if not self.since <= at <= self.until:
 *             return False, 0             # <<<<<<<<<<<<<<
 *         if self.type == LN_HORIZON:
 *             return self._get_horizon(at)
 */
    __pyx_t_3.f0 = 0;
    __pyx_t_3.f1 = 0.0;
    __pyx_r = __pyx_t_3;
    goto __pyx_L0;

    /* "gauge/deterministic.pyx":314
 * 
 *     cdef (bint, double) _get(self, double at):
 *         if not self.since <= at <= self.until:             # <<<<<<<<<<<<<<
 *             return False, 0
 *         if self.type == LN_HORIZON:
 */
  }

  /* "gauge/deterministic.pyx":316
 *         if not self.since <= at <= self.until:
 *             return False, 0
 *         if self.type == LN_HORIZON:             # <<<<<<<<<<<<<<
 *             return self._get_horizon(at)
 *         elif self.type == LN_RAY:
 */
  switch (__pyx_v_self->type) {
    case 1:

    /* "gauge/deterministic.pyx":317
 *             return False, 0
 *         if self.type == LN_HORIZON:
 *             return self._get_horizon(at)             # <<<<<<<<<<<<<<
 *         elif self.type == LN_RAY:
 *             return self._get_ray(at)
 */
    __pyx_r = __pyx_f_5gauge_13deterministic_4Line__get_horizon(__pyx_v_self, __pyx_v_at);
    goto __pyx_L0;

    /* "gauge/deterministic.pyx":316
 *         if not self.since <= at <= self.until:
 *             return False, 0
 *         if self.type == LN_HORIZON:             # <<<<<<<<<<<<<<
 *             return self._get_horizon(at)
 *         elif self.type == LN_RAY:
 */
    break;

    /* "gauge/deterministic.pyx":318
 *         if self.type == LN_HORIZON:
 *             return self._get_horizon(at)
 *         elif self.type == LN_RAY:             # <<<<<<<<<<<<<<
 *             return self._get_ray(at)
 *         elif self.type == LN_SEGMENT:
 */
    case 2:

    /* "gauge/deterministic.pyx":319
 *             return self._get_horizon(at)
 *         elif self.type == LN_RAY:
 *             return self._get_ray(at)             # <<<<<<<<<<<<<<
 *         elif self.type == LN_SEGMENT:
 *             return self._get_segment(at)
 */
    __pyx_r = __pyx_f_5gauge_13deterministic_4Line__get_ray(__pyx_v_self, __pyx_v_at);
    goto __pyx_L0;

    /* "gauge/deterministic.pyx":318
 *         if self.type == LN_HORIZON:
 *             return self._get_horizon(at)
 *         elif self.type == LN_RAY:             # <<<<<<<<<<<<<<
 *             return self._get_ray(at)
 *         elif self.type == LN_SEGMENT:
 */
    break;

    /* "gauge/deterministic.pyx":320
 *         elif self.type == LN_RAY:
 *             return self._get_ray(at)
 *         elif self.type == LN_SEGMENT:             # <<<<<<<<<<<<<<
 *             return self._get_segment(at)
 *         assert 0
 */
    case 3:

    /* "gauge/deterministic.pyx":321
 *             return self._get_ray(at)
 *         elif self.type == LN_SEGMENT:
 *             return self._get_segment(at)             # <<<<<<<<<<<<<<
 *         assert 0
 * 
 */
    __pyx_r = __pyx_f_5gauge_13deterministic_4Line__get_segment(__pyx_v_self, __pyx_v_at);
    goto __pyx_L0;

    /* "gauge/deterministic.pyx":320
 *         elif self.type == LN_RAY:
 *             return self._get_ray(at)
 *         elif self.type == LN_SEGMENT:             # <<<<<<<<<<<<<<
 *             return self._get_segment(at)
 *         assert 0
 */
    break;
    default: break;
  }

  /* "gauge/deterministic.pyx":322
 *         elif self.type == LN_SEGMENT:
 *             return self._get_segment(at)
 *         assert 0             # <<<<<<<<<<<<<<
 * 
 *     def get(self, double at):
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!0)) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 322, __pyx_L1_error)
    }
  }
  #endif

  /* "gauge/deterministic.pyx":313
 *         return self.value - self.velocity() * self.since
 * 
 *     cdef (bint, double) _get(self, double at):             # <<<<<<<<<<<<<<
 *         if not self.since <= at <= self.until:
 *             return False, 0
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("gauge.deterministic.Line._get", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":324
 *         assert 0
 * 
 *     def get(self, double at):             # <<<<<<<<<<<<<<
 *         """Returns the value at the given time."""
 *         cdef bint ok
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_7get(PyObject *__pyx_v_self, PyObject *__pyx_arg_at); /*proto*/
static char __pyx_doc_5gauge_13deterministic_4Line_6get[] = "Returns the value at the given time.";
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_7get(PyObject *__pyx_v_self, PyObject *__pyx_arg_at) {
  double __pyx_v_at;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get (wrapper)", 0);
  assert(__pyx_arg_at); {
    __pyx_v_at = __pyx_PyFloat_AsDouble(__pyx_arg_at); if (unlikely((__pyx_v_at == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 324, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("gauge.deterministic.Line.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_6get(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self), ((double)__pyx_v_at));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_4Line_6get(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, double __pyx_v_at) {
  int __pyx_v_ok;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_ctuple_int__and_double __pyx_t_1;
  int __pyx_t_2;
  double __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_RefNannySetupContext("get", 0);

  /* "gauge/deterministic.pyx":328
 *         cdef bint ok
 *         cdef double value
 *         ok, value = self._get(at)             # <<<<<<<<<<<<<<
 *         if not ok:
 *             raise ValueError('out of the time range: {0:.2f}~{1:.2f}'
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_v_at);
  __pyx_t_2 = __pyx_t_1.f0;
  __pyx_t_3 = __pyx_t_1.f1;
  __pyx_v_ok = __pyx_t_2;
  __pyx_v_value = __pyx_t_3;

  /* "gauge/deterministic.pyx":329
 *         cdef double value
 *         ok, value = self._get(at)
 *         if not ok:             # <<<<<<<<<<<<<<
 *             raise ValueError('out of the time range: {0:.2f}~{1:.2f}'
 *                              ''.format(self.since, self.until))
 */
  __pyx_t_2 = ((!(__pyx_v_ok != 0)) != 0);
  if (__pyx_t_2) {

    /* "gauge/deterministic.pyx":331
 *         if not ok:
 *             raise ValueError('out of the time range: {0:.2f}~{1:.2f}'
 *                              ''.format(self.since, self.until))             # <<<<<<<<<<<<<<
 *         return value
 * 
 */
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_out_of_the_time_range_0_2f_1_2f, __pyx_n_s_format); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->since); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->until); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = NULL;
    __pyx_t_9 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
        __pyx_t_9 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_6, __pyx_t_7};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 331, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[3] = {__pyx_t_8, __pyx_t_6, __pyx_t_7};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_9, 2+__pyx_t_9); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 331, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else
    #endif
    {
      __pyx_t_10 = PyTuple_New(2+__pyx_t_9); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 331, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      if (__pyx_t_8) {
        __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_8); __pyx_t_8 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_6);
      PyTuple_SET_ITEM(__pyx_t_10, 0+__pyx_t_9, __pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_10, 1+__pyx_t_9, __pyx_t_7);
      __pyx_t_6 = 0;
      __pyx_t_7 = 0;
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_10, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 331, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "gauge/deterministic.pyx":330
 *         ok, value = self._get(at)
 *         if not ok:
 *             raise ValueError('out of the time range: {0:.2f}~{1:.2f}'             # <<<<<<<<<<<<<<
 *                              ''.format(self.since, self.until))
 *         return value
 */
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 330, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 330, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 330, __pyx_L1_error)

    /* "gauge/deterministic.pyx":329
 *         cdef double value
 *         ok, value = self._get(at)
 *         if not ok:             # <<<<<<<<<<<<<<
 *             raise ValueError('out of the time range: {0:.2f}~{1:.2f}'
 *                              ''.format(self.since, self.until))
 */
  }

  /* "gauge/deterministic.pyx":332
 *             raise ValueError('out of the time range: {0:.2f}~{1:.2f}'
 *                              ''.format(self.since, self.until))
 *         return value             # <<<<<<<<<<<<<<
 * 
 *     cdef (bint, double) _guess(self, double at):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 332, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":324
 *         assert 0
 * 
 *     def get(self, double at):             # <<<<<<<<<<<<<<
 *         """Returns the value at the given time."""
 *         cdef bint ok
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("gauge.deterministic.Line.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":334
 *         return value
 * 
 *     cdef (bint, double) _guess(self, double at):             # <<<<<<<<<<<<<<
 *         """Returns the value at the given time even the time it out of the time
 *         range.
 */

static __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__guess(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, double __pyx_v_at) {
  __pyx_ctuple_int__and_double __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_guess", 0);

  /* "gauge/deterministic.pyx":338
 *         range.
 *         """
 *         if at < self.since:             # <<<<<<<<<<<<<<
 *             if self.type == LN_HORIZON:
 *                 return self._earlier_horizon(at)
 */
  __pyx_t_1 = ((__pyx_v_at < __pyx_v_self->since) != 0);
  if (__pyx_t_1) {

    /* "gauge/deterministic.pyx":339
 *         """
 *         if at < self.since:
 *             if self.type == LN_HORIZON:             # <<<<<<<<<<<<<<
 *                 return self._earlier_horizon(at)
 *             elif self.type == LN_RAY:
 */
    switch (__pyx_v_self->type) {
      case 1:

      /* "gauge/deterministic.pyx":340
 *         if at < self.since:
 *             if self.type == LN_HORIZON:
 *                 return self._earlier_horizon(at)             # <<<<<<<<<<<<<<
 *             elif self.type == LN_RAY:
 *                 return self._earlier_ray(at)
 */
      __pyx_r = __pyx_f_5gauge_13deterministic_4Line__earlier_horizon(__pyx_v_self, __pyx_v_at);
      goto __pyx_L0;

      /* "gauge/deterministic.pyx":339
 *         """
 *         if at < self.since:
 *             if self.type == LN_HORIZON:             # <<<<<<<<<<<<<<
 *                 return self._earlier_horizon(at)
 *             elif self.type == LN_RAY:
 */
      break;

      /* "gauge/deterministic.pyx":341
 *             if self.type == LN_HORIZON:
 *                 return self._earlier_horizon(at)
 *             elif self.type == LN_RAY:             # <<<<<<<<<<<<<<
 *                 return self._earlier_ray(at)
 *             elif self.type == LN_SEGMENT:
 */
      case 2:

      /* "gauge/deterministic.pyx":342
 *                 return self._earlier_horizon(at)
 *             elif self.type == LN_RAY:
 *                 return self._earlier_ray(at)             # <<<<<<<<<<<<<<
 *             elif self.type == LN_SEGMENT:
 *                 return self._earlier_segment(at)
 */
      __pyx_r = __pyx_f_5gauge_13deterministic_4Line__earlier_ray(__pyx_v_self, __pyx_v_at);
      goto __pyx_L0;

      /* "gauge/deterministic.pyx":341
 *             if self.type == LN_HORIZON:
 *                 return self._earlier_horizon(at)
 *             elif self.type == LN_RAY:             # <<<<<<<<<<<<<<
 *                 return self._earlier_ray(at)
 *             elif self.type == LN_SEGMENT:
 */
      break;

      /* "gauge/deterministic.pyx":343
 *             elif self.type == LN_RAY:
 *                 return self._earlier_ray(at)
 *             elif self.type == LN_SEGMENT:             # <<<<<<<<<<<<<<
 *                 return self._earlier_segment(at)
 *         elif at > self.until:
 */
      case 3:

      /* "gauge/deterministic.pyx":344
 *                 return self._earlier_ray(at)
 *             elif self.type == LN_SEGMENT:
 *                 return self._earlier_segment(at)             # <<<<<<<<<<<<<<
 *         elif at > self.until:
 *             if self.type == LN_HORIZON:
 */
      __pyx_r = __pyx_f_5gauge_13deterministic_4Line__earlier_segment(__pyx_v_self, __pyx_v_at);
      goto __pyx_L0;

      /* "gauge/deterministic.pyx":343
 *             elif self.type == LN_RAY:
 *                 return self._earlier_ray(at)
 *             elif self.type == LN_SEGMENT:             # <<<<<<<<<<<<<<
 *                 return self._earlier_segment(at)
 *         elif at > self.until:
 */
      break;
      default: break;
    }

    /* "gauge/deterministic.pyx":338
 *         range.
 *         """
 *         if at < self.since:             # <<<<<<<<<<<<<<
 *             if self.type == LN_HORIZON:
 *                 return self._earlier_horizon(at)
 */
    goto __pyx_L3;
  }

  /* "gauge/deterministic.pyx":345
 *             elif self.type == LN_SEGMENT:
 *                 return self._earlier_segment(at)
 *         elif at > self.until:             # <<<<<<<<<<<<<<
 *             if self.type == LN_HORIZON:
 *                 return self._later_horizon(at)
 */
  __pyx_t_1 = ((__pyx_v_at > __pyx_v_self->until) != 0);
  if (__pyx_t_1) {

    /* "gauge/deterministic.pyx":346
 *                 return self._earlier_segment(at)
 *         elif at > self.until:
 *             if self.type == LN_HORIZON:             # <<<<<<<<<<<<<<
 *                 return self._later_horizon(at)
 *             elif self.type == LN_RAY:
 */
    switch (__pyx_v_self->type) {
      case 1:

      /* "gauge/deterministic.pyx":347
 *         elif at > self.until:
 *             if self.type == LN_HORIZON:
 *                 return self._later_horizon(at)             # <<<<<<<<<<<<<<
 *             elif self.type == LN_RAY:
 *                 return self._later_ray(at)
 */
      __pyx_r = __pyx_f_5gauge_13deterministic_4Line__later_horizon(__pyx_v_self, __pyx_v_at);
      goto __pyx_L0;

      /* "gauge/deterministic.pyx":346
 *                 return self._earlier_segment(at)
 *         elif at > self.until:
 *             if self.type == LN_HORIZON:             # <<<<<<<<<<<<<<
 *                 return self._later_horizon(at)
 *             elif self.type == LN_RAY:
 */
      break;

      /* "gauge/deterministic.pyx":348
 *             if self.type == LN_HORIZON:
 *                 return self._later_horizon(at)
 *             elif self.type == LN_RAY:             # <<<<<<<<<<<<<<
 *                 return self._later_ray(at)
 *             elif self.type == LN_SEGMENT:
 */
      case 2:

      /* "gauge/deterministic.pyx":349
 *                 return self._later_horizon(at)
 *             elif self.type == LN_RAY:
 *                 return self._later_ray(at)             # <<<<<<<<<<<<<<
 *             elif self.type == LN_SEGMENT:
 *                 return self._later_segment(at)
 */
      __pyx_r = __pyx_f_5gauge_13deterministic_4Line__later_ray(__pyx_v_self, __pyx_v_at);
      goto __pyx_L0;

      /* "gauge/deterministic.pyx":348
 *             if self.type == LN_HORIZON:
 *                 return self._later_horizon(at)
 *             elif self.type == LN_RAY:             # <<<<<<<<<<<<<<
 *                 return self._later_ray(at)
 *             elif self.type == LN_SEGMENT:
 */
      break;

      /* "gauge/deterministic.pyx":350
 *             elif self.type == LN_RAY:
 *                 return self._later_ray(at)
 *             elif self.type == LN_SEGMENT:             # <<<<<<<<<<<<<<
 *                 return self._later_segment(at)
 *         else:
 */
      case 3:

      /* "gauge/deterministic.pyx":351
 *                 return self._later_ray(at)
 *             elif self.type == LN_SEGMENT:
 *                 return self._later_segment(at)             # <<<<<<<<<<<<<<
 *         else:
 *             return self._get(at)
 */
      __pyx_r = __pyx_f_5gauge_13deterministic_4Line__later_segment(__pyx_v_self, __pyx_v_at);
      goto __pyx_L0;

      /* "gauge/deterministic.pyx":350
 *             elif self.type == LN_RAY:
 *                 return self._later_ray(at)
 *             elif self.type == LN_SEGMENT:             # <<<<<<<<<<<<<<
 *                 return self._later_segment(at)
 *         else:
 */
      break;
      default: break;
    }

    /* "gauge/deterministic.pyx":345
 *             elif self.type == LN_SEGMENT:
 *                 return self._earlier_segment(at)
 *         elif at > self.until:             # <<<<<<<<<<<<<<
 *             if self.type == LN_HORIZON:
 *                 return self._later_horizon(at)
 */
    goto __pyx_L3;
  }

  /* "gauge/deterministic.pyx":353
 *                 return self._later_segment(at)
 *         else:
 *             return self._get(at)             # <<<<<<<<<<<<<<
 *         assert 0
 * 
 */
  /*else*/ {
    __pyx_r = ((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_v_at);
    goto __pyx_L0;
  }
  __pyx_L3:;

  /* "gauge/deterministic.pyx":354
 *         else:
 *             return self._get(at)
 *         assert 0             # <<<<<<<<<<<<<<
 * 
 *     def guess(self, double at):
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!0)) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 354, __pyx_L1_error)
    }
  }
  #endif

  /* "gauge/deterministic.pyx":334
 *         return value
 * 
 *     cdef (bint, double) _guess(self, double at):             # <<<<<<<<<<<<<<
 *         """Returns the value at the given time even the time it out of the time
 *         range.
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("gauge.deterministic.Line._guess", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":356
 *         assert 0
 * 
 *     def guess(self, double at):             # <<<<<<<<<<<<<<
 *         cdef bint ok
 *         cdef double value
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_9guess(PyObject *__pyx_v_self, PyObject *__pyx_arg_at); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_9guess(PyObject *__pyx_v_self, PyObject *__pyx_arg_at) {
  double __pyx_v_at;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("guess (wrapper)", 0);
  assert(__pyx_arg_at); {
    __pyx_v_at = __pyx_PyFloat_AsDouble(__pyx_arg_at); if (unlikely((__pyx_v_at == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 356, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("gauge.deterministic.Line.guess", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_8guess(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self), ((double)__pyx_v_at));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_4Line_8guess(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, double __pyx_v_at) {
  int __pyx_v_ok;
  double __pyx_v_value;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __pyx_ctuple_int__and_double __pyx_t_1;
  int __pyx_t_2;
  double __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("guess", 0);

  /* "gauge/deterministic.pyx":359
 *         cdef bint ok
 *         cdef double value
 *         ok, value = self._guess(at)             # <<<<<<<<<<<<<<
 *         if not ok:
 *             raise AssertionError('unexpected failure')
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_self->__pyx_vtab)->_guess(__pyx_v_self, __pyx_v_at);
  __pyx_t_2 = __pyx_t_1.f0;
  __pyx_t_3 = __pyx_t_1.f1;
  __pyx_v_ok = __pyx_t_2;
  __pyx_v_value = __pyx_t_3;

  /* "gauge/deterministic.pyx":360
 *         cdef double value
 *         ok, value = self._guess(at)
 *         if not ok:             # <<<<<<<<<<<<<<
 *             raise AssertionError('unexpected failure')
 *         return value
 */
  __pyx_t_2 = ((!(__pyx_v_ok != 0)) != 0);
  if (__pyx_t_2) {

    /* "gauge/deterministic.pyx":361
 *         ok, value = self._guess(at)
 *         if not ok:
 *             raise AssertionError('unexpected failure')             # <<<<<<<<<<<<<<
 *         return value
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_AssertionError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 361, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 361, __pyx_L1_error)

    /* "gauge/deterministic.pyx":360
 *         cdef double value
 *         ok, value = self._guess(at)
 *         if not ok:             # <<<<<<<<<<<<<<
 *             raise AssertionError('unexpected failure')
 *         return value
 */
  }

  /* "gauge/deterministic.pyx":362
 *         if not ok:
 *             raise AssertionError('unexpected failure')
 *         return value             # <<<<<<<<<<<<<<
 * 
 *     cpdef double velocity(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":356
 *         assert 0
 * 
 *     def guess(self, double at):             # <<<<<<<<<<<<<<
 *         cdef bint ok
 *         cdef double value
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("gauge.deterministic.Line.guess", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":364
 *         return value
 * 
 *     cpdef double velocity(self):             # <<<<<<<<<<<<<<
 *         if self.type == LN_HORIZON:
 *             return self._velocity_horizon()
 */

static PyObject *__pyx_pw_5gauge_13deterministic_4Line_11velocity(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static double __pyx_f_5gauge_13deterministic_4Line_velocity(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, int __pyx_skip_dispatch) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  double __pyx_t_5;
  __Pyx_RefNannySetupContext("velocity", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_velocity); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 364, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5gauge_13deterministic_4Line_11velocity)) {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 364, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 364, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_5 = __pyx_PyFloat_AsDouble(__pyx_t_2); if (unlikely((__pyx_t_5 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 364, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_r = __pyx_t_5;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "gauge/deterministic.pyx":365
 * 
 *     cpdef double velocity(self):
 *         if self.type == LN_HORIZON:             # <<<<<<<<<<<<<<
 *             return self._velocity_horizon()
 *         elif self.type == LN_RAY:
 */
  switch (__pyx_v_self->type) {
    case 1:

    /* "gauge/deterministic.pyx":366
 *     cpdef double velocity(self):
 *         if self.type == LN_HORIZON:
 *             return self._velocity_horizon()             # <<<<<<<<<<<<<<
 *         elif self.type == LN_RAY:
 *             return self._velocity_ray()
 */
    __pyx_r = __pyx_f_5gauge_13deterministic_4Line__velocity_horizon(__pyx_v_self);
    goto __pyx_L0;

    /* "gauge/deterministic.pyx":365
 * 
 *     cpdef double velocity(self):
 *         if self.type == LN_HORIZON:             # <<<<<<<<<<<<<<
 *             return self._velocity_horizon()
 *         elif self.type == LN_RAY:
 */
    break;

    /* "gauge/deterministic.pyx":367
 *         if self.type == LN_HORIZON:
 *             return self._velocity_horizon()
 *         elif self.type == LN_RAY:             # <<<<<<<<<<<<<<
 *             return self._velocity_ray()
 *         elif self.type == LN_SEGMENT:
 */
    case 2:

    /* "gauge/deterministic.pyx":368
 *             return self._velocity_horizon()
 *         elif self.type == LN_RAY:
 *             return self._velocity_ray()             # <<<<<<<<<<<<<<
 *         elif self.type == LN_SEGMENT:
 *             return self._velocity_segment()
 */
    __pyx_r = __pyx_f_5gauge_13deterministic_4Line__velocity_ray(__pyx_v_self);
    goto __pyx_L0;

    /* "gauge/deterministic.pyx":367
 *         if self.type == LN_HORIZON:
 *             return self._velocity_horizon()
 *         elif self.type == LN_RAY:             # <<<<<<<<<<<<<<
 *             return self._velocity_ray()
 *         elif self.type == LN_SEGMENT:
 */
    break;

    /* "gauge/deterministic.pyx":369
 *         elif self.type == LN_RAY:
 *             return self._velocity_ray()
 *         elif self.type == LN_SEGMENT:             # <<<<<<<<<<<<<<
 *             return self._velocity_segment()
 *         assert 0
 */
    case 3:

    /* "gauge/deterministic.pyx":370
 *             return self._velocity_ray()
 *         elif self.type == LN_SEGMENT:
 *             return self._velocity_segment()             # <<<<<<<<<<<<<<
 *         assert 0
 * 
 */
    __pyx_r = __pyx_f_5gauge_13deterministic_4Line__velocity_segment(__pyx_v_self);
    goto __pyx_L0;

    /* "gauge/deterministic.pyx":369
 *         elif self.type == LN_RAY:
 *             return self._velocity_ray()
 *         elif self.type == LN_SEGMENT:             # <<<<<<<<<<<<<<
 *             return self._velocity_segment()
 *         assert 0
 */
    break;
    default: break;
  }

  /* "gauge/deterministic.pyx":371
 *         elif self.type == LN_SEGMENT:
 *             return self._velocity_segment()
 *         assert 0             # <<<<<<<<<<<<<<
 * 
 *     # HORIZON
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    if (unlikely(!0)) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 371, __pyx_L1_error)
    }
  }
  #endif

  /* "gauge/deterministic.pyx":364
 *         return value
 * 
 *     cpdef double velocity(self):             # <<<<<<<<<<<<<<
 *         if self.type == LN_HORIZON:
 *             return self._velocity_horizon()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("gauge.deterministic.Line.velocity", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_11velocity(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_11velocity(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("velocity (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_10velocity(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_4Line_10velocity(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("velocity", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_f_5gauge_13deterministic_4Line_velocity(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 364, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("gauge.deterministic.Line.velocity", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":375
 *     # HORIZON
 * 
 *     cdef inline (bint, double) _get_horizon(self, double at):             # <<<<<<<<<<<<<<
 *         return True, self.value
 * 
 */

static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__get_horizon(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, CYTHON_UNUSED double __pyx_v_at) {
  __pyx_ctuple_int__and_double __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_ctuple_int__and_double __pyx_t_1;
  __Pyx_RefNannySetupContext("_get_horizon", 0);

  /* "gauge/deterministic.pyx":376
 * 
 *     cdef inline (bint, double) _get_horizon(self, double at):
 *         return True, self.value             # <<<<<<<<<<<<<<
 * 
 *     cdef inline (bint, double) _earlier_horizon(self, double at):
 */
  __pyx_t_1.f0 = 1;
  __pyx_t_1.f1 = __pyx_v_self->value;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":375
 *     # HORIZON
 * 
 *     cdef inline (bint, double) _get_horizon(self, double at):             # <<<<<<<<<<<<<<
 *         return True, self.value
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":378
 *         return True, self.value
 * 
 *     cdef inline (bint, double) _earlier_horizon(self, double at):             # <<<<<<<<<<<<<<
 *         return True, self.value
 * 
 */

static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__earlier_horizon(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, CYTHON_UNUSED double __pyx_v_at) {
  __pyx_ctuple_int__and_double __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_ctuple_int__and_double __pyx_t_1;
  __Pyx_RefNannySetupContext("_earlier_horizon", 0);

  /* "gauge/deterministic.pyx":379
 * 
 *     cdef inline (bint, double) _earlier_horizon(self, double at):
 *         return True, self.value             # <<<<<<<<<<<<<<
 * 
 *     cdef inline (bint, double) _later_horizon(self, double at):
 */
  __pyx_t_1.f0 = 1;
  __pyx_t_1.f1 = __pyx_v_self->value;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":378
 *         return True, self.value
 * 
 *     cdef inline (bint, double) _earlier_horizon(self, double at):             # <<<<<<<<<<<<<<
 *         return True, self.value
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":381
 *         return True, self.value
 * 
 *     cdef inline (bint, double) _later_horizon(self, double at):             # <<<<<<<<<<<<<<
 *         return True, self.value
 * 
 */

static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__later_horizon(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, CYTHON_UNUSED double __pyx_v_at) {
  __pyx_ctuple_int__and_double __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_ctuple_int__and_double __pyx_t_1;
  __Pyx_RefNannySetupContext("_later_horizon", 0);

  /* "gauge/deterministic.pyx":382
 * 
 *     cdef inline (bint, double) _later_horizon(self, double at):
 *         return True, self.value             # <<<<<<<<<<<<<<
 * 
 *     cdef inline double _velocity_horizon(self):
 */
  __pyx_t_1.f0 = 1;
  __pyx_t_1.f1 = __pyx_v_self->value;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":381
 *         return True, self.value
 * 
 *     cdef inline (bint, double) _later_horizon(self, double at):             # <<<<<<<<<<<<<<
 *         return True, self.value
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":384
 *         return True, self.value
 * 
 *     cdef inline double _velocity_horizon(self):             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */

static CYTHON_INLINE double __pyx_f_5gauge_13deterministic_4Line__velocity_horizon(CYTHON_UNUSED struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_velocity_horizon", 0);

  /* "gauge/deterministic.pyx":385
 * 
 *     cdef inline double _velocity_horizon(self):
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     # RAY
 */
  __pyx_r = 0.0;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":384
 *         return True, self.value
 * 
 *     cdef inline double _velocity_horizon(self):             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":389
 *     # RAY
 * 
 *     cdef inline (bint, double) _get_ray(self, double at):             # <<<<<<<<<<<<<<
 *         cdef double velocity = self.extra
 *         cdef double value = self.value + velocity * (at - self.since)
 */

static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__get_ray(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, double __pyx_v_at) {
  double __pyx_v_velocity;
  double __pyx_v_value;
  __pyx_ctuple_int__and_double __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  __pyx_ctuple_int__and_double __pyx_t_2;
  __Pyx_RefNannySetupContext("_get_ray", 0);

  /* "gauge/deterministic.pyx":390
 * 
 *     cdef inline (bint, double) _get_ray(self, double at):
 *         cdef double velocity = self.extra             # <<<<<<<<<<<<<<
 *         cdef double value = self.value + velocity * (at - self.since)
 *         return True, value
 */
  __pyx_t_1 = __pyx_v_self->extra;
  __pyx_v_velocity = __pyx_t_1;

  /* "gauge/deterministic.pyx":391
 *     cdef inline (bint, double) _get_ray(self, double at):
 *         cdef double velocity = self.extra
 *         cdef double value = self.value + velocity * (at - self.since)             # <<<<<<<<<<<<<<
 *         return True, value
 * 
 */
  __pyx_v_value = (__pyx_v_self->value + (__pyx_v_velocity * (__pyx_v_at - __pyx_v_self->since)));

  /* "gauge/deterministic.pyx":392
 *         cdef double velocity = self.extra
 *         cdef double value = self.value + velocity * (at - self.since)
 *         return True, value             # <<<<<<<<<<<<<<
 * 
 *     cdef inline (bint, double) _earlier_ray(self, double at):
 */
  __pyx_t_2.f0 = 1;
  __pyx_t_2.f1 = __pyx_v_value;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":389
 *     # RAY
 * 
 *     cdef inline (bint, double) _get_ray(self, double at):             # <<<<<<<<<<<<<<
 *         cdef double velocity = self.extra
 *         cdef double value = self.value + velocity * (at - self.since)
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":394
 *         return True, value
 * 
 *     cdef inline (bint, double) _earlier_ray(self, double at):             # <<<<<<<<<<<<<<
 *         return True, self.value
 * 
 */

static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__earlier_ray(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, CYTHON_UNUSED double __pyx_v_at) {
  __pyx_ctuple_int__and_double __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_ctuple_int__and_double __pyx_t_1;
  __Pyx_RefNannySetupContext("_earlier_ray", 0);

  /* "gauge/deterministic.pyx":395
 * 
 *     cdef inline (bint, double) _earlier_ray(self, double at):
 *         return True, self.value             # <<<<<<<<<<<<<<
 * 
 *     cdef inline (bint, double) _later_ray(self, double at):
 */
  __pyx_t_1.f0 = 1;
  __pyx_t_1.f1 = __pyx_v_self->value;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":394
 *         return True, value
 * 
 *     cdef inline (bint, double) _earlier_ray(self, double at):             # <<<<<<<<<<<<<<
 *         return True, self.value
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":397
 *         return True, self.value
 * 
 *     cdef inline (bint, double) _later_ray(self, double at):             # <<<<<<<<<<<<<<
 *         return self._get(self.until)
 * 
 */

static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__later_ray(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, CYTHON_UNUSED double __pyx_v_at) {
  __pyx_ctuple_int__and_double __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_later_ray", 0);

  /* "gauge/deterministic.pyx":398
 * 
 *     cdef inline (bint, double) _later_ray(self, double at):
 *         return self._get(self.until)             # <<<<<<<<<<<<<<
 * 
 *     cdef inline double _velocity_ray(self):
 */
  __pyx_r = ((struct __pyx_vtabstruct_5gauge_13deterministic_Line *)__pyx_v_self->__pyx_vtab)->_get(__pyx_v_self, __pyx_v_self->until);
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":397
 *         return True, self.value
 * 
 *     cdef inline (bint, double) _later_ray(self, double at):             # <<<<<<<<<<<<<<
 *         return self._get(self.until)
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":400
 *         return self._get(self.until)
 * 
 *     cdef inline double _velocity_ray(self):             # <<<<<<<<<<<<<<
 *         return self.extra
 * 
 */

static CYTHON_INLINE double __pyx_f_5gauge_13deterministic_4Line__velocity_ray(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_velocity_ray", 0);

  /* "gauge/deterministic.pyx":401
 * 
 *     cdef inline double _velocity_ray(self):
 *         return self.extra             # <<<<<<<<<<<<<<
 * 
 *     # SEGMENT
 */
  __pyx_r = __pyx_v_self->extra;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":400
 *         return self._get(self.until)
 * 
 *     cdef inline double _velocity_ray(self):             # <<<<<<<<<<<<<<
 *         return self.extra
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":405
 *     # SEGMENT
 * 
 *     cdef inline (bint, double) _get_segment(self, double at):             # <<<<<<<<<<<<<<
 *         cdef double value
 *         cdef double final = self.extra
 */

static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__get_segment(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, double __pyx_v_at) {
  double __pyx_v_value;
  double __pyx_v_final;
  __pyx_ctuple_int__and_double __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  __pyx_ctuple_int__and_double __pyx_t_2;
  __Pyx_RefNannySetupContext("_get_segment", 0);

  /* "gauge/deterministic.pyx":407
 *     cdef inline (bint, double) _get_segment(self, double at):
 *         cdef double value
 *         cdef double final = self.extra             # <<<<<<<<<<<<<<
 *         value = SEGMENT_VALUE(at, self.since, self.until, self.value, final)
 *         return True, value
 */
  __pyx_t_1 = __pyx_v_self->extra;
  __pyx_v_final = __pyx_t_1;

  /* "gauge/deterministic.pyx":408
 *         cdef double value
 *         cdef double final = self.extra
 *         value = SEGMENT_VALUE(at, self.since, self.until, self.value, final)             # <<<<<<<<<<<<<<
 *         return True, value
 * 
 */
  __pyx_v_value = __pyx_f_5gauge_13deterministic_SEGMENT_VALUE(__pyx_v_at, __pyx_v_self->since, __pyx_v_self->until, __pyx_v_self->value, __pyx_v_final);

  /* "gauge/deterministic.pyx":409
 *         cdef double final = self.extra
 *         value = SEGMENT_VALUE(at, self.since, self.until, self.value, final)
 *         return True, value             # <<<<<<<<<<<<<<
 * 
 *     cdef inline (bint, double) _earlier_segment(self, double at):
 */
  __pyx_t_2.f0 = 1;
  __pyx_t_2.f1 = __pyx_v_value;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":405
 *     # SEGMENT
 * 
 *     cdef inline (bint, double) _get_segment(self, double at):             # <<<<<<<<<<<<<<
 *         cdef double value
 *         cdef double final = self.extra
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":411
 *         return True, value
 * 
 *     cdef inline (bint, double) _earlier_segment(self, double at):             # <<<<<<<<<<<<<<
 *         return True, self.value
 * 
 */

static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__earlier_segment(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, CYTHON_UNUSED double __pyx_v_at) {
  __pyx_ctuple_int__and_double __pyx_r;
  __Pyx_RefNannyDeclarations
  __pyx_ctuple_int__and_double __pyx_t_1;
  __Pyx_RefNannySetupContext("_earlier_segment", 0);

  /* "gauge/deterministic.pyx":412
 * 
 *     cdef inline (bint, double) _earlier_segment(self, double at):
 *         return True, self.value             # <<<<<<<<<<<<<<
 * 
 *     cdef inline (bint, double) _later_segment(self, double at):
 */
  __pyx_t_1.f0 = 1;
  __pyx_t_1.f1 = __pyx_v_self->value;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":411
 *         return True, value
 * 
 *     cdef inline (bint, double) _earlier_segment(self, double at):             # <<<<<<<<<<<<<<
 *         return True, self.value
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":414
 *         return True, self.value
 * 
 *     cdef inline (bint, double) _later_segment(self, double at):             # <<<<<<<<<<<<<<
 *         cdef double final = self.extra
 *         return True, final
 */

static CYTHON_INLINE __pyx_ctuple_int__and_double __pyx_f_5gauge_13deterministic_4Line__later_segment(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, CYTHON_UNUSED double __pyx_v_at) {
  double __pyx_v_final;
  __pyx_ctuple_int__and_double __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  __pyx_ctuple_int__and_double __pyx_t_2;
  __Pyx_RefNannySetupContext("_later_segment", 0);

  /* "gauge/deterministic.pyx":415
 * 
 *     cdef inline (bint, double) _later_segment(self, double at):
 *         cdef double final = self.extra             # <<<<<<<<<<<<<<
 *         return True, final
 * 
 */
  __pyx_t_1 = __pyx_v_self->extra;
  __pyx_v_final = __pyx_t_1;

  /* "gauge/deterministic.pyx":416
 *     cdef inline (bint, double) _later_segment(self, double at):
 *         cdef double final = self.extra
 *         return True, final             # <<<<<<<<<<<<<<
 * 
 *     cdef inline double _velocity_segment(self):
 */
  __pyx_t_2.f0 = 1;
  __pyx_t_2.f1 = __pyx_v_final;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":414
 *         return True, self.value
 * 
 *     cdef inline (bint, double) _later_segment(self, double at):             # <<<<<<<<<<<<<<
 *         cdef double final = self.extra
 *         return True, final
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":418
 *         return True, final
 * 
 *     cdef inline double _velocity_segment(self):             # <<<<<<<<<<<<<<
 *         cdef double final = self.extra
 *         return SEGMENT_VELOCITY(self.since, self.until, self.value, final)
 */

static CYTHON_INLINE double __pyx_f_5gauge_13deterministic_4Line__velocity_segment(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self) {
  double __pyx_v_final;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  __Pyx_RefNannySetupContext("_velocity_segment", 0);

  /* "gauge/deterministic.pyx":419
 * 
 *     cdef inline double _velocity_segment(self):
 *         cdef double final = self.extra             # <<<<<<<<<<<<<<
 *         return SEGMENT_VELOCITY(self.since, self.until, self.value, final)
 * 
 */
  __pyx_t_1 = __pyx_v_self->extra;
  __pyx_v_final = __pyx_t_1;

  /* "gauge/deterministic.pyx":420
 *     cdef inline double _velocity_segment(self):
 *         cdef double final = self.extra
 *         return SEGMENT_VELOCITY(self.since, self.until, self.value, final)             # <<<<<<<<<<<<<<
 * 
 *     # __repr__
 */
  __pyx_r = __pyx_f_5gauge_13deterministic_SEGMENT_VELOCITY(__pyx_v_self->since, __pyx_v_self->until, __pyx_v_self->value, __pyx_v_final);
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":418
 *         return True, final
 * 
 *     cdef inline double _velocity_segment(self):             # <<<<<<<<<<<<<<
 *         cdef double final = self.extra
 *         return SEGMENT_VELOCITY(self.since, self.until, self.value, final)
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":424
 *     # __repr__
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         cdef str string
 *         if self.type == LN_HORIZON:
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_13__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_13__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_12__repr__(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_4Line_12__repr__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self) {
  PyObject *__pyx_v_string = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "gauge/deterministic.pyx":426
 *     def __repr__(self):
 *         cdef str string
 *         if self.type == LN_HORIZON:             # <<<<<<<<<<<<<<
 *             string = '[HORIZON] {0:.2f}'.format(self.value)
 *         elif self.type == LN_RAY:  # extra is velocity.
 */
  switch (__pyx_v_self->type) {
    case 1:

    /* "gauge/deterministic.pyx":427
 *         cdef str string
 *         if self.type == LN_HORIZON:
 *             string = '[HORIZON] {0:.2f}'.format(self.value)             # <<<<<<<<<<<<<<
 *         elif self.type == LN_RAY:  # extra is velocity.
 *             string = '[RAY] {0:.2f}{1:+.2f}/s'.format(self.value, self.extra)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_HORIZON_0_2f, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 427, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->value); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 427, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_3};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 427, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_3};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 427, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      } else
      #endif
      {
        __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 427, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_5, 0+1, __pyx_t_3);
        __pyx_t_3 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 427, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 427, __pyx_L1_error)
    __pyx_v_string = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "gauge/deterministic.pyx":426
 *     def __repr__(self):
 *         cdef str string
 *         if self.type == LN_HORIZON:             # <<<<<<<<<<<<<<
 *             string = '[HORIZON] {0:.2f}'.format(self.value)
 *         elif self.type == LN_RAY:  # extra is velocity.
 */
    break;

    /* "gauge/deterministic.pyx":428
 *         if self.type == LN_HORIZON:
 *             string = '[HORIZON] {0:.2f}'.format(self.value)
 *         elif self.type == LN_RAY:  # extra is velocity.             # <<<<<<<<<<<<<<
 *             string = '[RAY] {0:.2f}{1:+.2f}/s'.format(self.value, self.extra)
 *         elif self.type == LN_SEGMENT:  # extra is final.
 */
    case 2:

    /* "gauge/deterministic.pyx":429
 *             string = '[HORIZON] {0:.2f}'.format(self.value)
 *         elif self.type == LN_RAY:  # extra is velocity.
 *             string = '[RAY] {0:.2f}{1:+.2f}/s'.format(self.value, self.extra)             # <<<<<<<<<<<<<<
 *         elif self.type == LN_SEGMENT:  # extra is final.
 *             string = '[SEGMENT] {0:.2f}~{1:.2f}'.format(self.value, self.extra)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_RAY_0_2f_1_2f_s, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 429, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->value); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 429, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->extra); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 429, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_5, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 429, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_5, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 429, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 429, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      if (__pyx_t_4) {
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_4); __pyx_t_4 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_3);
      __pyx_t_5 = 0;
      __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 429, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 429, __pyx_L1_error)
    __pyx_v_string = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "gauge/deterministic.pyx":428
 *         if self.type == LN_HORIZON:
 *             string = '[HORIZON] {0:.2f}'.format(self.value)
 *         elif self.type == LN_RAY:  # extra is velocity.             # <<<<<<<<<<<<<<
 *             string = '[RAY] {0:.2f}{1:+.2f}/s'.format(self.value, self.extra)
 *         elif self.type == LN_SEGMENT:  # extra is final.
 */
    break;

    /* "gauge/deterministic.pyx":430
 *         elif self.type == LN_RAY:  # extra is velocity.
 *             string = '[RAY] {0:.2f}{1:+.2f}/s'.format(self.value, self.extra)
 *         elif self.type == LN_SEGMENT:  # extra is final.             # <<<<<<<<<<<<<<
 *             string = '[SEGMENT] {0:.2f}~{1:.2f}'.format(self.value, self.extra)
 *         else:
 */
    case 3:

    /* "gauge/deterministic.pyx":431
 *             string = '[RAY] {0:.2f}{1:+.2f}/s'.format(self.value, self.extra)
 *         elif self.type == LN_SEGMENT:  # extra is final.
 *             string = '[SEGMENT] {0:.2f}~{1:.2f}'.format(self.value, self.extra)             # <<<<<<<<<<<<<<
 *         else:
 *             assert 0
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_SEGMENT_0_2f_1_2f, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->value); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->extra); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 431, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = NULL;
    __pyx_t_6 = 0;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
        __pyx_t_6 = 1;
      }
    }
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_7, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 431, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_t_7, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 431, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 431, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (__pyx_t_5) {
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
      }
      __Pyx_GIVEREF(__pyx_t_7);
      PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, __pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, __pyx_t_3);
      __pyx_t_7 = 0;
      __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 431, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (!(likely(PyString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 431, __pyx_L1_error)
    __pyx_v_string = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "gauge/deterministic.pyx":430
 *         elif self.type == LN_RAY:  # extra is velocity.
 *             string = '[RAY] {0:.2f}{1:+.2f}/s'.format(self.value, self.extra)
 *         elif self.type == LN_SEGMENT:  # extra is final.             # <<<<<<<<<<<<<<
 *             string = '[SEGMENT] {0:.2f}~{1:.2f}'.format(self.value, self.extra)
 *         else:
 */
    break;
    default:

    /* "gauge/deterministic.pyx":433
 *             string = '[SEGMENT] {0:.2f}~{1:.2f}'.format(self.value, self.extra)
 *         else:
 *             assert 0             # <<<<<<<<<<<<<<
 *         return ('<{0}{1} for {2:.2f}~{3:.2f}>'
 *                 ''.format(CLASS_NAME(self), string, self.since, self.until))
 */
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      if (unlikely(!0)) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 433, __pyx_L1_error)
      }
    }
    #endif
    break;
  }

  /* "gauge/deterministic.pyx":434
 *         else:
 *             assert 0
 *         return ('<{0}{1} for {2:.2f}~{3:.2f}>'             # <<<<<<<<<<<<<<
 *                 ''.format(CLASS_NAME(self), string, self.since, self.until))
 * 
 */
  __Pyx_XDECREF(__pyx_r);

  /* "gauge/deterministic.pyx":435
 *             assert 0
 *         return ('<{0}{1} for {2:.2f}~{3:.2f}>'
 *                 ''.format(CLASS_NAME(self), string, self.since, self.until))             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_0_1_for_2_2f_3_2f, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __pyx_f_5gauge_9constants_CLASS_NAME(((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (unlikely(!__pyx_v_string)) { __Pyx_RaiseUnboundLocalError("string"); __PYX_ERR(0, 435, __pyx_L1_error) }
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->since); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = PyFloat_FromDouble(__pyx_v_self->until); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_5, __pyx_t_4, __pyx_v_string, __pyx_t_3, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 4+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 435, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[5] = {__pyx_t_5, __pyx_t_4, __pyx_v_string, __pyx_t_3, __pyx_t_7};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_6, 4+__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 435, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  } else
  #endif
  {
    __pyx_t_8 = PyTuple_New(4+__pyx_t_6); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 435, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_6, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_string);
    __Pyx_GIVEREF(__pyx_v_string);
    PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_6, __pyx_v_string);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_6, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_8, 3+__pyx_t_6, __pyx_t_7);
    __pyx_t_4 = 0;
    __pyx_t_3 = 0;
    __pyx_t_7 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 435, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":424
 *     # __repr__
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         cdef str string
 *         if self.type == LN_HORIZON:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("gauge.deterministic.Line.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_string);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":245
 * 
 *     cdef:
 *         public int type             # <<<<<<<<<<<<<<
 *         public double since
 *         public double until
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_4type_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_4type_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_4type___get__(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_4Line_4type___get__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 245, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("gauge.deterministic.Line.type.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_4Line_4type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5gauge_13deterministic_4Line_4type_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_4type_2__set__(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_4Line_4type_2__set__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_As_int(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 245, __pyx_L1_error)
  __pyx_v_self->type = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("gauge.deterministic.Line.type.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":246
 *     cdef:
 *         public int type
 *         public double since             # <<<<<<<<<<<<<<
 *         public double until
 *         public double value
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_5since_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_5since_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_5since___get__(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_4Line_5since___get__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->since); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("gauge.deterministic.Line.since.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_4Line_5since_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5gauge_13deterministic_4Line_5since_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_5since_2__set__(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_4Line_5since_2__set__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 246, __pyx_L1_error)
  __pyx_v_self->since = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("gauge.deterministic.Line.since.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":247
 *         public int type
 *         public double since
 *         public double until             # <<<<<<<<<<<<<<
 *         public double value
 *         public double extra
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_5until_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_5until_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_5until___get__(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_4Line_5until___get__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->until); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("gauge.deterministic.Line.until.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_4Line_5until_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5gauge_13deterministic_4Line_5until_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_5until_2__set__(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_4Line_5until_2__set__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 247, __pyx_L1_error)
  __pyx_v_self->until = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("gauge.deterministic.Line.until.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":248
 *         public double since
 *         public double until
 *         public double value             # <<<<<<<<<<<<<<
 *         public double extra
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_5value_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_5value_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_5value___get__(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_4Line_5value___get__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("gauge.deterministic.Line.value.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_4Line_5value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5gauge_13deterministic_4Line_5value_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_5value_2__set__(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_4Line_5value_2__set__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 248, __pyx_L1_error)
  __pyx_v_self->value = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("gauge.deterministic.Line.value.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":249
 *         public double until
 *         public double value
 *         public double extra             # <<<<<<<<<<<<<<
 * 
 *     def __cinit__(self, int type,
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_5extra_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_4Line_5extra_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_5extra___get__(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_4Line_5extra___get__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->extra); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("gauge.deterministic.Line.extra.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_4Line_5extra_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5gauge_13deterministic_4Line_5extra_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Line_5extra_2__set__(((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_4Line_5extra_2__set__(struct __pyx_obj_5gauge_13deterministic_Line *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 249, __pyx_L1_error)
  __pyx_v_self->extra = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("gauge.deterministic.Line.extra.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":438
 * 
 * 
 * cpdef Line Horizon(double since, double until, double value):             # <<<<<<<<<<<<<<
 *     return Line(LN_HORIZON, since, until, value)
 * 
 */

static PyObject *__pyx_pw_5gauge_13deterministic_1Horizon(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_5gauge_13deterministic_Line *__pyx_f_5gauge_13deterministic_Horizon(double __pyx_v_since, double __pyx_v_until, double __pyx_v_value, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_5gauge_13deterministic_Line *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("Horizon", 0);

  /* "gauge/deterministic.pyx":439
 * 
 * cpdef Line Horizon(double since, double until, double value):
 *     return Line(LN_HORIZON, since, until, value)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_since); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_until); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_int_1);
  __Pyx_GIVEREF(__pyx_int_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_int_1);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5gauge_13deterministic_Line), __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 439, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":438
 * 
 * 
 * cpdef Line Horizon(double since, double until, double value):             # <<<<<<<<<<<<<<
 *     return Line(LN_HORIZON, since, until, value)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("gauge.deterministic.Horizon", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_1Horizon(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_1Horizon(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_since;
  double __pyx_v_until;
  double __pyx_v_value;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("Horizon (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_since,&__pyx_n_s_until,&__pyx_n_s_value,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_since)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_until)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("Horizon", 1, 3, 3, 1); __PYX_ERR(0, 438, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("Horizon", 1, 3, 3, 2); __PYX_ERR(0, 438, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "Horizon") < 0)) __PYX_ERR(0, 438, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_since = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_since == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 438, __pyx_L3_error)
    __pyx_v_until = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_until == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 438, __pyx_L3_error)
    __pyx_v_value = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_value == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 438, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("Horizon", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 438, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("gauge.deterministic.Horizon", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5gauge_13deterministic_Horizon(__pyx_self, __pyx_v_since, __pyx_v_until, __pyx_v_value);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_Horizon(CYTHON_UNUSED PyObject *__pyx_self, double __pyx_v_since, double __pyx_v_until, double __pyx_v_value) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("Horizon", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5gauge_13deterministic_Horizon(__pyx_v_since, __pyx_v_until, __pyx_v_value, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("gauge.deterministic.Horizon", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":442
 * 
 * 
 * cpdef Line Ray(double since, double until, double value, double velocity):             # <<<<<<<<<<<<<<
 *     return Line(LN_RAY, since, until, value, velocity)
 * 
 */

static PyObject *__pyx_pw_5gauge_13deterministic_3Ray(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_5gauge_13deterministic_Line *__pyx_f_5gauge_13deterministic_Ray(double __pyx_v_since, double __pyx_v_until, double __pyx_v_value, double __pyx_v_velocity, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_5gauge_13deterministic_Line *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("Ray", 0);

  /* "gauge/deterministic.pyx":443
 * 
 * cpdef Line Ray(double since, double until, double value, double velocity):
 *     return Line(LN_RAY, since, until, value, velocity)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_since); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_until); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_velocity); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_int_2);
  __Pyx_GIVEREF(__pyx_int_2);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_int_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5gauge_13deterministic_Line), __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 443, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_r = ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_t_4);
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":442
 * 
 * 
 * cpdef Line Ray(double since, double until, double value, double velocity):             # <<<<<<<<<<<<<<
 *     return Line(LN_RAY, since, until, value, velocity)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("gauge.deterministic.Ray", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_3Ray(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_3Ray(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_since;
  double __pyx_v_until;
  double __pyx_v_value;
  double __pyx_v_velocity;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("Ray (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_since,&__pyx_n_s_until,&__pyx_n_s_value,&__pyx_n_s_velocity,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_since)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_until)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("Ray", 1, 4, 4, 1); __PYX_ERR(0, 442, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("Ray", 1, 4, 4, 2); __PYX_ERR(0, 442, __pyx_L3_error)
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_velocity)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("Ray", 1, 4, 4, 3); __PYX_ERR(0, 442, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "Ray") < 0)) __PYX_ERR(0, 442, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_since = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_since == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 442, __pyx_L3_error)
    __pyx_v_until = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_until == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 442, __pyx_L3_error)
    __pyx_v_value = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_value == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 442, __pyx_L3_error)
    __pyx_v_velocity = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_velocity == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 442, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("Ray", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 442, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("gauge.deterministic.Ray", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5gauge_13deterministic_2Ray(__pyx_self, __pyx_v_since, __pyx_v_until, __pyx_v_value, __pyx_v_velocity);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_2Ray(CYTHON_UNUSED PyObject *__pyx_self, double __pyx_v_since, double __pyx_v_until, double __pyx_v_value, double __pyx_v_velocity) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("Ray", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5gauge_13deterministic_Ray(__pyx_v_since, __pyx_v_until, __pyx_v_value, __pyx_v_velocity, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("gauge.deterministic.Ray", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":446
 * 
 * 
 * cpdef Line Segment(double since, double until, double value, double final):             # <<<<<<<<<<<<<<
 *     return Line(LN_SEGMENT, since, until, value, final)
 * 
 */

static PyObject *__pyx_pw_5gauge_13deterministic_5Segment(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_5gauge_13deterministic_Line *__pyx_f_5gauge_13deterministic_Segment(double __pyx_v_since, double __pyx_v_until, double __pyx_v_value, double __pyx_v_final, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_5gauge_13deterministic_Line *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("Segment", 0);

  /* "gauge/deterministic.pyx":447
 * 
 * cpdef Line Segment(double since, double until, double value, double final):
 *     return Line(LN_SEGMENT, since, until, value, final)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_since); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_until); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_final); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_int_3);
  __Pyx_GIVEREF(__pyx_int_3);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_int_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_t_4);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5gauge_13deterministic_Line), __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 447, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_r = ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_t_4);
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":446
 * 
 * 
 * cpdef Line Segment(double since, double until, double value, double final):             # <<<<<<<<<<<<<<
 *     return Line(LN_SEGMENT, since, until, value, final)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("gauge.deterministic.Segment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_5Segment(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_5Segment(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_since;
  double __pyx_v_until;
  double __pyx_v_value;
  double __pyx_v_final;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("Segment (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_since,&__pyx_n_s_until,&__pyx_n_s_value,&__pyx_n_s_final,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_since)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_until)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("Segment", 1, 4, 4, 1); __PYX_ERR(0, 446, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("Segment", 1, 4, 4, 2); __PYX_ERR(0, 446, __pyx_L3_error)
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_final)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("Segment", 1, 4, 4, 3); __PYX_ERR(0, 446, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "Segment") < 0)) __PYX_ERR(0, 446, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_since = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_since == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L3_error)
    __pyx_v_until = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_until == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L3_error)
    __pyx_v_value = __pyx_PyFloat_AsDouble(values[2]); if (unlikely((__pyx_v_value == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L3_error)
    __pyx_v_final = __pyx_PyFloat_AsDouble(values[3]); if (unlikely((__pyx_v_final == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 446, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("Segment", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 446, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("gauge.deterministic.Segment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5gauge_13deterministic_4Segment(__pyx_self, __pyx_v_since, __pyx_v_until, __pyx_v_value, __pyx_v_final);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_4Segment(CYTHON_UNUSED PyObject *__pyx_self, double __pyx_v_since, double __pyx_v_until, double __pyx_v_value, double __pyx_v_final) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("Segment", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_5gauge_13deterministic_Segment(__pyx_v_since, __pyx_v_until, __pyx_v_value, __pyx_v_final, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("gauge.deterministic.Segment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":458
 *         public best
 * 
 *     def __init__(self, list lines, cmp=operator.lt):             # <<<<<<<<<<<<<<
 *         assert cmp in [operator.lt, operator.gt]
 *         self.lines_iter = iter(lines)
 */

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_8Boundary_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_5gauge_13deterministic_8Boundary_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_lines = 0;
  PyObject *__pyx_v_cmp = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_lines,&__pyx_n_s_cmp,0};
    PyObject* values[2] = {0,0};
    values[1] = __pyx_k__5;
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_lines)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_cmp);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 458, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_lines = ((PyObject*)values[0]);
    __pyx_v_cmp = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 458, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("gauge.deterministic.Boundary.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_lines), (&PyList_Type), 1, "lines", 1))) __PYX_ERR(0, 458, __pyx_L1_error)
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary___init__(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self), __pyx_v_lines, __pyx_v_cmp);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_8Boundary___init__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, PyObject *__pyx_v_lines, PyObject *__pyx_v_cmp) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "gauge/deterministic.pyx":459
 * 
 *     def __init__(self, list lines, cmp=operator.lt):
 *         assert cmp in [operator.lt, operator.gt]             # <<<<<<<<<<<<<<
 *         self.lines_iter = iter(lines)
 *         self.cmp = cmp
 */
  #ifndef CYTHON_WITHOUT_ASSERTIONS
  if (unlikely(!Py_OptimizeFlag)) {
    __Pyx_INCREF(__pyx_v_cmp);
    __pyx_t_1 = __pyx_v_cmp;
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_operator); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_lt); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!__pyx_t_5) {
    } else {
      __pyx_t_2 = __pyx_t_5;
      goto __pyx_L3_bool_binop_done;
    }
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_operator); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_gt); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 459, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_2 = __pyx_t_5;
    __pyx_L3_bool_binop_done:;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!(__pyx_t_2 != 0))) {
      PyErr_SetNone(PyExc_AssertionError);
      __PYX_ERR(0, 459, __pyx_L1_error)
    }
  }
  #endif

  /* "gauge/deterministic.pyx":460
 *     def __init__(self, list lines, cmp=operator.lt):
 *         assert cmp in [operator.lt, operator.gt]
 *         self.lines_iter = iter(lines)             # <<<<<<<<<<<<<<
 *         self.cmp = cmp
 *         self.best = {operator.lt: min, operator.gt: max}[cmp]
 */
  __pyx_t_1 = PyObject_GetIter(__pyx_v_lines); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 460, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->lines_iter);
  __Pyx_DECREF(__pyx_v_self->lines_iter);
  __pyx_v_self->lines_iter = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "gauge/deterministic.pyx":461
 *         assert cmp in [operator.lt, operator.gt]
 *         self.lines_iter = iter(lines)
 *         self.cmp = cmp             # <<<<<<<<<<<<<<
 *         self.best = {operator.lt: min, operator.gt: max}[cmp]
 *         self.walk()
 */
  __Pyx_INCREF(__pyx_v_cmp);
  __Pyx_GIVEREF(__pyx_v_cmp);
  __Pyx_GOTREF(__pyx_v_self->cmp);
  __Pyx_DECREF(__pyx_v_self->cmp);
  __pyx_v_self->cmp = __pyx_v_cmp;

  /* "gauge/deterministic.pyx":462
 *         self.lines_iter = iter(lines)
 *         self.cmp = cmp
 *         self.best = {operator.lt: min, operator.gt: max}[cmp]             # <<<<<<<<<<<<<<
 *         self.walk()
 * 
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_operator); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_lt); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_t_4, __pyx_builtin_min) < 0) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_operator); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_gt); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_t_1, __pyx_t_3, __pyx_builtin_max) < 0) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyDict_GetItem(__pyx_t_1, __pyx_v_cmp); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->best);
  __Pyx_DECREF(__pyx_v_self->best);
  __pyx_v_self->best = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "gauge/deterministic.pyx":463
 *         self.cmp = cmp
 *         self.best = {operator.lt: min, operator.gt: max}[cmp]
 *         self.walk()             # <<<<<<<<<<<<<<
 * 
 *     cpdef walk(self):
 */
  __pyx_t_3 = ((struct __pyx_vtabstruct_5gauge_13deterministic_Boundary *)__pyx_v_self->__pyx_vtab)->walk(__pyx_v_self, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 463, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "gauge/deterministic.pyx":458
 *         public best
 * 
 *     def __init__(self, list lines, cmp=operator.lt):             # <<<<<<<<<<<<<<
 *         assert cmp in [operator.lt, operator.gt]
 *         self.lines_iter = iter(lines)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("gauge.deterministic.Boundary.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":465
 *         self.walk()
 * 
 *     cpdef walk(self):             # <<<<<<<<<<<<<<
 *         """Choose the next line."""
 *         self.line = next(self.lines_iter)
 */

static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_3walk(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_5gauge_13deterministic_8Boundary_walk(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("walk", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_walk); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5gauge_13deterministic_8Boundary_3walk)) {
      __Pyx_XDECREF(__pyx_r);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_4)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_4);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      if (__pyx_t_4) {
        __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 465, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 465, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "gauge/deterministic.pyx":467
 *     cpdef walk(self):
 *         """Choose the next line."""
 *         self.line = next(self.lines_iter)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint cmp_eq(self, double x, double y):
 */
  __pyx_t_1 = __pyx_v_self->lines_iter;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyIter_Next(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5gauge_13deterministic_Line))))) __PYX_ERR(0, 467, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->line);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->line));
  __pyx_v_self->line = ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "gauge/deterministic.pyx":465
 *         self.walk()
 * 
 *     cpdef walk(self):             # <<<<<<<<<<<<<<
 *         """Choose the next line."""
 *         self.line = next(self.lines_iter)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("gauge.deterministic.Boundary.walk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_3walk(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_5gauge_13deterministic_8Boundary_2walk[] = "Choose the next line.";
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_3walk(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("walk (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_2walk(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_2walk(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("walk", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_5gauge_13deterministic_8Boundary_walk(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("gauge.deterministic.Boundary.walk", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":469
 *         self.line = next(self.lines_iter)
 * 
 *     cpdef bint cmp_eq(self, double x, double y):             # <<<<<<<<<<<<<<
 *         return x == y or self.cmp(x, y)
 * 
 */

static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_5cmp_eq(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_f_5gauge_13deterministic_8Boundary_cmp_eq(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  __Pyx_RefNannySetupContext("cmp_eq", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_cmp_eq); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 469, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5gauge_13deterministic_8Boundary_5cmp_eq)) {
      __pyx_t_3 = PyFloat_FromDouble(__pyx_v_x); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 469, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyFloat_FromDouble(__pyx_v_y); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 469, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_5 = __pyx_t_1; __pyx_t_6 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 469, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 469, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 469, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        if (__pyx_t_6) {
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
        __pyx_t_3 = 0;
        __pyx_t_4 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 469, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 469, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_r = __pyx_t_9;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "gauge/deterministic.pyx":470
 * 
 *     cpdef bint cmp_eq(self, double x, double y):
 *         return x == y or self.cmp(x, y)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint cmp_inv(self, double x, double y):
 */
  __pyx_t_10 = ((__pyx_v_x == __pyx_v_y) != 0);
  if (!__pyx_t_10) {
  } else {
    __pyx_t_9 = __pyx_t_10;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_x); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_y); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_self->cmp);
  __pyx_t_8 = __pyx_v_self->cmp; __pyx_t_4 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_8)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_2, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 470, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_2, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 470, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 470, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_7, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_7, __pyx_t_5);
    __pyx_t_2 = 0;
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 470, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_9 = __pyx_t_10;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_9;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":469
 *         self.line = next(self.lines_iter)
 * 
 *     cpdef bint cmp_eq(self, double x, double y):             # <<<<<<<<<<<<<<
 *         return x == y or self.cmp(x, y)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_WriteUnraisable("gauge.deterministic.Boundary.cmp_eq", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_5cmp_eq(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_5cmp_eq(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_x;
  double __pyx_v_y;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cmp_eq (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("cmp_eq", 1, 2, 2, 1); __PYX_ERR(0, 469, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "cmp_eq") < 0)) __PYX_ERR(0, 469, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_x = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_x == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 469, __pyx_L3_error)
    __pyx_v_y = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_y == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 469, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("cmp_eq", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 469, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("gauge.deterministic.Boundary.cmp_eq", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_4cmp_eq(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self), __pyx_v_x, __pyx_v_y);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_4cmp_eq(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, double __pyx_v_x, double __pyx_v_y) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("cmp_eq", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_5gauge_13deterministic_8Boundary_cmp_eq(__pyx_v_self, __pyx_v_x, __pyx_v_y, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("gauge.deterministic.Boundary.cmp_eq", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":472
 *         return x == y or self.cmp(x, y)
 * 
 *     cpdef bint cmp_inv(self, double x, double y):             # <<<<<<<<<<<<<<
 *         return x != y and not self.cmp(x, y)
 * 
 */

static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_7cmp_inv(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_f_5gauge_13deterministic_8Boundary_cmp_inv(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, double __pyx_v_x, double __pyx_v_y, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  __Pyx_RefNannySetupContext("cmp_inv", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0)) {
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_cmp_inv); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 472, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)__pyx_pw_5gauge_13deterministic_8Boundary_7cmp_inv)) {
      __pyx_t_3 = PyFloat_FromDouble(__pyx_v_x); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 472, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = PyFloat_FromDouble(__pyx_v_y); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 472, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_5 = __pyx_t_1; __pyx_t_6 = NULL;
      __pyx_t_7 = 0;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_7 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 472, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_6, __pyx_t_3, __pyx_t_4};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 472, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_8 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 472, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        if (__pyx_t_6) {
          __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
        }
        __Pyx_GIVEREF(__pyx_t_3);
        PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
        __pyx_t_3 = 0;
        __pyx_t_4 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 472, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_9 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 472, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_r = __pyx_t_9;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L0;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }

  /* "gauge/deterministic.pyx":473
 * 
 *     cpdef bint cmp_inv(self, double x, double y):
 *         return x != y and not self.cmp(x, y)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_t_10 = ((__pyx_v_x != __pyx_v_y) != 0);
  if (__pyx_t_10) {
  } else {
    __pyx_t_9 = __pyx_t_10;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_x); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 473, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_y); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 473, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_self->cmp);
  __pyx_t_8 = __pyx_v_self->cmp; __pyx_t_4 = NULL;
  __pyx_t_7 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_8);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_8, function);
      __pyx_t_7 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_8)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_2, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 473, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_t_2, __pyx_t_5};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-__pyx_t_7, 2+__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 473, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  } else
  #endif
  {
    __pyx_t_3 = PyTuple_New(2+__pyx_t_7); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 473, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0+__pyx_t_7, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 1+__pyx_t_7, __pyx_t_5);
    __pyx_t_2 = 0;
    __pyx_t_5 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 473, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 473, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_11 = ((!__pyx_t_10) != 0);
  __pyx_t_9 = __pyx_t_11;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_9;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":472
 *         return x == y or self.cmp(x, y)
 * 
 *     cpdef bint cmp_inv(self, double x, double y):             # <<<<<<<<<<<<<<
 *         return x != y and not self.cmp(x, y)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_WriteUnraisable("gauge.deterministic.Boundary.cmp_inv", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_7cmp_inv(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_7cmp_inv(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  double __pyx_v_x;
  double __pyx_v_y;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("cmp_inv (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("cmp_inv", 1, 2, 2, 1); __PYX_ERR(0, 472, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "cmp_inv") < 0)) __PYX_ERR(0, 472, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_x = __pyx_PyFloat_AsDouble(values[0]); if (unlikely((__pyx_v_x == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 472, __pyx_L3_error)
    __pyx_v_y = __pyx_PyFloat_AsDouble(values[1]); if (unlikely((__pyx_v_y == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 472, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("cmp_inv", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 472, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("gauge.deterministic.Boundary.cmp_inv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_6cmp_inv(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self), __pyx_v_x, __pyx_v_y);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_6cmp_inv(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, double __pyx_v_x, double __pyx_v_y) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("cmp_inv", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_5gauge_13deterministic_8Boundary_cmp_inv(__pyx_v_self, __pyx_v_x, __pyx_v_y, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 472, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("gauge.deterministic.Boundary.cmp_inv", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":475
 *         return x != y and not self.cmp(x, y)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<{0} line={1}, cmp={2}>'.format(CLASS_NAME(self),
 *                                                 self.line, self.cmp)
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_9__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_9__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_8__repr__(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_8__repr__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "gauge/deterministic.pyx":476
 * 
 *     def __repr__(self):
 *         return '<{0} line={1}, cmp={2}>'.format(CLASS_NAME(self),             # <<<<<<<<<<<<<<
 *                                                 self.line, self.cmp)
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_0_line_1_cmp_2, __pyx_n_s_format); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __pyx_f_5gauge_9constants_CLASS_NAME(((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "gauge/deterministic.pyx":477
 *     def __repr__(self):
 *         return '<{0} line={1}, cmp={2}>'.format(CLASS_NAME(self),
 *                                                 self.line, self.cmp)             # <<<<<<<<<<<<<<
 */
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_t_3, ((PyObject *)__pyx_v_self->line), __pyx_v_self->cmp};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 476, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[4] = {__pyx_t_4, __pyx_t_3, ((PyObject *)__pyx_v_self->line), __pyx_v_self->cmp};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 3+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 476, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(3+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 476, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_t_3);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->line));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self->line));
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, ((PyObject *)__pyx_v_self->line));
    __Pyx_INCREF(__pyx_v_self->cmp);
    __Pyx_GIVEREF(__pyx_v_self->cmp);
    PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_self->cmp);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 476, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "gauge/deterministic.pyx":475
 *         return x != y and not self.cmp(x, y)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<{0} line={1}, cmp={2}>'.format(CLASS_NAME(self),
 *                                                 self.line, self.cmp)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("gauge.deterministic.Boundary.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":453
 * 
 *     cdef:
 *         public Line line             # <<<<<<<<<<<<<<
 *         public lines_iter
 *         public cmp
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_4line_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_4line_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_4line___get__(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_4line___get__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->line));
  __pyx_r = ((PyObject *)__pyx_v_self->line);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_8Boundary_4line_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5gauge_13deterministic_8Boundary_4line_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_4line_2__set__(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_8Boundary_4line_2__set__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_5gauge_13deterministic_Line))))) __PYX_ERR(0, 453, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->line);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->line));
  __pyx_v_self->line = ((struct __pyx_obj_5gauge_13deterministic_Line *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("gauge.deterministic.Boundary.line.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_8Boundary_4line_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5gauge_13deterministic_8Boundary_4line_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_4line_4__del__(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_8Boundary_4line_4__del__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->line);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->line));
  __pyx_v_self->line = ((struct __pyx_obj_5gauge_13deterministic_Line *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":454
 *     cdef:
 *         public Line line
 *         public lines_iter             # <<<<<<<<<<<<<<
 *         public cmp
 *         public best
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_10lines_iter_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_10lines_iter_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_10lines_iter___get__(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_10lines_iter___get__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->lines_iter);
  __pyx_r = __pyx_v_self->lines_iter;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_8Boundary_10lines_iter_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5gauge_13deterministic_8Boundary_10lines_iter_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_10lines_iter_2__set__(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_8Boundary_10lines_iter_2__set__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->lines_iter);
  __Pyx_DECREF(__pyx_v_self->lines_iter);
  __pyx_v_self->lines_iter = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_8Boundary_10lines_iter_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5gauge_13deterministic_8Boundary_10lines_iter_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_10lines_iter_4__del__(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_8Boundary_10lines_iter_4__del__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->lines_iter);
  __Pyx_DECREF(__pyx_v_self->lines_iter);
  __pyx_v_self->lines_iter = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":455
 *         public Line line
 *         public lines_iter
 *         public cmp             # <<<<<<<<<<<<<<
 *         public best
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_3cmp_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_3cmp_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_3cmp___get__(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_3cmp___get__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->cmp);
  __pyx_r = __pyx_v_self->cmp;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_8Boundary_3cmp_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5gauge_13deterministic_8Boundary_3cmp_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_3cmp_2__set__(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_8Boundary_3cmp_2__set__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->cmp);
  __Pyx_DECREF(__pyx_v_self->cmp);
  __pyx_v_self->cmp = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_8Boundary_3cmp_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5gauge_13deterministic_8Boundary_3cmp_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_3cmp_4__del__(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_8Boundary_3cmp_4__del__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->cmp);
  __Pyx_DECREF(__pyx_v_self->cmp);
  __pyx_v_self->cmp = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pyx":456
 *         public lines_iter
 *         public cmp
 *         public best             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, list lines, cmp=operator.lt):
 */

/* Python wrapper */
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_4best_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_5gauge_13deterministic_8Boundary_4best_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_4best___get__(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_5gauge_13deterministic_8Boundary_4best___get__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->best);
  __pyx_r = __pyx_v_self->best;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_8Boundary_4best_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_5gauge_13deterministic_8Boundary_4best_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_4best_2__set__(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_8Boundary_4best_2__set__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->best);
  __Pyx_DECREF(__pyx_v_self->best);
  __pyx_v_self->best = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_5gauge_13deterministic_8Boundary_4best_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_5gauge_13deterministic_8Boundary_4best_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_5gauge_13deterministic_8Boundary_4best_4__del__(((struct __pyx_obj_5gauge_13deterministic_Boundary *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_5gauge_13deterministic_8Boundary_4best_4__del__(struct __pyx_obj_5gauge_13deterministic_Boundary *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->best);
  __Pyx_DECREF(__pyx_v_self->best);
  __pyx_v_self->best = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pxd":14
 * 
 * 
 * cdef inline double SEGMENT_VALUE(double at,             # <<<<<<<<<<<<<<
 *                                  double time1, double time2,
 *                                  double value1, double value2):
 */

static CYTHON_INLINE double __pyx_f_5gauge_13deterministic_SEGMENT_VALUE(double __pyx_v_at, double __pyx_v_time1, double __pyx_v_time2, double __pyx_v_value1, double __pyx_v_value2) {
  double __pyx_v_rate;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  double __pyx_t_2;
  __Pyx_RefNannySetupContext("SEGMENT_VALUE", 0);

  /* "gauge/deterministic.pxd":18
 *                                  double value1, double value2):
 *     cdef double rate
 *     if at == time1:             # <<<<<<<<<<<<<<
 *         return value1
 *     elif at == time2:
 */
  __pyx_t_1 = ((__pyx_v_at == __pyx_v_time1) != 0);
  if (__pyx_t_1) {

    /* "gauge/deterministic.pxd":19
 *     cdef double rate
 *     if at == time1:
 *         return value1             # <<<<<<<<<<<<<<
 *     elif at == time2:
 *         return value2
 */
    __pyx_r = __pyx_v_value1;
    goto __pyx_L0;

    /* "gauge/deterministic.pxd":18
 *                                  double value1, double value2):
 *     cdef double rate
 *     if at == time1:             # <<<<<<<<<<<<<<
 *         return value1
 *     elif at == time2:
 */
  }

  /* "gauge/deterministic.pxd":20
 *     if at == time1:
 *         return value1
 *     elif at == time2:             # <<<<<<<<<<<<<<
 *         return value2
 *     else:
 */
  __pyx_t_1 = ((__pyx_v_at == __pyx_v_time2) != 0);
  if (__pyx_t_1) {

    /* "gauge/deterministic.pxd":21
 *         return value1
 *     elif at == time2:
 *         return value2             # <<<<<<<<<<<<<<
 *     else:
 *         rate = float(at - time1) / (time2 - time1)
 */
    __pyx_r = __pyx_v_value2;
    goto __pyx_L0;

    /* "gauge/deterministic.pxd":20
 *     if at == time1:
 *         return value1
 *     elif at == time2:             # <<<<<<<<<<<<<<
 *         return value2
 *     else:
 */
  }

  /* "gauge/deterministic.pxd":23
 *         return value2
 *     else:
 *         rate = float(at - time1) / (time2 - time1)             # <<<<<<<<<<<<<<
 *         return value1 + rate * (value2 - value1)
 * 
 */
  /*else*/ {
    __pyx_t_2 = (__pyx_v_time2 - __pyx_v_time1);
    if (unlikely(__pyx_t_2 == 0)) {
      PyErr_SetString(PyExc_ZeroDivisionError, "float division");
      __PYX_ERR(1, 23, __pyx_L1_error)
    }
    __pyx_v_rate = ((__pyx_v_at - __pyx_v_time1) / __pyx_t_2);

    /* "gauge/deterministic.pxd":24
 *     else:
 *         rate = float(at - time1) / (time2 - time1)
 *         return value1 + rate * (value2 - value1)             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __pyx_r = (__pyx_v_value1 + (__pyx_v_rate * (__pyx_v_value2 - __pyx_v_value1)));
    goto __pyx_L0;
  }

  /* "gauge/deterministic.pxd":14
 * 
 * 
 * cdef inline double SEGMENT_VALUE(double at,             # <<<<<<<<<<<<<<
 *                                  double time1, double time2,
 *                                  double value1, double value2):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("gauge.deterministic.SEGMENT_VALUE", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "gauge/deterministic.pxd":27
 * 
 * 
 * cdef inline double SEGMENT_VELOCITY(double time1, double time2,             # <<<<<<<<<<<<<<
 *                                     double value1, double value2):
 *     return (value2 - value1) / (time2 - time1)
 */

static CYTHON_INLINE double __pyx_f_5gauge_13deterministic_SEGMENT_VELOCITY(double __pyx_v_time1, double __pyx_v_time2, double __pyx_v_value1, double __pyx_v_value2) {
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  double __pyx_t_2;
  __Pyx_RefNannySetupContext("SEGMENT_VELOCITY", 0);

  /* "gauge/deterministic.pxd":29
 * cdef inline double SEGMENT_VELOCITY(double time1, double time2,
 *                                     double value1, double value2):
 *     return (value2 - value1) / (time2 - time1)             # <<<<<<<<<<<<<<
 */
  __pyx_t_1 = (__pyx_v_value2 - __pyx_v_value1);
  __pyx_t_2 = (__pyx_v_time2 - __pyx_v_time1);
  if (unlikely(__pyx_t_2 == 0)) {
    PyErr_SetString(PyExc_ZeroDivisionError, "float division");
    __PYX_ERR(1, 29, __pyx_L1_error)
  }
  __pyx_r = (__pyx_t_1 / __pyx_t_2);
  goto __pyx_L0;

  /* "gauge/deterministic.pxd":27
 * 
 * 
 * cdef inline double SEGMENT_VELOCITY(double time1, double time2,             # <<<<<<<<<<<<<<
 *                                     double value1, double value2):
 *     return (value2 - value1) / (time2 - time1)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("gauge.deterministic.SEGMENT_VELOCITY", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_5gauge_13deterministic_Determination __pyx_vtable_5gauge_13deterministic_Determination;

static PyObject *__pyx_tp_new_5gauge_13deterministic_Determination(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5gauge_13deterministic_Determination *p;
  PyObject *o = (&PyList_Type)->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5gauge_13deterministic_Determination *)o);
  p->__pyx_vtab = __pyx_vtabptr_5gauge_13deterministic_Determination;
  return o;
}

static void __pyx_tp_dealloc_5gauge_13deterministic_Determination(PyObject *o) {
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  PyObject_GC_Track(o);
  (&PyList_Type)->tp_dealloc(o);
}

static int __pyx_tp_traverse_5gauge_13deterministic_Determination(PyObject *o, visitproc v, void *a) {
  int e;
  if (!(&PyList_Type)->tp_traverse); else { e = (&PyList_Type)->tp_traverse(o,v,a); if (e) return e; }
  return 0;
}

static int __pyx_tp_clear_5gauge_13deterministic_Determination(PyObject *o) {
  if (!(&PyList_Type)->tp_clear); else (&PyList_Type)->tp_clear(o);
  return 0;
}

static PyObject *__pyx_getprop_5gauge_13deterministic_13Determination_in_range_since(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5gauge_13deterministic_13Determination_14in_range_since_1__get__(o);
}

static PyMethodDef __pyx_methods_5gauge_13deterministic_Determination[] = {
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5gauge_13deterministic_Determination[] = {
  {(char *)"in_range_since", __pyx_getprop_5gauge_13deterministic_13Determination_in_range_since, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5gauge_13deterministic_Determination = {
  PyVarObject_HEAD_INIT(0, 0)
  "gauge.deterministic.Determination", /*tp_name*/
  sizeof(struct __pyx_obj_5gauge_13deterministic_Determination), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5gauge_13deterministic_Determination, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "Determination of a gauge is a list of `(time, value)` pairs.\n\n    :param determining: a :meth:`Gauge.determine` iterator.\n\n    ", /*tp_doc*/
  __pyx_tp_traverse_5gauge_13deterministic_Determination, /*tp_traverse*/
  __pyx_tp_clear_5gauge_13deterministic_Determination, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5gauge_13deterministic_Determination, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5gauge_13deterministic_Determination, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5gauge_13deterministic_13Determination_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5gauge_13deterministic_Determination, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5gauge_13deterministic_Line __pyx_vtable_5gauge_13deterministic_Line;

static PyObject *__pyx_tp_new_5gauge_13deterministic_Line(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_5gauge_13deterministic_Line *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5gauge_13deterministic_Line *)o);
  p->__pyx_vtab = __pyx_vtabptr_5gauge_13deterministic_Line;
  if (unlikely(__pyx_pw_5gauge_13deterministic_4Line_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_5gauge_13deterministic_Line(PyObject *o) {
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_5gauge_13deterministic_4Line_type(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5gauge_13deterministic_4Line_4type_1__get__(o);
}

static int __pyx_setprop_5gauge_13deterministic_4Line_type(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5gauge_13deterministic_4Line_4type_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5gauge_13deterministic_4Line_since(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5gauge_13deterministic_4Line_5since_1__get__(o);
}

static int __pyx_setprop_5gauge_13deterministic_4Line_since(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5gauge_13deterministic_4Line_5since_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5gauge_13deterministic_4Line_until(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5gauge_13deterministic_4Line_5until_1__get__(o);
}

static int __pyx_setprop_5gauge_13deterministic_4Line_until(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5gauge_13deterministic_4Line_5until_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5gauge_13deterministic_4Line_value(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5gauge_13deterministic_4Line_5value_1__get__(o);
}

static int __pyx_setprop_5gauge_13deterministic_4Line_value(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5gauge_13deterministic_4Line_5value_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_5gauge_13deterministic_4Line_extra(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5gauge_13deterministic_4Line_5extra_1__get__(o);
}

static int __pyx_setprop_5gauge_13deterministic_4Line_extra(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5gauge_13deterministic_4Line_5extra_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_5gauge_13deterministic_Line[] = {
  {"intersect", (PyCFunction)__pyx_pw_5gauge_13deterministic_4Line_3intersect, METH_O, 0},
  {"intercept", (PyCFunction)__pyx_pw_5gauge_13deterministic_4Line_5intercept, METH_NOARGS, __pyx_doc_5gauge_13deterministic_4Line_4intercept},
  {"get", (PyCFunction)__pyx_pw_5gauge_13deterministic_4Line_7get, METH_O, __pyx_doc_5gauge_13deterministic_4Line_6get},
  {"guess", (PyCFunction)__pyx_pw_5gauge_13deterministic_4Line_9guess, METH_O, 0},
  {"velocity", (PyCFunction)__pyx_pw_5gauge_13deterministic_4Line_11velocity, METH_NOARGS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5gauge_13deterministic_Line[] = {
  {(char *)"type", __pyx_getprop_5gauge_13deterministic_4Line_type, __pyx_setprop_5gauge_13deterministic_4Line_type, (char *)0, 0},
  {(char *)"since", __pyx_getprop_5gauge_13deterministic_4Line_since, __pyx_setprop_5gauge_13deterministic_4Line_since, (char *)0, 0},
  {(char *)"until", __pyx_getprop_5gauge_13deterministic_4Line_until, __pyx_setprop_5gauge_13deterministic_4Line_until, (char *)0, 0},
  {(char *)"value", __pyx_getprop_5gauge_13deterministic_4Line_value, __pyx_setprop_5gauge_13deterministic_4Line_value, (char *)0, 0},
  {(char *)"extra", __pyx_getprop_5gauge_13deterministic_4Line_extra, __pyx_setprop_5gauge_13deterministic_4Line_extra, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5gauge_13deterministic_Line = {
  PyVarObject_HEAD_INIT(0, 0)
  "gauge.deterministic.Line", /*tp_name*/
  sizeof(struct __pyx_obj_5gauge_13deterministic_Line), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5gauge_13deterministic_Line, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_5gauge_13deterministic_4Line_13__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "An abstract class to represent lines between 2 times which start from\n    `value`.  Subclasses should describe where lines end.\n\n    .. note::\n\n       Each subclass must implement :meth:`_get`, :meth:`_earlier`,\n       :meth:`_later`, and :attr:`velocity` property.\n\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5gauge_13deterministic_Line, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5gauge_13deterministic_Line, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5gauge_13deterministic_Line, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_5gauge_13deterministic_Boundary __pyx_vtable_5gauge_13deterministic_Boundary;

static PyObject *__pyx_tp_new_5gauge_13deterministic_Boundary(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_5gauge_13deterministic_Boundary *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_5gauge_13deterministic_Boundary *)o);
  p->__pyx_vtab = __pyx_vtabptr_5gauge_13deterministic_Boundary;
  p->line = ((struct __pyx_obj_5gauge_13deterministic_Line *)Py_None); Py_INCREF(Py_None);
  p->lines_iter = Py_None; Py_INCREF(Py_None);
  p->cmp = Py_None; Py_INCREF(Py_None);
  p->best = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_5gauge_13deterministic_Boundary(PyObject *o) {
  struct __pyx_obj_5gauge_13deterministic_Boundary *p = (struct __pyx_obj_5gauge_13deterministic_Boundary *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->line);
  Py_CLEAR(p->lines_iter);
  Py_CLEAR(p->cmp);
  Py_CLEAR(p->best);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_5gauge_13deterministic_Boundary(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5gauge_13deterministic_Boundary *p = (struct __pyx_obj_5gauge_13deterministic_Boundary *)o;
  if (p->line) {
    e = (*v)(((PyObject*)p->line), a); if (e) return e;
  }
  if (p->lines_iter) {
    e = (*v)(p->lines_iter, a); if (e) return e;
  }
  if (p->cmp) {
    e = (*v)(p->cmp, a); if (e) return e;
  }
  if (p->best) {
    e = (*v)(p->best, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5gauge_13deterministic_Boundary(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5gauge_13deterministic_Boundary *p = (struct __pyx_obj_5gauge_13deterministic_Boundary *)o;
  tmp = ((PyObject*)p->line);
  p->line = ((struct __pyx_obj_5gauge_13deterministic_Line *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->lines_iter);
  p->lines_iter = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->cmp);
  p->cmp = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->best);
  p->best = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_5gauge_13deterministic_8Boundary_line(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5gauge_13deterministic_8Boundary_4line_1__get__(o);
}

static int __pyx_setprop_5gauge_13deterministic_8Boundary_line(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5gauge_13deterministic_8Boundary_4line_3__set__(o, v);
  }
  else {
    return __pyx_pw_5gauge_13deterministic_8Boundary_4line_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5gauge_13deterministic_8Boundary_lines_iter(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5gauge_13deterministic_8Boundary_10lines_iter_1__get__(o);
}

static int __pyx_setprop_5gauge_13deterministic_8Boundary_lines_iter(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5gauge_13deterministic_8Boundary_10lines_iter_3__set__(o, v);
  }
  else {
    return __pyx_pw_5gauge_13deterministic_8Boundary_10lines_iter_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5gauge_13deterministic_8Boundary_cmp(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5gauge_13deterministic_8Boundary_3cmp_1__get__(o);
}

static int __pyx_setprop_5gauge_13deterministic_8Boundary_cmp(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5gauge_13deterministic_8Boundary_3cmp_3__set__(o, v);
  }
  else {
    return __pyx_pw_5gauge_13deterministic_8Boundary_3cmp_5__del__(o);
  }
}

static PyObject *__pyx_getprop_5gauge_13deterministic_8Boundary_best(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_5gauge_13deterministic_8Boundary_4best_1__get__(o);
}

static int __pyx_setprop_5gauge_13deterministic_8Boundary_best(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_5gauge_13deterministic_8Boundary_4best_3__set__(o, v);
  }
  else {
    return __pyx_pw_5gauge_13deterministic_8Boundary_4best_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_5gauge_13deterministic_Boundary[] = {
  {"walk", (PyCFunction)__pyx_pw_5gauge_13deterministic_8Boundary_3walk, METH_NOARGS, __pyx_doc_5gauge_13deterministic_8Boundary_2walk},
  {"cmp_eq", (PyCFunction)__pyx_pw_5gauge_13deterministic_8Boundary_5cmp_eq, METH_VARARGS|METH_KEYWORDS, 0},
  {"cmp_inv", (PyCFunction)__pyx_pw_5gauge_13deterministic_8Boundary_7cmp_inv, METH_VARARGS|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_5gauge_13deterministic_Boundary[] = {
  {(char *)"line", __pyx_getprop_5gauge_13deterministic_8Boundary_line, __pyx_setprop_5gauge_13deterministic_8Boundary_line, (char *)0, 0},
  {(char *)"lines_iter", __pyx_getprop_5gauge_13deterministic_8Boundary_lines_iter, __pyx_setprop_5gauge_13deterministic_8Boundary_lines_iter, (char *)0, 0},
  {(char *)"cmp", __pyx_getprop_5gauge_13deterministic_8Boundary_cmp, __pyx_setprop_5gauge_13deterministic_8Boundary_cmp, (char *)0, 0},
  {(char *)"best", __pyx_getprop_5gauge_13deterministic_8Boundary_best, __pyx_setprop_5gauge_13deterministic_8Boundary_best, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_5gauge_13deterministic_Boundary = {
  PyVarObject_HEAD_INIT(0, 0)
  "gauge.deterministic.Boundary", /*tp_name*/
  sizeof(struct __pyx_obj_5gauge_13deterministic_Boundary), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5gauge_13deterministic_Boundary, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_5gauge_13deterministic_8Boundary_9__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5gauge_13deterministic_Boundary, /*tp_traverse*/
  __pyx_tp_clear_5gauge_13deterministic_Boundary, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_5gauge_13deterministic_Boundary, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_5gauge_13deterministic_Boundary, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_5gauge_13deterministic_8Boundary_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5gauge_13deterministic_Boundary, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ *__pyx_freelist_5gauge_13deterministic___pyx_scope_struct____init__[8];
static int __pyx_freecount_5gauge_13deterministic___pyx_scope_struct____init__ = 0;

static PyObject *__pyx_tp_new_5gauge_13deterministic___pyx_scope_struct____init__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_5gauge_13deterministic___pyx_scope_struct____init__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__)))) {
    o = (PyObject*)__pyx_freelist_5gauge_13deterministic___pyx_scope_struct____init__[--__pyx_freecount_5gauge_13deterministic___pyx_scope_struct____init__];
    memset(o, 0, sizeof(struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5gauge_13deterministic___pyx_scope_struct____init__(PyObject *o) {
  struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ *p = (struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_bound);
  Py_CLEAR(p->__pyx_v_velocities);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_5gauge_13deterministic___pyx_scope_struct____init__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__)))) {
    __pyx_freelist_5gauge_13deterministic___pyx_scope_struct____init__[__pyx_freecount_5gauge_13deterministic___pyx_scope_struct____init__++] = ((struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_5gauge_13deterministic___pyx_scope_struct____init__(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ *p = (struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ *)o;
  if (p->__pyx_v_bound) {
    e = (*v)(((PyObject*)p->__pyx_v_bound), a); if (e) return e;
  }
  if (p->__pyx_v_velocities) {
    e = (*v)(p->__pyx_v_velocities, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5gauge_13deterministic___pyx_scope_struct____init__(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ *p = (struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ *)o;
  tmp = ((PyObject*)p->__pyx_v_bound);
  p->__pyx_v_bound = ((struct __pyx_obj_5gauge_13deterministic_Boundary *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_velocities);
  p->__pyx_v_velocities = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_5gauge_13deterministic___pyx_scope_struct____init__ = {
  PyVarObject_HEAD_INIT(0, 0)
  "gauge.deterministic.__pyx_scope_struct____init__", /*tp_name*/
  sizeof(struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5gauge_13deterministic___pyx_scope_struct____init__, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5gauge_13deterministic___pyx_scope_struct____init__, /*tp_traverse*/
  __pyx_tp_clear_5gauge_13deterministic___pyx_scope_struct____init__, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5gauge_13deterministic___pyx_scope_struct____init__, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr *__pyx_freelist_5gauge_13deterministic___pyx_scope_struct_1_genexpr[8];
static int __pyx_freecount_5gauge_13deterministic___pyx_scope_struct_1_genexpr = 0;

static PyObject *__pyx_tp_new_5gauge_13deterministic___pyx_scope_struct_1_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_5gauge_13deterministic___pyx_scope_struct_1_genexpr > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr)))) {
    o = (PyObject*)__pyx_freelist_5gauge_13deterministic___pyx_scope_struct_1_genexpr[--__pyx_freecount_5gauge_13deterministic___pyx_scope_struct_1_genexpr];
    memset(o, 0, sizeof(struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_5gauge_13deterministic___pyx_scope_struct_1_genexpr(PyObject *o) {
  struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_outer_scope);
  Py_CLEAR(p->__pyx_v_v);
  Py_CLEAR(p->__pyx_t_0);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_5gauge_13deterministic___pyx_scope_struct_1_genexpr < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr)))) {
    __pyx_freelist_5gauge_13deterministic___pyx_scope_struct_1_genexpr[__pyx_freecount_5gauge_13deterministic___pyx_scope_struct_1_genexpr++] = ((struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_5gauge_13deterministic___pyx_scope_struct_1_genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr *)o;
  if (p->__pyx_outer_scope) {
    e = (*v)(((PyObject*)p->__pyx_outer_scope), a); if (e) return e;
  }
  if (p->__pyx_v_v) {
    e = (*v)(p->__pyx_v_v, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_5gauge_13deterministic___pyx_scope_struct_1_genexpr(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr *p = (struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr *)o;
  tmp = ((PyObject*)p->__pyx_outer_scope);
  p->__pyx_outer_scope = ((struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct____init__ *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_v);
  p->__pyx_v_v = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_t_0);
  p->__pyx_t_0 = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyTypeObject __pyx_type_5gauge_13deterministic___pyx_scope_struct_1_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "gauge.deterministic.__pyx_scope_struct_1_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_5gauge_13deterministic___pyx_scope_struct_1_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_5gauge_13deterministic___pyx_scope_struct_1_genexpr, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_5gauge_13deterministic___pyx_scope_struct_1_genexpr, /*tp_traverse*/
  __pyx_tp_clear_5gauge_13deterministic___pyx_scope_struct_1_genexpr, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_5gauge_13deterministic___pyx_scope_struct_1_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {"Horizon", (PyCFunction)__pyx_pw_5gauge_13deterministic_1Horizon, METH_VARARGS|METH_KEYWORDS, 0},
  {"Ray", (PyCFunction)__pyx_pw_5gauge_13deterministic_3Ray, METH_VARARGS|METH_KEYWORDS, 0},
  {"Segment", (PyCFunction)__pyx_pw_5gauge_13deterministic_5Segment, METH_VARARGS|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "deterministic",
    __pyx_k_gauge_deterministic_Determining, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_0_1_for_2_2f_3_2f, __pyx_k_0_1_for_2_2f_3_2f, sizeof(__pyx_k_0_1_for_2_2f_3_2f), 0, 0, 1, 0},
  {&__pyx_kp_s_0_line_1_cmp_2, __pyx_k_0_line_1_cmp_2, sizeof(__pyx_k_0_line_1_cmp_2), 0, 0, 1, 0},
  {&__pyx_n_s_AssertionError, __pyx_k_AssertionError, sizeof(__pyx_k_AssertionError), 0, 0, 1, 1},
  {&__pyx_n_s_Boundary, __pyx_k_Boundary, sizeof(__pyx_k_Boundary), 0, 0, 1, 1},
  {&__pyx_n_s_Determination, __pyx_k_Determination, sizeof(__pyx_k_Determination), 0, 0, 1, 1},
  {&__pyx_n_s_HORIZON, __pyx_k_HORIZON, sizeof(__pyx_k_HORIZON), 0, 0, 1, 1},
  {&__pyx_kp_s_HORIZON_0_2f, __pyx_k_HORIZON_0_2f, sizeof(__pyx_k_HORIZON_0_2f), 0, 0, 1, 0},
  {&__pyx_n_s_Horizon, __pyx_k_Horizon, sizeof(__pyx_k_Horizon), 0, 0, 1, 1},
  {&__pyx_n_s_Line, __pyx_k_Line, sizeof(__pyx_k_Line), 0, 0, 1, 1},
  {&__pyx_n_s_RAY, __pyx_k_RAY, sizeof(__pyx_k_RAY), 0, 0, 1, 1},
  {&__pyx_kp_s_RAY_0_2f_1_2f_s, __pyx_k_RAY_0_2f_1_2f_s, sizeof(__pyx_k_RAY_0_2f_1_2f_s), 0, 0, 1, 0},
  {&__pyx_n_s_Ray, __pyx_k_Ray, sizeof(__pyx_k_Ray), 0, 0, 1, 1},
  {&__pyx_n_s_SEGMENT, __pyx_k_SEGMENT, sizeof(__pyx_k_SEGMENT), 0, 0, 1, 1},
  {&__pyx_kp_s_SEGMENT_0_2f_1_2f, __pyx_k_SEGMENT_0_2f_1_2f, sizeof(__pyx_k_SEGMENT_0_2f_1_2f), 0, 0, 1, 0},
  {&__pyx_n_s_Segment, __pyx_k_Segment, sizeof(__pyx_k_Segment), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_all, __pyx_k_all, sizeof(__pyx_k_all), 0, 0, 1, 1},
  {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_s_cmp, __pyx_k_cmp, sizeof(__pyx_k_cmp), 0, 0, 1, 1},
  {&__pyx_n_s_cmp_eq, __pyx_k_cmp_eq, sizeof(__pyx_k_cmp_eq), 0, 0, 1, 1},
  {&__pyx_n_s_cmp_inv, __pyx_k_cmp_inv, sizeof(__pyx_k_cmp_inv), 0, 0, 1, 1},
  {&__pyx_n_s_determination, __pyx_k_determination, sizeof(__pyx_k_determination), 0, 0, 1, 1},
  {&__pyx_n_s_extra, __pyx_k_extra, sizeof(__pyx_k_extra), 0, 0, 1, 1},
  {&__pyx_n_s_final, __pyx_k_final, sizeof(__pyx_k_final), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_gauge, __pyx_k_gauge, sizeof(__pyx_k_gauge), 0, 0, 1, 1},
  {&__pyx_n_s_gauge_deterministic, __pyx_k_gauge_deterministic, sizeof(__pyx_k_gauge_deterministic), 0, 0, 1, 1},
  {&__pyx_n_s_genexpr, __pyx_k_genexpr, sizeof(__pyx_k_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_gt, __pyx_k_gt, sizeof(__pyx_k_gt), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_init___locals_genexpr, __pyx_k_init___locals_genexpr, sizeof(__pyx_k_init___locals_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_intercept, __pyx_k_intercept, sizeof(__pyx_k_intercept), 0, 0, 1, 1},
  {&__pyx_kp_s_intersection_not_available, __pyx_k_intersection_not_available, sizeof(__pyx_k_intersection_not_available), 0, 0, 1, 0},
  {&__pyx_n_s_isinf, __pyx_k_isinf, sizeof(__pyx_k_isinf), 0, 0, 1, 1},
  {&__pyx_n_s_lines, __pyx_k_lines, sizeof(__pyx_k_lines), 0, 0, 1, 1},
  {&__pyx_n_s_lt, __pyx_k_lt, sizeof(__pyx_k_lt), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_math, __pyx_k_math, sizeof(__pyx_k_math), 0, 0, 1, 1},
  {&__pyx_n_s_max, __pyx_k_max, sizeof(__pyx_k_max), 0, 0, 1, 1},
  {&__pyx_n_s_min, __pyx_k_min, sizeof(__pyx_k_min), 0, 0, 1, 1},
  {&__pyx_n_s_operator, __pyx_k_operator, sizeof(__pyx_k_operator), 0, 0, 1, 1},
  {&__pyx_kp_s_out_of_the_time_range_0_2f_1_2f, __pyx_k_out_of_the_time_range_0_2f_1_2f, sizeof(__pyx_k_out_of_the_time_range_0_2f_1_2f), 0, 0, 1, 0},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_remove, __pyx_k_remove, sizeof(__pyx_k_remove), 0, 0, 1, 1},
  {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {&__pyx_n_s_since, __pyx_k_since, sizeof(__pyx_k_since), 0, 0, 1, 1},
  {&__pyx_n_s_sum, __pyx_k_sum, sizeof(__pyx_k_sum), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {&__pyx_n_s_type, __pyx_k_type, sizeof(__pyx_k_type), 0, 0, 1, 1},
  {&__pyx_kp_s_unexpected_failure, __pyx_k_unexpected_failure, sizeof(__pyx_k_unexpected_failure), 0, 0, 1, 0},
  {&__pyx_n_s_until, __pyx_k_until, sizeof(__pyx_k_until), 0, 0, 1, 1},
  {&__pyx_n_s_value, __pyx_k_value, sizeof(__pyx_k_value), 0, 0, 1, 1},
  {&__pyx_n_s_velocity, __pyx_k_velocity, sizeof(__pyx_k_velocity), 0, 0, 1, 1},
  {&__pyx_n_s_walk, __pyx_k_walk, sizeof(__pyx_k_walk), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_n_s_y, __pyx_k_y, sizeof(__pyx_k_y), 0, 0, 1, 1},
  {&__pyx_n_s_zip, __pyx_k_zip, sizeof(__pyx_k_zip), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_zip = __Pyx_GetBuiltinName(__pyx_n_s_zip); if (!__pyx_builtin_zip) __PYX_ERR(0, 52, __pyx_L1_error)
  __pyx_builtin_sum = __Pyx_GetBuiltinName(__pyx_n_s_sum); if (!__pyx_builtin_sum) __PYX_ERR(0, 163, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 306, __pyx_L1_error)
  __pyx_builtin_AssertionError = __Pyx_GetBuiltinName(__pyx_n_s_AssertionError); if (!__pyx_builtin_AssertionError) __PYX_ERR(0, 361, __pyx_L1_error)
  __pyx_builtin_min = __Pyx_GetBuiltinName(__pyx_n_s_min); if (!__pyx_builtin_min) __PYX_ERR(0, 462, __pyx_L1_error)
  __pyx_builtin_max = __Pyx_GetBuiltinName(__pyx_n_s_max); if (!__pyx_builtin_max) __PYX_ERR(0, 462, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "gauge/deterministic.pyx":52
 *         line = Line(LN_HORIZON, gauge._base_time, first[TIME], first[VALUE])
 *         lines.append(line)
 *     zipped_determination = zip(determination[:-1], determination[1:])             # <<<<<<<<<<<<<<
 *     for (time1, value1), (time2, value2) in zipped_determination:
 *         line = Line(LN_SEGMENT, time1, time2, value1, value2)
 */
  __pyx_slice_ = PySlice_New(Py_None, __pyx_int_neg_1, Py_None); if (unlikely(!__pyx_slice_)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice_);
  __Pyx_GIVEREF(__pyx_slice_);
  __pyx_slice__2 = PySlice_New(__pyx_int_1, Py_None, Py_None); if (unlikely(!__pyx_slice__2)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__2);
  __Pyx_GIVEREF(__pyx_slice__2);

  /* "gauge/deterministic.pyx":306
 *         ok, intersection = self._intersect(line)
 *         if not ok:
 *             raise ValueError('intersection not available')             # <<<<<<<<<<<<<<
 *         return intersection
 * 
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_intersection_not_available); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "gauge/deterministic.pyx":361
 *         ok, value = self._guess(at)
 *         if not ok:
 *             raise AssertionError('unexpected failure')             # <<<<<<<<<<<<<<
 *         return value
 * 
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_unexpected_failure); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 361, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_3 = PyInt_FromLong(3); if (unlikely(!__pyx_int_3)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_neg_1 = PyInt_FromLong(-1); if (unlikely(!__pyx_int_neg_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initdeterministic(void); /*proto*/
PyMODINIT_FUNC initdeterministic(void)
#else
PyMODINIT_FUNC PyInit_deterministic(void); /*proto*/
PyMODINIT_FUNC PyInit_deterministic(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_deterministic(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("deterministic", __pyx_methods, __pyx_k_gauge_deterministic_Determining, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_gauge__deterministic) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "gauge.deterministic")) {
      if (unlikely(PyDict_SetItemString(modules, "gauge.deterministic", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_vtabptr_5gauge_13deterministic_Determination = &__pyx_vtable_5gauge_13deterministic_Determination;
  __pyx_vtable_5gauge_13deterministic_Determination._determine = (void (*)(struct __pyx_obj_5gauge_13deterministic_Determination *, double, double, struct __pyx_opt_args_5gauge_13deterministic_13Determination__determine *__pyx_optional_args))__pyx_f_5gauge_13deterministic_13Determination__determine;
  __pyx_type_5gauge_13deterministic_Determination.tp_base = (&PyList_Type);
  if (PyType_Ready(&__pyx_type_5gauge_13deterministic_Determination) < 0) __PYX_ERR(0, 61, __pyx_L1_error)
  __pyx_type_5gauge_13deterministic_Determination.tp_print = 0;
  #if CYTHON_COMPILING_IN_CPYTHON
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)&__pyx_type_5gauge_13deterministic_Determination, "__init__"); if (unlikely(!wrapper)) __PYX_ERR(0, 61, __pyx_L1_error)
    if (Py_TYPE(wrapper) == &PyWrapperDescr_Type) {
      __pyx_wrapperbase_5gauge_13deterministic_13Determination___init__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_5gauge_13deterministic_13Determination___init__.doc = __pyx_doc_5gauge_13deterministic_13Determination___init__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_5gauge_13deterministic_13Determination___init__;
    }
  }
  #endif
  if (__Pyx_SetVtable(__pyx_type_5gauge_13deterministic_Determination.tp_dict, __pyx_vtabptr_5gauge_13deterministic_Determination) < 0) __PYX_ERR(0, 61, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Determination", (PyObject *)&__pyx_type_5gauge_13deterministic_Determination) < 0) __PYX_ERR(0, 61, __pyx_L1_error)
  __pyx_ptype_5gauge_13deterministic_Determination = &__pyx_type_5gauge_13deterministic_Determination;
  __pyx_vtabptr_5gauge_13deterministic_Line = &__pyx_vtable_5gauge_13deterministic_Line;
  __pyx_vtable_5gauge_13deterministic_Line._intersect = (__pyx_ctuple_int__and___dunderpyx_ctuple_double__dunderand_double (*)(struct __pyx_obj_5gauge_13deterministic_Line *, struct __pyx_obj_5gauge_13deterministic_Line *))__pyx_f_5gauge_13deterministic_4Line__intersect;
  __pyx_vtable_5gauge_13deterministic_Line.intercept = (double (*)(struct __pyx_obj_5gauge_13deterministic_Line *, int __pyx_skip_dispatch))__pyx_f_5gauge_13deterministic_4Line_intercept;
  __pyx_vtable_5gauge_13deterministic_Line._get = (__pyx_ctuple_int__and_double (*)(struct __pyx_obj_5gauge_13deterministic_Line *, double))__pyx_f_5gauge_13deterministic_4Line__get;
  __pyx_vtable_5gauge_13deterministic_Line._guess = (__pyx_ctuple_int__and_double (*)(struct __pyx_obj_5gauge_13deterministic_Line *, double))__pyx_f_5gauge_13deterministic_4Line__guess;
  __pyx_vtable_5gauge_13deterministic_Line.velocity = (double (*)(struct __pyx_obj_5gauge_13deterministic_Line *, int __pyx_skip_dispatch))__pyx_f_5gauge_13deterministic_4Line_velocity;
  __pyx_vtable_5gauge_13deterministic_Line._get_horizon = (__pyx_ctuple_int__and_double (*)(struct __pyx_obj_5gauge_13deterministic_Line *, double))__pyx_f_5gauge_13deterministic_4Line__get_horizon;
  __pyx_vtable_5gauge_13deterministic_Line._earlier_horizon = (__pyx_ctuple_int__and_double (*)(struct __pyx_obj_5gauge_13deterministic_Line *, double))__pyx_f_5gauge_13deterministic_4Line__earlier_horizon;
  __pyx_vtable_5gauge_13deterministic_Line._later_horizon = (__pyx_ctuple_int__and_double (*)(struct __pyx_obj_5gauge_13deterministic_Line *, double))__pyx_f_5gauge_13deterministic_4Line__later_horizon;
  __pyx_vtable_5gauge_13deterministic_Line._velocity_horizon = (double (*)(struct __pyx_obj_5gauge_13deterministic_Line *))__pyx_f_5gauge_13deterministic_4Line__velocity_horizon;
  __pyx_vtable_5gauge_13deterministic_Line._get_ray = (__pyx_ctuple_int__and_double (*)(struct __pyx_obj_5gauge_13deterministic_Line *, double))__pyx_f_5gauge_13deterministic_4Line__get_ray;
  __pyx_vtable_5gauge_13deterministic_Line._earlier_ray = (__pyx_ctuple_int__and_double (*)(struct __pyx_obj_5gauge_13deterministic_Line *, double))__pyx_f_5gauge_13deterministic_4Line__earlier_ray;
  __pyx_vtable_5gauge_13deterministic_Line._later_ray = (__pyx_ctuple_int__and_double (*)(struct __pyx_obj_5gauge_13deterministic_Line *, double))__pyx_f_5gauge_13deterministic_4Line__later_ray;
  __pyx_vtable_5gauge_13deterministic_Line._velocity_ray = (double (*)(struct __pyx_obj_5gauge_13deterministic_Line *))__pyx_f_5gauge_13deterministic_4Line__velocity_ray;
  __pyx_vtable_5gauge_13deterministic_Line._get_segment = (__pyx_ctuple_int__and_double (*)(struct __pyx_obj_5gauge_13deterministic_Line *, double))__pyx_f_5gauge_13deterministic_4Line__get_segment;
  __pyx_vtable_5gauge_13deterministic_Line._earlier_segment = (__pyx_ctuple_int__and_double (*)(struct __pyx_obj_5gauge_13deterministic_Line *, double))__pyx_f_5gauge_13deterministic_4Line__earlier_segment;
  __pyx_vtable_5gauge_13deterministic_Line._later_segment = (__pyx_ctuple_int__and_double (*)(struct __pyx_obj_5gauge_13deterministic_Line *, double))__pyx_f_5gauge_13deterministic_4Line__later_segment;
  __pyx_vtable_5gauge_13deterministic_Line._velocity_segment = (double (*)(struct __pyx_obj_5gauge_13deterministic_Line *))__pyx_f_5gauge_13deterministic_4Line__velocity_segment;
  if (PyType_Ready(&__pyx_type_5gauge_13deterministic_Line) < 0) __PYX_ERR(0, 233, __pyx_L1_error)
  __pyx_type_5gauge_13deterministic_Line.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5gauge_13deterministic_Line.tp_dict, __pyx_vtabptr_5gauge_13deterministic_Line) < 0) __PYX_ERR(0, 233, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Line", (PyObject *)&__pyx_type_5gauge_13deterministic_Line) < 0) __PYX_ERR(0, 233, __pyx_L1_error)
  __pyx_ptype_5gauge_13deterministic_Line = &__pyx_type_5gauge_13deterministic_Line;
  __pyx_vtabptr_5gauge_13deterministic_Boundary = &__pyx_vtable_5gauge_13deterministic_Boundary;
  __pyx_vtable_5gauge_13deterministic_Boundary.walk = (PyObject *(*)(struct __pyx_obj_5gauge_13deterministic_Boundary *, int __pyx_skip_dispatch))__pyx_f_5gauge_13deterministic_8Boundary_walk;
  __pyx_vtable_5gauge_13deterministic_Boundary.cmp_eq = (int (*)(struct __pyx_obj_5gauge_13deterministic_Boundary *, double, double, int __pyx_skip_dispatch))__pyx_f_5gauge_13deterministic_8Boundary_cmp_eq;
  __pyx_vtable_5gauge_13deterministic_Boundary.cmp_inv = (int (*)(struct __pyx_obj_5gauge_13deterministic_Boundary *, double, double, int __pyx_skip_dispatch))__pyx_f_5gauge_13deterministic_8Boundary_cmp_inv;
  if (PyType_Ready(&__pyx_type_5gauge_13deterministic_Boundary) < 0) __PYX_ERR(0, 450, __pyx_L1_error)
  __pyx_type_5gauge_13deterministic_Boundary.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_5gauge_13deterministic_Boundary.tp_dict, __pyx_vtabptr_5gauge_13deterministic_Boundary) < 0) __PYX_ERR(0, 450, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Boundary", (PyObject *)&__pyx_type_5gauge_13deterministic_Boundary) < 0) __PYX_ERR(0, 450, __pyx_L1_error)
  __pyx_ptype_5gauge_13deterministic_Boundary = &__pyx_type_5gauge_13deterministic_Boundary;
  if (PyType_Ready(&__pyx_type_5gauge_13deterministic___pyx_scope_struct____init__) < 0) __PYX_ERR(0, 81, __pyx_L1_error)
  __pyx_type_5gauge_13deterministic___pyx_scope_struct____init__.tp_print = 0;
  __pyx_ptype_5gauge_13deterministic___pyx_scope_struct____init__ = &__pyx_type_5gauge_13deterministic___pyx_scope_struct____init__;
  if (PyType_Ready(&__pyx_type_5gauge_13deterministic___pyx_scope_struct_1_genexpr) < 0) __PYX_ERR(0, 168, __pyx_L1_error)
  __pyx_type_5gauge_13deterministic___pyx_scope_struct_1_genexpr.tp_print = 0;
  __pyx_ptype_5gauge_13deterministic___pyx_scope_struct_1_genexpr = &__pyx_type_5gauge_13deterministic___pyx_scope_struct_1_genexpr;
  /*--- Type import code ---*/
  __pyx_ptype_5gauge_4core_Gauge = __Pyx_ImportType("gauge.core", "Gauge", sizeof(struct __pyx_obj_5gauge_4core_Gauge), 1); if (unlikely(!__pyx_ptype_5gauge_4core_Gauge)) __PYX_ERR(2, 7, __pyx_L1_error)
  __pyx_vtabptr_5gauge_4core_Gauge = (struct __pyx_vtabstruct_5gauge_4core_Gauge*)__Pyx_GetVtable(__pyx_ptype_5gauge_4core_Gauge->tp_dict); if (unlikely(!__pyx_vtabptr_5gauge_4core_Gauge)) __PYX_ERR(2, 7, __pyx_L1_error)
  __pyx_ptype_5gauge_4core_Momentum = __Pyx_ImportType("gauge.core", "Momentum", sizeof(struct __pyx_obj_5gauge_4core_Momentum), 1); if (unlikely(!__pyx_ptype_5gauge_4core_Momentum)) __PYX_ERR(2, 39, __pyx_L1_error)
  /*--- Variable import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("gauge.constants"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "EV_NONE", (void **)&__pyx_vp_5gauge_9constants_EV_NONE, "int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "EV_ADD", (void **)&__pyx_vp_5gauge_9constants_EV_ADD, "int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "EV_REMOVE", (void **)&__pyx_vp_5gauge_9constants_EV_REMOVE, "int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "LI_ERROR", (void **)&__pyx_vp_5gauge_9constants_LI_ERROR, "int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "LI_OK", (void **)&__pyx_vp_5gauge_9constants_LI_OK, "int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "LI_ONCE", (void **)&__pyx_vp_5gauge_9constants_LI_ONCE, "int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "LI_CLAMP", (void **)&__pyx_vp_5gauge_9constants_LI_CLAMP, "int") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "INF", (void **)&__pyx_vp_5gauge_9constants_INF, "double") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  /*--- Function import code ---*/
  __pyx_t_2 = __Pyx_ImportModule("gauge.constants"); if (!__pyx_t_2) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_2, "CLASS_NAME", (void (**)(void))&__pyx_f_5gauge_9constants_CLASS_NAME, "PyObject *(PyObject *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "gauge/deterministic.pyx":14
 * from __future__ import absolute_import
 * 
 * import math             # <<<<<<<<<<<<<<
 * import operator
 * 
 */
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_math, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_math, __pyx_t_3) < 0) __PYX_ERR(0, 14, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "gauge/deterministic.pyx":15
 * 
 * import math
 * import operator             # <<<<<<<<<<<<<<
 * 
 * from gauge.constants cimport CLASS_NAME, EV_ADD, EV_REMOVE, INF
 */
  __pyx_t_3 = __Pyx_Import(__pyx_n_s_operator, 0, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_operator, __pyx_t_3) < 0) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "gauge/deterministic.pyx":22
 * 
 * 
 * __all__ = ['Determination', 'Line', 'Horizon', 'Ray', 'Segment', 'Boundary']             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_3 = PyList_New(6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_n_s_Determination);
  __Pyx_GIVEREF(__pyx_n_s_Determination);
  PyList_SET_ITEM(__pyx_t_3, 0, __pyx_n_s_Determination);
  __Pyx_INCREF(__pyx_n_s_Line);
  __Pyx_GIVEREF(__pyx_n_s_Line);
  PyList_SET_ITEM(__pyx_t_3, 1, __pyx_n_s_Line);
  __Pyx_INCREF(__pyx_n_s_Horizon);
  __Pyx_GIVEREF(__pyx_n_s_Horizon);
  PyList_SET_ITEM(__pyx_t_3, 2, __pyx_n_s_Horizon);
  __Pyx_INCREF(__pyx_n_s_Ray);
  __Pyx_GIVEREF(__pyx_n_s_Ray);
  PyList_SET_ITEM(__pyx_t_3, 3, __pyx_n_s_Ray);
  __Pyx_INCREF(__pyx_n_s_Segment);
  __Pyx_GIVEREF(__pyx_n_s_Segment);
  PyList_SET_ITEM(__pyx_t_3, 4, __pyx_n_s_Segment);
  __Pyx_INCREF(__pyx_n_s_Boundary);
  __Pyx_GIVEREF(__pyx_n_s_Boundary);
  PyList_SET_ITEM(__pyx_t_3, 5, __pyx_n_s_Boundary);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_all, __pyx_t_3) < 0) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "gauge/deterministic.pyx":34
 * DEF LN_RAY = 2
 * DEF LN_SEGMENT = 3
 * HORIZON = LN_HORIZON             # <<<<<<<<<<<<<<
 * RAY = LN_RAY
 * SEGMENT = LN_SEGMENT
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_HORIZON, __pyx_int_1) < 0) __PYX_ERR(0, 34, __pyx_L1_error)

  /* "gauge/deterministic.pyx":35
 * DEF LN_SEGMENT = 3
 * HORIZON = LN_HORIZON
 * RAY = LN_RAY             # <<<<<<<<<<<<<<
 * SEGMENT = LN_SEGMENT
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_RAY, __pyx_int_2) < 0) __PYX_ERR(0, 35, __pyx_L1_error)

  /* "gauge/deterministic.pyx":36
 * HORIZON = LN_HORIZON
 * RAY = LN_RAY
 * SEGMENT = LN_SEGMENT             # <<<<<<<<<<<<<<
 * 
 * 
 */
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SEGMENT, __pyx_int_3) < 0) __PYX_ERR(0, 36, __pyx_L1_error)

  /* "gauge/deterministic.pyx":458
 *         public best
 * 
 *     def __init__(self, list lines, cmp=operator.lt):             # <<<<<<<<<<<<<<
 *         assert cmp in [operator.lt, operator.gt]
 *         self.lines_iter = iter(lines)
 */
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_operator); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_lt); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_k__5 = __pyx_t_4;
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_t_4 = 0;

  /* "gauge/deterministic.pyx":1
 * # -*- coding: utf-8 -*-             # <<<<<<<<<<<<<<
 * """
 *    gauge.deterministic
 */
  __pyx_t_4 = PyDict_New(); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_4) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "gauge/deterministic.pxd":27
 * 
 * 
 * cdef inline double SEGMENT_VELOCITY(double time1, double time2,             # <<<<<<<<<<<<<<
 *                                     double value1, double value2):
 *     return (value2 - value1) / (time2 - time1)
 */

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init gauge.deterministic", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init gauge.deterministic");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* GetItemInt */
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (wraparound & unlikely(i < 0)) i += PyList_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (wraparound & unlikely(i < 0)) i += PyTuple_GET_SIZE(o);
    if ((!boundscheck) || likely((0 <= i) & (i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* SliceObject */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetSlice(PyObject* obj,
        Py_ssize_t cstart, Py_ssize_t cstop,
        PyObject** _py_start, PyObject** _py_stop, PyObject** _py_slice,
        int has_cstart, int has_cstop, CYTHON_UNUSED int wraparound) {
#if CYTHON_USE_TYPE_SLOTS
    PyMappingMethods* mp;
#if PY_MAJOR_VERSION < 3
    PySequenceMethods* ms = Py_TYPE(obj)->tp_as_sequence;
    if (likely(ms && ms->sq_slice)) {
        if (!has_cstart) {
            if (_py_start && (*_py_start != Py_None)) {
                cstart = __Pyx_PyIndex_AsSsize_t(*_py_start);
                if ((cstart == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstart = 0;
        }
        if (!has_cstop) {
            if (_py_stop && (*_py_stop != Py_None)) {
                cstop = __Pyx_PyIndex_AsSsize_t(*_py_stop);
                if ((cstop == (Py_ssize_t)-1) && PyErr_Occurred()) goto bad;
            } else
                cstop = PY_SSIZE_T_MAX;
        }
        if (wraparound && unlikely((cstart < 0) | (cstop < 0)) && likely(ms->sq_length)) {
            Py_ssize_t l = ms->sq_length(obj);
            if (likely(l >= 0)) {
                if (cstop < 0) {
                    cstop += l;
                    if (cstop < 0) cstop = 0;
                }
                if (cstart < 0) {
                    cstart += l;
                    if (cstart < 0) cstart = 0;
                }
            } else {
                if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                    goto bad;
                PyErr_Clear();
            }
        }
        return ms->sq_slice(obj, cstart, cstop);
    }
#endif
    mp = Py_TYPE(obj)->tp_as_mapping;
    if (likely(mp && mp->mp_subscript))
#endif
    {
        PyObject* result;
        PyObject *py_slice, *py_start, *py_stop;
        if (_py_slice) {
            py_slice = *_py_slice;
        } else {
            PyObject* owned_start = NULL;
            PyObject* owned_stop = NULL;
            if (_py_start) {
                py_start = *_py_start;
            } else {
                if (has_cstart) {
                    owned_start = py_start = PyInt_FromSsize_t(cstart);
                    if (unlikely(!py_start)) goto bad;
                } else
                    py_start = Py_None;
            }
            if (_py_stop) {
                py_stop = *_py_stop;
            } else {
                if (has_cstop) {
                    owned_stop = py_stop = PyInt_FromSsize_t(cstop);
                    if (unlikely(!py_stop)) {
                        Py_XDECREF(owned_start);
                        goto bad;
                    }
                } else
                    py_stop = Py_None;
            }
            py_slice = PySlice_New(py_start, py_stop, Py_None);
            Py_XDECREF(owned_start);
            Py_XDECREF(owned_stop);
            if (unlikely(!py_slice)) goto bad;
        }
#if CYTHON_USE_TYPE_SLOTS
        result = mp->mp_subscript(obj, py_slice);
#else
        result = PyObject_GetItem(obj, py_slice);
#endif
        if (!_py_slice) {
            Py_DECREF(py_slice);
        }
        return result;
    }
    PyErr_Format(PyExc_TypeError,
        "'%.200s' object is unsliceable", Py_TYPE(obj)->tp_name);
bad:
    return NULL;
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = PyThreadState_GET();
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(exc_type == PyExc_StopIteration) || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs, NULL);
}
#endif  // CYTHON_FAST_PYCCALL

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = PyThreadState_GET();
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif  // CPython < 3.6
#endif  // CYTHON_FAST_PYCALL

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallMethod1 */
  static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) goto done;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *args;
            PyObject *function = PyMethod_GET_FUNCTION(method);
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyCFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            args = PyTuple_New(2);
            if (unlikely(!args)) goto done;
            Py_INCREF(self);
            PyTuple_SET_ITEM(args, 0, self);
            Py_INCREF(arg);
            PyTuple_SET_ITEM(args, 1, arg);
            Py_INCREF(function);
            Py_DECREF(method); method = NULL;
            result = __Pyx_PyObject_Call(function, args, NULL);
            Py_DECREF(args);
            Py_DECREF(function);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallOneArg(method, arg);
done:
    Py_XDECREF(method);
    return result;
}

/* append */
  static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_n_s_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* PyErrFetchRestore */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
  static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* RaiseDoubleKeywords */
  static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
  static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
  static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTest */
  static void __Pyx_RaiseArgumentTypeInvalid(const char* name, PyObject *obj, PyTypeObject *type) {
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
}
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (likely(Py_TYPE(obj) == type)) return 1;
        #if PY_MAJOR_VERSION == 2
        else if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(PyObject_TypeCheck(obj, type))) return 1;
    }
    __Pyx_RaiseArgumentTypeInvalid(name, obj, type);
    return 0;
}

/* None */
  static CYTHON_INLINE void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* GetModuleGlobalName */
  static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* None */
    static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* RaiseException */
    #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyObjectCallNoArg */
      #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* IterNext */
        static CYTHON_INLINE PyObject *__Pyx_PyIter_Next2(PyObject* iterator, PyObject* defval) {
    PyObject* next;
    iternextfunc iternext = Py_TYPE(iterator)->tp_iternext;
#if CYTHON_USE_TYPE_SLOTS
    if (unlikely(!iternext)) {
#else
    if (unlikely(!iternext) || unlikely(!PyIter_Check(iterator))) {
#endif
        PyErr_Format(PyExc_TypeError,
            "%.200s object is not an iterator", Py_TYPE(iterator)->tp_name);
        return NULL;
    }
    next = iternext(iterator);
    if (likely(next))
        return next;
#if CYTHON_USE_TYPE_SLOTS
#if PY_VERSION_HEX >= 0x02070000
    if (unlikely(iternext == &_PyObject_NextNotImplemented))
        return NULL;
#endif
#endif
    if (defval) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
            if (unlikely(exc_type != PyExc_StopIteration) &&
                    !PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))
                return NULL;
            PyErr_Clear();
        }
        Py_INCREF(defval);
        return defval;
    }
    if (!PyErr_Occurred())
        PyErr_SetNone(PyExc_StopIteration);
    return NULL;
}

/* SetVTable */
          static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* GetVTable */
          static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* Import */
          static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* CodeObjectCache */
          static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
          #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
          #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* ToPyCTupleUtility */
          static PyObject* __pyx_convert__to_py___pyx_ctuple_double__and_double(__pyx_ctuple_double__and_double value) {
    PyObject* item = NULL;
    PyObject* result = PyTuple_New(2);
    if (!result) goto bad;
        item = PyFloat_FromDouble(value.f0);
        if (!item) goto bad;
        PyTuple_SET_ITEM(result, 0, item);
        item = PyFloat_FromDouble(value.f1);
        if (!item) goto bad;
        PyTuple_SET_ITEM(result, 1, item);
    return result;
bad:
    Py_XDECREF(item);
    Py_XDECREF(result);
    return NULL;
}

/* CIntToPy */
          static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
          static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntFromPy */
          static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* FetchCommonType */
          static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* SwapException */
          #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* CoroutineBase */
          #include <structmember.h>
#include <frameobject.h>
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
#if 1 || PY_VERSION_HEX < 0x030300B0
static int __Pyx_PyGen_FetchStopIterationValue(PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (Py_TYPE(ev) == (PyTypeObject*)PyExc_StopIteration) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
#endif
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__pyx_CoroutineObject *self) {
    PyObject *exc_type = self->exc_type;
    PyObject *exc_value = self->exc_value;
    PyObject *exc_traceback = self->exc_traceback;
    self->exc_type = NULL;
    self->exc_value = NULL;
    self->exc_traceback = NULL;
    Py_XDECREF(exc_type);
    Py_XDECREF(exc_value);
    Py_XDECREF(exc_traceback);
}
static CYTHON_INLINE
int __Pyx_Coroutine_CheckRunning(__pyx_CoroutineObject *gen) {
    if (unlikely(gen->is_running)) {
        PyErr_SetString(PyExc_ValueError,
                        "generator already executing");
        return 1;
    }
    return 0;
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value) {
    PyObject *retval;
    __Pyx_PyThreadState_declare
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            PyErr_SetString(PyExc_TypeError,
                            "can't send non-None value to a "
                            "just-started generator");
            return NULL;
        }
    }
    if (unlikely(self->resume_label == -1)) {
        PyErr_SetNone(PyExc_StopIteration);
        return NULL;
    }
    __Pyx_PyThreadState_assign
    if (value) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        if (self->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) self->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            Py_XINCREF(__pyx_tstate->frame);
            assert(f->f_back == NULL);
            f->f_back = __pyx_tstate->frame;
        }
#endif
        __Pyx_ExceptionSwap(&self->exc_type, &self->exc_value,
                            &self->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(self);
    }
    self->is_running = 1;
    retval = self->body((PyObject *) self, value);
    self->is_running = 0;
    if (retval) {
        __Pyx_ExceptionSwap(&self->exc_type, &self->exc_value,
                            &self->exc_traceback);
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        if (self->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) self->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            Py_CLEAR(f->f_back);
        }
#endif
    } else {
        __Pyx_Coroutine_ExceptionClear(self);
    }
    return retval;
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(PyObject *retval) {
    if (unlikely(!retval && !PyErr_Occurred())) {
        PyErr_SetNone(PyExc_StopIteration);
    }
    return retval;
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen_FetchStopIterationValue(&val);
    ret = __Pyx_Coroutine_SendEx(gen, val);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(__Pyx_Coroutine_CheckRunning(gen)))
        return NULL;
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = Py_TYPE(yf)->tp_iternext(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value);
    }
    return __Pyx_Coroutine_MethodReturn(retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                PyErr_WriteUnraisable(yf);
            }
            PyErr_Clear();
        } else {
            retval = PyObject_CallFunction(meth, NULL);
            Py_DECREF(meth);
            if (!retval)
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(__Pyx_Coroutine_CheckRunning(gen)))
        return NULL;
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
            ret = Py_TYPE(yf)->tp_iternext(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(__Pyx_Coroutine_CheckRunning(gen)))
        return NULL;
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL);
    if (retval) {
        Py_DECREF(retval);
        PyErr_SetString(PyExc_RuntimeError,
                        "generator ignored GeneratorExit");
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (!raised_exception
        || raised_exception == PyExc_StopIteration
        || raised_exception == PyExc_GeneratorExit
        || PyErr_GivenExceptionMatches(raised_exception, PyExc_GeneratorExit)
        || PyErr_GivenExceptionMatches(raised_exception, PyExc_StopIteration))
    {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *typ;
    PyObject *tb = NULL;
    PyObject *val = NULL;
    PyObject *yf = gen->yieldfrom;
    if (!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb))
        return NULL;
    if (unlikely(__Pyx_Coroutine_CheckRunning(gen)))
        return NULL;
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit)) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(__Pyx_Coroutine_SendEx(gen, NULL));
            goto throw_here;
        }
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Throw(yf, args);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Throw(yf, args);
        } else
        #endif
        {
            PyObject *meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                    gen->is_running = 0;
                    return NULL;
                }
                PyErr_Clear();
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            ret = PyObject_CallObject(meth, args);
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(__Pyx_Coroutine_SendEx(gen, NULL));
}
static int __Pyx_Coroutine_traverse(PyObject *self, visitproc visit, void *arg) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    Py_VISIT(gen->exc_type);
    Py_VISIT(gen->exc_value);
    Py_VISIT(gen->exc_traceback);
    return 0;
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    Py_CLEAR(gen->exc_type);
    Py_CLEAR(gen->exc_value);
    Py_CLEAR(gen->exc_traceback);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label > 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1
        if (PyObject_CallFinalizerFromDealloc(self))
#else
        Py_TYPE(gen)->tp_del(self);
        if (self->ob_refcnt > 0)
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
    __Pyx_Coroutine_clear(self);
    PyObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *res;
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label <= 0)
        return ;
#if PY_VERSION_HEX < 0x030400a1
    assert(self->ob_refcnt == 0);
    self->ob_refcnt = 1;
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
    res = __Pyx_Coroutine_Close(self);
    if (res == NULL)
        PyErr_WriteUnraisable(self);
    else
        Py_DECREF(res);
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if PY_VERSION_HEX < 0x030400a1
    assert(self->ob_refcnt > 0);
    if (--self->ob_refcnt == 0) {
        return;
    }
    {
        Py_ssize_t refcnt = self->ob_refcnt;
        _Py_NewReference(self);
        self->ob_refcnt = refcnt;
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(self->ob_type) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self)
{
    PyObject *name = self->gi_name;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_name;
    Py_INCREF(value);
    self->gi_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self)
{
    PyObject *name = self->gi_qualname;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_qualname;
    Py_INCREF(value);
    self->gi_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (gen == NULL)
        return NULL;
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->exc_type = NULL;
    gen->exc_value = NULL;
    gen->exc_traceback = NULL;
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchModuleWithCoroutine */
              static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchGeneratorABC */
              #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (!abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_VERSION_HEX >= 0x03030000) ? "collections.abc" : "collections");
        if (!module) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_VERSION_HEX >= 0x03030000) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if (0) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
              static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {0, 0, 0, 0, 0}
};
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if PY_VERSION_HEX >= 0x030400a1
    __Pyx_Coroutine_del,
#endif
};
static int __pyx_Generator_init(void) {
    __pyx_GeneratorType_type.tp_getattro = PyObject_GenericGetAttr;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
              static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* ModuleImport */
              #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
              #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* VoidPtrImport */
              #ifndef __PYX_HAVE_RT_ImportVoidPtr
#define __PYX_HAVE_RT_ImportVoidPtr
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* FunctionImport */
              #ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
              static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
  #else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
  #endif
#else
  res = PyNumber_Int(x);
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
